@@ -86,15 +86,15 @@
 (define (test-from-file file)
   (match-let1 (input . results) (parse-infofile file)
     (if-let1 r (ref results 'sha1 #f)
-      (test* #`",file sha1" r (digest-hexify (sha1-digest-string input))))
+      (test* #"~file sha1" r (digest-hexify (sha1-digest-string input))))
     (if-let1 r (ref results 'sha224 #f)
-      (test* #`",file sha224" r (digest-hexify (sha224-digest-string input))))
+      (test* #"~file sha224" r (digest-hexify (sha224-digest-string input))))
     (if-let1 r (ref results 'sha256 #f)
-      (test* #`",file sha256" r (digest-hexify (sha256-digest-string input))))
+      (test* #"~file sha256" r (digest-hexify (sha256-digest-string input))))
     (if-let1 r (ref results 'sha384 #f)
-      (test* #`",file sha384" r (digest-hexify (sha384-digest-string input))))
+      (test* #"~file sha384" r (digest-hexify (sha384-digest-string input))))
     (if-let1 r (ref results 'sha512 #f)
-      (test* #`",file sha512" r (digest-hexify (sha512-digest-string input))))
+      (test* #"~file sha512" r (digest-hexify (sha512-digest-string input))))
     ))
 
 (for-each test-from-file (glob "data/*.info"))
