@@ -1,7 +1,7 @@
 /*
  * net.h - network interface
  *
- *   Copyright (c) 2000-2016  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2000-2018  Shiro Kawai  <shiro@acm.org>
  *
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -90,11 +90,12 @@ const char *inet_ntop(int af, const void *src, char *dst, socklen_t size);
 #define AI_ALL        0x00000100
 #define AI_ADDRCONFIG 0x00000400
 #define AI_V4MAPPED   0x00000800
-#if !defined(__MINGW64_VERSION_MAJOR)
+/* for MinGW32 runtime v5.0 */
+#if defined(__MINGW32__) && !defined(__MINGW64_VERSION_MAJOR) && (__MINGW32_MAJOR_VERSION >= 5)
 void WSAAPI freeaddrinfo(struct addrinfo*);
 int  WSAAPI getaddrinfo(const char*, const char*, const struct addrinfo*, struct addrinfo**);
 int  WSAAPI getnameinfo(const struct sockaddr*, socklen_t, char*, DWORD, char*, DWORD, int);
-#endif /*!defined(__MINGW64_VERSION_MAJOR)*/
+#endif /* defined(__MINGW32__) && !defined(__MINGW64_VERSION_MAJOR) && (__MINGW32_MAJOR_VERSION >= 5) */
 #endif /* HAVE_IPV6 */
 #endif /*GAUCHE_WINDOWS*/
 
@@ -362,8 +363,12 @@ extern ScmObj Scm_GetAddrinfo(const char *nodename,
                               struct addrinfo *hints);
 extern ScmObj Scm_GetNameinfo(ScmSockAddr *addr, int flags);
 
+#ifndef NI_MAXHOST
 #define NI_MAXHOST  1025
+#endif
+#ifndef NI_MAXSERV
 #define NI_MAXSERV    32
+#endif
 
 #endif /* HAVE_IPV6 */
 
