@@ -1,7 +1,7 @@
 ;;;
 ;;; net - network interface
 ;;;
-;;;   Copyright (c) 2000-2014  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2000-2018  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -37,6 +37,7 @@
   (use srfi-1)
   (use gauche.uvector)
   (use gauche.sequence)
+  (use gauche.lazy)
   (export <socket> make-socket
           PF_UNSPEC PF_UNIX PF_INET AF_UNSPEC AF_UNIX AF_INET
           SOCK_STREAM SOCK_DGRAM SOCK_RAW
@@ -88,6 +89,8 @@
  SIOCGIFHWADDR SIOCSIFHWADDR SIOCSIFHWBROADCAST
  SIOCGIFMAP SIOCSIFMAP SIOCADDMULTI SIOCDELMULTI
  SIOGIFTXQLEN SIOSIFTXQLEN SIOCGIFCONF
+ SIOCGIFADDR SIOCSIFADDR SIOCGIFDSTADDR SIOCSIFDSTADDR
+ SIOCGIFBRDADDR SIOCSIFBRDADDR SIOCGIFNETMASK SIOCSIFNETMASK
 
  IFF_UP IFF_BROADCAST IFF_DEBUG IFF_LOOPBACK IFF_POINTTOPOINT
  IFF_RUNNING IFF_NOARP IFF_PROMISC IFF_NOTRAILERS IFF_ALLMULTI
