@@ -1,20 +1,27 @@
 ;;
-;; testing util.sparse
+;; testing data.sparse
 ;;
 
 (use gauche.test)
+(use gauche.generator)
+(use data.random)
+(use util.match)
 (use srfi-1)
 (use srfi-27)
 
-(test-start "util.sparse")
-(use util.sparse)
-(test-module 'util.sparse)
+(test-start "data.sparse")
+(use data.sparse)
+(test-module 'data.sparse)
+(use gauche.collection)
 
-(define (simple-test name obj %ref %set! %exists? key1 key2
+(define (simple-test name obj %ref %set! %exists? %fold key1 key2
                      :optional (val1 'ok) (val2 'okok) (val3 'okokok))
   (test* #"~name basic set!/ref" val1
          (begin (%set! obj (key1) val1)
                 (%ref obj (key1))))
+  (test* #"~name generic set!/ref" val1
+         (begin (set! (~ obj (key1)) val1)
+                (~ obj (key1))))
   (test* #"~name referencing nokey" (test-error)
          (%ref obj (key2)))
   (test* #"~name referencing nokey fallback" 'huh?
@@ -31,6 +38,10 @@
   (test* #"~name generic setter" val1
          (begin (set! (%ref obj (key1)) val1)
                 (%ref obj (key1))))
+  (test* #"~name folder" '(2 #t)
+         (let1 lis (%fold obj acons '())
+           (list (length lis)
+                 (every (^p (eqv? (%ref obj (car p)) (cdr p))) lis))))
   )
 
 (define (const x) (^[] x))
@@ -97,6 +108,11 @@
                                (+ cnt 1)
                                (return `(error ,cnt ,v))))
                    0 (%vals obj)))))
+  (test* #"~name generic iteration" *data-set-size*
+         (fold (^[kv cnt] (if (equal? (%ref obj (car kv)) (cdr kv))
+                            (+ cnt 1)
+                            cnt))
+               0 obj))
   (test* #"~name many copy" (list *data-set-size* #t #t)
          (let* ([new (%copy obj)]
                 [keys (%keys new)])
@@ -115,7 +131,6 @@
                                   (return `(error ,cnt ,kk ,vv ,(%ref obj kk)))
                                   (+ cnt 1))))
                             0)))
-
   (test* #"~name many delete!" '(#t 0)
          (begin
            (hash-table-for-each *data-set*
@@ -136,6 +151,7 @@
 (define (spvec-simple tag)
   (apply simple-test #"sparse-~(or tag \"\")vector" (make-sparse-vector tag)
          sparse-vector-ref sparse-vector-set! sparse-vector-exists?
+         sparse-vector-fold
          (const 0) (const 1)
          (if (memq tag '(f16 f32 f64))
            '(3.0 6.0 9.0)
@@ -173,6 +189,7 @@
 (define (sptab-simple type key1 key2)
   (simple-test #"sparse-table (~type)" (make-sparse-table type)
                sparse-table-ref sparse-table-set! sparse-table-exists?
+               sparse-table-fold
                key1 key2))
 
 (sptab-simple 'eq?     (const 'a) (const 'b))
@@ -227,4 +244,117 @@
            (begin (sparse-table-delete! u '(1 . 0)) (vals u)))
     ))
 
+;; custom comparator
+(let ()
+  (define c (make-comparator #t (^[a b] (= (modulo a 3) (modulo b 3))) #f
+                             (^x (modulo x 3))))
+  (define t (make-sparse-table c))
+
+  (dotimes [x 10] (sparse-table-set! t x (+ x 100)))
+  (test* "custom comparator" '((0 . 109) (1 . 107) (2 . 108))
+         (sort-by (sparse-table-map t cons) car))
+  (test* "sparse-table-comparator" c
+         (sparse-table-comparator t)))
+
+(let ([predefs `((eq? . ,eq-comparator)
+                 (eqv? . ,eqv-comparator)
+                 (equal? . ,equal-comparator)
+                 (string=? . ,string-comparator))])
+  (test* "predefined comparators" predefs
+         (map (^p (cons (car p)
+                        ($ sparse-table-comparator
+                           $ make-sparse-table (car p))))
+              predefs)))
+  
+
+;; Default value behavior
+(let ()
+  (define z (make-sparse-vector #f :default 99))
+  (define y (make-sparse-vector #f :default '(a)))
+
+  (test* "sparse vector with default value" 99
+         (sparse-vector-ref z (expt 2 1000)))
+  (test* "sparse-vector-inc! with default value" '(100 10)
+         (begin
+           (sparse-vector-inc! z 3 1)
+           (sparse-vector-inc! z 5 1 9)
+           (list (sparse-vector-ref z 3)
+                 (sparse-vector-ref z 5))))
+  (test* "sparse-vector-pop! with default value" 'a
+         (sparse-vector-pop! y 0))
+  (test* "sparse-vector-push! with default value" '(b a)
+         (begin
+           (sparse-vector-push! y 1 'b)
+           (sparse-vector-ref y 1)))
+  )
+
+;; sparse matrix----------------------------------------------------
+(test-section "sparse-matrix")
+
+(set! (random-data-seed) 42) ; for consistent result
+
+(let ()
+  (define data
+    (delete-duplicates
+     (generator->list
+      (gmap cons
+            (lists-of 2 (integers$ (if (> (greatest-fixnum) (expt 2 32))
+                                     (expt 2 32)
+                                     (expt 2 16))))
+            (strings-of))
+      1000)
+     (^[a b] (equal? (car a) (car b)))))
+
+  (test* "set! and ref" (list (length data) #f)
+         (let1 mat (make-sparse-matrix)
+           (dolist [d data]
+             (match-let1 ((x y) . v) d
+               (sparse-matrix-set! mat x y v)))
+           (list
+            (sparse-matrix-num-entries mat)
+            (any (^[d]
+                   (match-let1 ((x y) . v) d
+                     (if (equal? (sparse-matrix-ref mat x y) v)
+                       #f
+                       (list x y v (sparse-matrix-ref mat x y)))))
+                 data))))
+
+  (test* "fold" '(() ())
+         (let ([mat (make-sparse-matrix)]
+               [tab (make-hash-table 'equal?)]
+               [bad '()])
+           (dolist [d data]
+             (match-let1 ((x y) . v) d
+               (sparse-matrix-set! mat x y v)
+               (hash-table-put! tab (cons x y) v)))
+           ($ sparse-matrix-for-each mat
+              (^[x y v]
+                (unless (equal? v (hash-table-get tab (cons x y) #f))
+                  (push! bad (list x y (hash-table-get tab (cons x y) #f) v)))
+                (hash-table-delete! tab (cons x y))))
+           (list bad (hash-table->alist tab))))
+
+  (test* "setter, default value" '(a b (z . a) (a . a))
+         (let* ([mat (make-sparse-matrix #f :default 'a)]
+                [a (sparse-matrix-ref mat 100 200)]
+                [_ (set! (sparse-matrix-ref mat 100 200) 'b)]
+                [b (sparse-matrix-ref mat 100 200)]
+                [_ (sparse-matrix-push! mat 200 100 'z)]
+                [za (sparse-matrix-ref mat 200 100)]
+                [_ (sparse-matrix-update! mat 300 400
+                                          (^x (cons x x)))]
+                [aa (sparse-matrix-ref mat 300 400)])
+           (list a b za aa)))
+
+  (test* "uniform" '(#f 3 #t 13)
+         (let* ([mat (make-sparse-matrix 'u8)]
+                [A (sparse-matrix-exists? mat 123 456)]
+                [_ (sparse-matrix-set! mat 123 456 3)]
+                [a (sparse-matrix-ref mat 123 456)]
+                [B (sparse-matrix-exists? mat 123 456)]
+                [_ (sparse-matrix-inc! mat 123 456 10)]
+                [b (sparse-matrix-ref mat 123 456)])
+           (list A a B b)))
+  )
+
 (test-end)
