@@ -15,7 +15,7 @@
 (test-module 'srfi-1)
 
 (test* "xcons" '(a . b) (xcons 'b 'a))
-(test* "cons*" '() (cons*))
+(test* "cons*" (test-error) (apply cons* '())) ; use apply to avoid compile error during inlining
 (test* "cons*" 'o  (cons* 'o))
 (test* "cons*" '(1 2 3 . 4) (cons* 1 2 3 4))
 (test* "make-list" 5 (length (make-list 5)))
