@@ -281,10 +281,14 @@
             (array-set! res (- i a-start-row) (- k b-start-col) tmp)))))))
 
 (define (array-div-left a b)
-  (array-mul (array-inverse b) a))
+  (if-let1 b-1 (array-inverse b)
+    (array-mul b-1 a)
+    (error "Matrix is not regular:" b)))
 
 (define (array-div-right a b)
-  (array-mul a (array-inverse b)))
+  (if-let1 b-1 (array-inverse b)
+    (array-mul a b-1)
+    (error "Matrix is not regular:" b)))
 
 (define (array-expt ar pow)
   (let loop ([a ar] [n pow])
