@@ -73,7 +73,7 @@ GC_INNER ptr_t GC_FindTopOfStack(unsigned long stack_start)
   }
 
 # ifdef DEBUG_THREADS_EXTRA
-    GC_log_printf("FindTopOfStack start at sp = %p\n", frame);
+    GC_log_printf("FindTopOfStack start at sp = %p\n", (void *)frame);
 # endif
   while (frame->savedSP != 0) {
     /* if there are no more stack frames, stop */
@@ -83,11 +83,11 @@ GC_INNER ptr_t GC_FindTopOfStack(unsigned long stack_start)
     /* we do these next two checks after going to the next frame
        because the LR for the first stack frame in the loop
        is not set up on purpose, so we shouldn't check it. */
-    if ((frame->savedLR & ~0x3) == 0 || (frame->savedLR & ~0x3) == ~0x3U)
+    if ((frame->savedLR & ~0x3) == 0 || (frame->savedLR & ~0x3) == ~0x3UL)
       break; /* if the next LR is bogus, stop */
   }
 # ifdef DEBUG_THREADS_EXTRA
-    GC_log_printf("FindTopOfStack finish at sp = %p\n", frame);
+    GC_log_printf("FindTopOfStack finish at sp = %p\n", (void *)frame);
 # endif
   return (ptr_t)frame;
 }
@@ -130,7 +130,8 @@ GC_API void GC_CALL GC_use_threads_discovery(void)
 /* bound and sets *phi to the upper one.                                */
 STATIC ptr_t GC_stack_range_for(ptr_t *phi, thread_act_t thread, GC_thread p,
                                 GC_bool thread_blocked, mach_port_t my_thread,
-                                ptr_t *paltstack_lo, ptr_t *paltstack_hi)
+                                ptr_t *paltstack_lo,
+                                ptr_t *paltstack_hi GC_ATTR_UNUSED)
 {
   ptr_t lo;
   if (thread == my_thread) {
@@ -141,6 +142,9 @@ STATIC ptr_t GC_stack_range_for(ptr_t *phi, thread_act_t thread, GC_thread p,
 #   endif
 
   } else if (thread_blocked) {
+#   if defined(CPPCHECK)
+      if (NULL == p) ABORT("Invalid GC_thread passed to GC_stack_range_for");
+#   endif
     lo = p->stop_info.stack_ptr;
 #   ifndef DARWIN_DONT_PARSE_STACK
       *phi = p->topOfStack;
@@ -168,14 +172,20 @@ STATIC ptr_t GC_stack_range_for(ptr_t *phi, thread_act_t thread, GC_thread p,
         arm_unified_thread_state_t unified_state;
         mach_msg_type_number_t unified_thread_state_count
                                         = ARM_UNIFIED_THREAD_STATE_COUNT;
-
-        kern_result = thread_get_state(thread, ARM_UNIFIED_THREAD_STATE,
+#       if defined(CPPCHECK)
+#         define GC_ARM_UNIFIED_THREAD_STATE 1
+#       else
+#         define GC_ARM_UNIFIED_THREAD_STATE ARM_UNIFIED_THREAD_STATE
+#       endif
+        kern_result = thread_get_state(thread, GC_ARM_UNIFIED_THREAD_STATE,
                                        (natural_t *)&unified_state,
                                        &unified_thread_state_count);
-        if (unified_state.ash.flavor != ARM_THREAD_STATE32) {
-          ABORT("unified_state flavor should be ARM_THREAD_STATE32");
-        }
-	state = unified_state;
+#       if !defined(CPPCHECK)
+          if (unified_state.ash.flavor != ARM_THREAD_STATE32) {
+            ABORT("unified_state flavor should be ARM_THREAD_STATE32");
+          }
+#       endif
+        state = unified_state;
       } else
 #   endif
     /* else */ {
@@ -294,6 +304,8 @@ STATIC ptr_t GC_stack_range_for(ptr_t *phi, thread_act_t thread, GC_thread p,
       GC_push_one(state.THREAD_FLD(fp));
       GC_push_one(state.THREAD_FLD(lr));
 
+#   elif defined(CPPCHECK)
+      lo = NULL;
 #   else
 #     error FIXME for non-x86 || ppc || arm architectures
 #   endif
@@ -303,26 +315,30 @@ STATIC ptr_t GC_stack_range_for(ptr_t *phi, thread_act_t thread, GC_thread p,
     /* p is guaranteed to be non-NULL regardless of GC_query_task_threads. */
     *phi = (p->flags & MAIN_THREAD) != 0 ? GC_stackbottom : p->stack_end;
 # endif
+
+  /* TODO: Determine p and handle altstack if !DARWIN_DONT_PARSE_STACK */
+# ifdef DARWIN_DONT_PARSE_STACK
   if (p->altstack != NULL && (word)p->altstack <= (word)lo
       && (word)lo <= (word)p->altstack + p->altstack_size) {
     *paltstack_lo = lo;
     *paltstack_hi = p->altstack + p->altstack_size;
     lo = p->stack;
     *phi = p->stack + p->stack_size;
-  } else {
+  } else
+# endif
+  /* else */ {
     *paltstack_lo = NULL;
   }
 # ifdef DEBUG_THREADS
     GC_log_printf("Darwin: Stack for thread %p = [%p,%p)\n",
-                  (void *)thread, lo, *phi);
+                  (void *)(word)thread, (void *)lo, (void *)(*phi));
 # endif
   return lo;
 }
 
 GC_INNER void GC_push_all_stacks(void)
 {
-  int i;
-  ptr_t lo, hi, altstack_lo, altstack_hi;
+  ptr_t hi, altstack_lo, altstack_hi;
   task_t my_task = current_task();
   mach_port_t my_thread = mach_thread_self();
   GC_bool found_me = FALSE;
@@ -334,6 +350,7 @@ GC_INNER void GC_push_all_stacks(void)
 
 # ifndef DARWIN_DONT_PARSE_STACK
     if (GC_query_task_threads) {
+      int i;
       kern_return_t kern_result;
       thread_act_array_t act_list = 0;
 
@@ -344,17 +361,15 @@ GC_INNER void GC_push_all_stacks(void)
 
       for (i = 0; i < (int)listcount; i++) {
         thread_act_t thread = act_list[i];
-        lo = GC_stack_range_for(&hi, thread, NULL, FALSE, my_thread,
-                                &altstack_lo, &altstack_hi);
+        ptr_t lo = GC_stack_range_for(&hi, thread, NULL, FALSE, my_thread,
+                                      &altstack_lo, &altstack_hi);
+
         if (lo) {
           GC_ASSERT((word)lo <= (word)hi);
           total_size += hi - lo;
           GC_push_all_stack(lo, hi);
         }
-        if (altstack_lo) {
-          total_size += altstack_hi - altstack_lo;
-          GC_push_all_stack(altstack_lo, altstack_hi);
-        }
+        /* TODO: Handle altstack */
         nthreads++;
         if (thread == my_thread)
           found_me = TRUE;
@@ -366,13 +381,19 @@ GC_INNER void GC_push_all_stacks(void)
     } else
 # endif /* !DARWIN_DONT_PARSE_STACK */
   /* else */ {
+    int i;
+
     for (i = 0; i < (int)listcount; i++) {
       GC_thread p;
+
       for (p = GC_threads[i]; p != NULL; p = p->next)
         if ((p->flags & FINISHED) == 0) {
           thread_act_t thread = (thread_act_t)p->stop_info.mach_thread;
-          lo = GC_stack_range_for(&hi, thread, p, (GC_bool)p->thread_blocked,
-                                  my_thread, &altstack_lo, &altstack_hi);
+          ptr_t lo = GC_stack_range_for(&hi, thread, p,
+                                        (GC_bool)p->thread_blocked,
+                                        my_thread, &altstack_lo,
+                                        &altstack_hi);
+
           if (lo) {
             GC_ASSERT((word)lo <= (word)hi);
             total_size += hi - lo;
@@ -452,7 +473,8 @@ STATIC GC_bool GC_suspend_thread_list(thread_act_array_t act_list, int count,
 #   endif
 
 #   ifdef DEBUG_THREADS
-      GC_log_printf("Attempting to suspend thread %p\n", (void *)thread);
+      GC_log_printf("Attempting to suspend thread %p\n",
+                    (void *)(word)thread);
 #   endif
     /* find the current thread in the old list */
     found = FALSE;
@@ -496,7 +518,8 @@ STATIC GC_bool GC_suspend_thread_list(thread_act_array_t act_list, int count,
       continue;
     }
 #   ifdef DEBUG_THREADS
-      GC_log_printf("Thread state for %p = %d\n", (void *)thread, info.run_state);
+      GC_log_printf("Thread state for %p = %d\n", (void *)(word)thread,
+                    info.run_state);
 #   endif
     if (info.suspend_count != 0) {
       /* thread is already suspended. */
@@ -506,7 +529,7 @@ STATIC GC_bool GC_suspend_thread_list(thread_act_array_t act_list, int count,
     }
 
 #   ifdef DEBUG_THREADS
-      GC_log_printf("Suspending %p\n", (void *)thread);
+      GC_log_printf("Suspending %p\n", (void *)(word)thread);
 #   endif
     kern_result = thread_suspend(thread);
     if (kern_result != KERN_SUCCESS) {
@@ -519,7 +542,7 @@ STATIC GC_bool GC_suspend_thread_list(thread_act_array_t act_list, int count,
     if (!found)
       GC_mach_threads_count++;
     if (GC_on_thread_event)
-      GC_on_thread_event(GC_EVENT_THREAD_SUSPENDED, (void *)thread);
+      GC_on_thread_event(GC_EVENT_THREAD_SUSPENDED, (void *)(word)thread);
   }
   return changed;
 }
@@ -529,13 +552,13 @@ STATIC GC_bool GC_suspend_thread_list(thread_act_array_t act_list, int count,
 /* Caller holds allocation lock.        */
 GC_INNER void GC_stop_world(void)
 {
-  unsigned i;
   task_t my_task = current_task();
   mach_port_t my_thread = mach_thread_self();
   kern_return_t kern_result;
 
 # ifdef DEBUG_THREADS
-    GC_log_printf("Stopping the world from thread %p\n", (void *)my_thread);
+    GC_log_printf("Stopping the world from thread %p\n",
+                  (void *)(word)my_thread);
 # endif
 # ifdef PARALLEL_MARK
     if (GC_parallel) {
@@ -551,6 +574,7 @@ GC_INNER void GC_stop_world(void)
 
   if (GC_query_task_threads) {
 #   ifndef GC_NO_THREADS_DISCOVERY
+      unsigned i;
       GC_bool changed;
       thread_act_array_t act_list, prev_list;
       mach_msg_type_number_t listcount, prevcount;
@@ -598,6 +622,8 @@ GC_INNER void GC_stop_world(void)
 #   endif /* !GC_NO_THREADS_DISCOVERY */
 
   } else {
+    unsigned i;
+
     for (i = 0; i < THREAD_TABLE_SZ; i++) {
       GC_thread p;
 
@@ -610,7 +636,7 @@ GC_INNER void GC_stop_world(void)
             ABORT("thread_suspend failed");
           if (GC_on_thread_event)
             GC_on_thread_event(GC_EVENT_THREAD_SUSPENDED,
-                               (void *)p->stop_info.mach_thread);
+                               (void *)(word)p->stop_info.mach_thread);
         }
       }
     }
@@ -627,7 +653,7 @@ GC_INNER void GC_stop_world(void)
 # endif
 
 # ifdef DEBUG_THREADS
-    GC_log_printf("World stopped from %p\n", (void *)my_thread);
+    GC_log_printf("World stopped from %p\n", (void *)(word)my_thread);
 # endif
   mach_port_deallocate(my_task, my_thread);
 }
@@ -644,7 +670,7 @@ GC_INLINE void GC_thread_resume(thread_act_t thread)
       ABORT("thread_info failed");
 # endif
 # ifdef DEBUG_THREADS
-    GC_log_printf("Resuming thread %p with state %d\n", (void *)thread,
+    GC_log_printf("Resuming thread %p with state %d\n", (void *)(word)thread,
                   info.run_state);
 # endif
   /* Resume the thread */
@@ -652,7 +678,7 @@ GC_INLINE void GC_thread_resume(thread_act_t thread)
   if (kern_result != KERN_SUCCESS)
     ABORT("thread_resume failed");
   if (GC_on_thread_event)
-    GC_on_thread_event(GC_EVENT_THREAD_UNSUSPENDED, (void *)thread);
+    GC_on_thread_event(GC_EVENT_THREAD_UNSUSPENDED, (void *)(word)thread);
 }
 
 /* Caller holds allocation lock, and has held it continuously since     */
@@ -660,7 +686,6 @@ GC_INLINE void GC_thread_resume(thread_act_t thread)
 GC_INNER void GC_start_world(void)
 {
   task_t my_task = current_task();
-  int i;
 # ifdef DEBUG_THREADS
     GC_log_printf("World starting\n");
 # endif
@@ -672,6 +697,7 @@ GC_INNER void GC_start_world(void)
 
   if (GC_query_task_threads) {
 #   ifndef GC_NO_THREADS_DISCOVERY
+      int i;
       int j = GC_mach_threads_count;
       kern_return_t kern_result;
       thread_act_array_t act_list;
@@ -705,7 +731,7 @@ GC_INNER void GC_start_world(void)
           if (GC_mach_threads[j].already_suspended) {
 #           ifdef DEBUG_THREADS
               GC_log_printf("Not resuming already suspended thread %p\n",
-                            (void *)thread);
+                            (void *)(word)thread);
 #           endif
           } else {
             GC_thread_resume(thread);
@@ -719,6 +745,7 @@ GC_INNER void GC_start_world(void)
 #   endif /* !GC_NO_THREADS_DISCOVERY */
 
   } else {
+    int i;
     mach_port_t my_thread = mach_thread_self();
 
     for (i = 0; i < THREAD_TABLE_SZ; i++) {
