@@ -52,18 +52,18 @@ STATIC GC_has_static_roots_func GC_has_static_roots = 0;
 #if (defined(DYNAMIC_LOADING) || defined(MSWIN32) || defined(MSWINCE) \
     || defined(CYGWIN32)) && !defined(PCR)
 
-#if !defined(SOLARISDL) && !defined(IRIX5) && \
-    !defined(MSWIN32) && !defined(MSWINCE) && !defined(CYGWIN32) && \
-    !(defined(ALPHA) && defined(OSF1)) && \
-    !defined(HPUX) && !(defined(LINUX) && defined(__ELF__)) && \
-    !defined(AIX) && !defined(SCO_ELF) && !defined(DGUX) && \
-    !(defined(FREEBSD) && defined(__ELF__)) && \
-    !(defined(OPENBSD) && (defined(__ELF__) || defined(M68K))) && \
-    !(defined(NETBSD) && defined(__ELF__)) && !defined(HURD) && \
-    !defined(DARWIN) && !defined(CYGWIN32)
- --> We only know how to find data segments of dynamic libraries for the
- --> above.  Additional SVR4 variants might not be too
- --> hard to add.
+#if !defined(DARWIN) && !defined(SCO_ELF) && !defined(SOLARISDL) \
+    && !defined(AIX) && !defined(DGUX) && !defined(IRIX5) && !defined(HPUX) \
+    && !defined(CYGWIN32) && !defined(MSWIN32) && !defined(MSWINCE) \
+    && !(defined(ALPHA) && defined(OSF1)) \
+    && !(defined(FREEBSD) && defined(__ELF__)) \
+    && !((defined(LINUX) || defined(NACL)) && defined(__ELF__)) \
+    && !(defined(NETBSD) && defined(__ELF__)) \
+    && !defined(HAIKU) && !defined(HURD) \
+    && !(defined(OPENBSD) && (defined(__ELF__) || defined(M68K))) \
+    && !defined(CPPCHECK)
+# error We only know how to find data segments of dynamic libraries for above.
+# error Additional SVR4 variants might not be too hard to add.
 #endif
 
 #include <stdio.h>
@@ -82,14 +82,15 @@ STATIC GC_has_static_roots_func GC_has_static_roots = 0;
 
 #if defined(OPENBSD)
 # include <sys/param.h>
-# if OpenBSD >= 200519
+# if (OpenBSD >= 200519) && !defined(HAVE_DL_ITERATE_PHDR)
 #   define HAVE_DL_ITERATE_PHDR
 # endif
 #endif /* OPENBSD */
 
 #if defined(SCO_ELF) || defined(DGUX) || defined(HURD) \
     || (defined(__ELF__) && (defined(LINUX) || defined(FREEBSD) \
-                             || defined(NETBSD) || defined(OPENBSD)))
+                             || defined(NACL) || defined(NETBSD) \
+                             || defined(OPENBSD)))
 # include <stddef.h>
 # if !defined(OPENBSD) && !defined(PLATFORM_ANDROID)
     /* OpenBSD does not have elf.h file; link.h below is sufficient.    */
@@ -109,9 +110,9 @@ STATIC GC_has_static_roots_func GC_has_static_roots = 0;
 #     undef EM_ALPHA
 #   endif
 #   include <link.h>
-#   if !defined(GC_DONT_DEFINE_LINK_MAP)
-      /* link_map and r_debug should be defined explicitly,             */
-      /* as only bionic/linker/linker.h defines them but the header     */
+#   if !defined(GC_DONT_DEFINE_LINK_MAP) && !(__ANDROID_API__ >= 21)
+      /* link_map and r_debug are defined in link.h of NDK r10+.        */
+      /* bionic/linker/linker.h defines them too but the header         */
       /* itself is a C++ one starting from Android 4.3.                 */
       struct link_map {
         uintptr_t l_addr;
@@ -179,23 +180,26 @@ GC_FirstDLOpenedLinkMap(void)
         /* at program startup.                                          */
         if( dynStructureAddr == 0 ) {
           void* startupSyms = dlopen(0, RTLD_LAZY);
-          dynStructureAddr = (ElfW(Dyn)*)dlsym(startupSyms, "_DYNAMIC");
+          dynStructureAddr = (ElfW(Dyn)*)(word)dlsym(startupSyms, "_DYNAMIC");
         }
 #   else
         dynStructureAddr = &_DYNAMIC;
 #   endif
 
-    if (dynStructureAddr == 0) {
+    if (0 == COVERT_DATAFLOW(dynStructureAddr)) {
         /* _DYNAMIC symbol not resolved. */
         return(0);
     }
-    if( cachedResult == 0 ) {
+    if (cachedResult == 0) {
         int tag;
         for( dp = ((ElfW(Dyn) *)(&_DYNAMIC)); (tag = dp->d_tag) != 0; dp++ ) {
-            if( tag == DT_DEBUG ) {
-                struct link_map *lm
-                        = ((struct r_debug *)(dp->d_un.d_ptr))->r_map;
-                if( lm != 0 ) cachedResult = lm->l_next; /* might be NULL */
+            if (tag == DT_DEBUG) {
+                struct r_debug *rd = (struct r_debug *)dp->d_un.d_ptr;
+                if (rd != NULL) {
+                    struct link_map *lm = rd->r_map;
+                    if (lm != NULL)
+                        cachedResult = lm->l_next; /* might be NULL */
+                }
                 break;
             }
         }
@@ -213,8 +217,9 @@ GC_FirstDLOpenedLinkMap(void)
 # if defined(SOLARISDL)
 
 /* Add dynamic library data sections to the root set.           */
-# if !defined(PCR) && !defined(GC_SOLARIS_THREADS) && defined(THREADS)
-        --> fix mutual exclusion with dlopen
+# if !defined(PCR) && !defined(GC_SOLARIS_THREADS) && defined(THREADS) \
+     && !defined(CPPCHECK)
+#   error Fix mutual exclusion with dlopen
 # endif
 
 # ifndef USE_PROC_FOR_LIBRARIES
@@ -257,7 +262,8 @@ GC_INNER void GC_register_dynamic_libraries(void)
 
 #if defined(SCO_ELF) || defined(DGUX) || defined(HURD) \
     || (defined(__ELF__) && (defined(LINUX) || defined(FREEBSD) \
-                             || defined(NETBSD) || defined(OPENBSD)))
+                             || defined(NACL) || defined(NETBSD) \
+                             || defined(OPENBSD)))
 
 #ifdef USE_PROC_FOR_LIBRARIES
 
@@ -279,9 +285,10 @@ static void sort_heap_sects(struct HeapSect *base, size_t number_of_elements)
 {
     signed_word n = (signed_word)number_of_elements;
     signed_word nsorted = 1;
-    signed_word i;
 
     while (nsorted < n) {
+      signed_word i;
+
       while (nsorted < n &&
              (word)base[nsorted-1].hs_start < (word)base[nsorted].hs_start)
           ++nsorted;
@@ -307,19 +314,6 @@ STATIC word GC_register_map_entries(char *maps)
     unsigned int maj_dev;
     ptr_t least_ha, greatest_ha;
     unsigned i;
-    ptr_t datastart;
-
-#   ifdef DATASTART_IS_FUNC
-      static ptr_t datastart_cached = (ptr_t)(word)-1;
-
-      /* Evaluate DATASTART only once.  */
-      if (datastart_cached == (ptr_t)(word)-1) {
-        datastart_cached = (ptr_t)(DATASTART);
-      }
-      datastart = datastart_cached;
-#   else
-      datastart = (ptr_t)(DATASTART);
-#   endif
 
     GC_ASSERT(I_HOLD_LOCK());
     sort_heap_sects(GC_our_memory, GC_n_memory);
@@ -423,23 +417,28 @@ GC_INNER GC_bool GC_register_main_static_data(void)
 #if __GLIBC__ > 2 || (__GLIBC__ == 2 && __GLIBC_MINOR__ > 2) \
     || (__GLIBC__ == 2 && __GLIBC_MINOR__ == 2 && defined(DT_CONFIG)) \
     || defined(PLATFORM_ANDROID) /* Are others OK here, too? */
-/* We have the header files for a glibc that includes dl_iterate_phdr.  */
-/* It may still not be available in the library on the target system.   */
-/* Thus we also treat it as a weak symbol.                              */
-# define HAVE_DL_ITERATE_PHDR
+# ifndef HAVE_DL_ITERATE_PHDR
+#   define HAVE_DL_ITERATE_PHDR
+# endif
 # ifdef PLATFORM_ANDROID
     /* Android headers might have no such definition for some targets.  */
     int dl_iterate_phdr(int (*cb)(struct dl_phdr_info *, size_t, void *),
                         void *data);
 # endif
-# pragma weak dl_iterate_phdr
-#endif
+#endif /* __GLIBC__ >= 2 || PLATFORM_ANDROID */
 
-#if (defined(FREEBSD) && __FreeBSD__ >= 7)
+#if (defined(FREEBSD) && __FreeBSD__ >= 7) || defined(__DragonFly__)
   /* On the FreeBSD system, any target system at major version 7 shall   */
-  /* have dl_iterate_phdr; therefore, we need not make it weak as above. */
-# define HAVE_DL_ITERATE_PHDR
+  /* have dl_iterate_phdr; therefore, we need not make it weak as below. */
+# ifndef HAVE_DL_ITERATE_PHDR
+#   define HAVE_DL_ITERATE_PHDR
+# endif
 # define DL_ITERATE_PHDR_STRONG
+#elif defined(HAVE_DL_ITERATE_PHDR)
+  /* We have the header files for a glibc that includes dl_iterate_phdr.*/
+  /* It may still not be available in the library on the target system. */
+  /* Thus we also treat it as a weak symbol.                            */
+# pragma weak dl_iterate_phdr
 #endif
 
 #if defined(HAVE_DL_ITERATE_PHDR)
@@ -464,6 +463,7 @@ GC_INNER GC_bool GC_register_main_static_data(void)
     } load_segs[MAX_LOAD_SEGS];
 
     static int n_load_segs;
+    static GC_bool load_segs_overflow;
 # endif /* PT_GNU_RELRO */
 
 STATIC int GC_register_dynlib_callback(struct dl_phdr_info * info,
@@ -479,75 +479,80 @@ STATIC int GC_register_dynlib_callback(struct dl_phdr_info * info,
     return -1;
 
   p = info->dlpi_phdr;
-  for( i = 0; i < (int)info->dlpi_phnum; i++, p++ ) {
-    switch( p->p_type ) {
+  for (i = 0; i < (int)info->dlpi_phnum; i++, p++) {
+    if (p->p_type == PT_LOAD) {
+      GC_has_static_roots_func callback = GC_has_static_roots;
+      if ((p->p_flags & PF_W) == 0) continue;
+
+      start = (ptr_t)p->p_vaddr + info->dlpi_addr;
+      end = start + p->p_memsz;
+      if (callback != 0 && !callback(info->dlpi_name, start, p->p_memsz))
+        continue;
 #     ifdef PT_GNU_RELRO
-        case PT_GNU_RELRO:
-        /* This entry is known to be constant and will eventually be remapped
-           read-only.  However, the address range covered by this entry is
-           typically a subset of a previously encountered "LOAD" segment, so
-           we need to exclude it.  */
-        {
-            int j;
-
-            start = ((ptr_t)(p->p_vaddr)) + info->dlpi_addr;
-            end = start + p->p_memsz;
-            for (j = n_load_segs; --j >= 0; ) {
-              if ((word)start >= (word)load_segs[j].start
-                  && (word)start < (word)load_segs[j].end) {
-                if (load_segs[j].start2 != 0) {
-                  WARN("More than one GNU_RELRO segment per load seg\n",0);
-                } else {
-                  GC_ASSERT((word)end <= (word)load_segs[j].end);
-                  /* Remove from the existing load segment */
-                  load_segs[j].end2 = load_segs[j].end;
-                  load_segs[j].end = start;
-                  load_segs[j].start2 = end;
-                }
-                break;
-              }
-              if (j == 0) WARN("Failed to find PT_GNU_RELRO segment"
-                               " inside PT_LOAD region", 0);
-            }
+#       if CPP_WORDSZ == 64
+          /* TODO: GC_push_all eventually does the correct          */
+          /* rounding to the next multiple of ALIGNMENT, so, most   */
+          /* probably, we should remove the corresponding assertion */
+          /* check in GC_add_roots_inner along with this code line. */
+          /* start pointer value may require aligning.              */
+          start = (ptr_t)((word)start & ~(word)(sizeof(word) - 1));
+#       endif
+        if (n_load_segs >= MAX_LOAD_SEGS) {
+          if (!load_segs_overflow) {
+            WARN("Too many PT_LOAD segments;"
+                 " registering as roots directly...\n", 0);
+            load_segs_overflow = TRUE;
+          }
+          GC_add_roots_inner(start, end, TRUE);
+        } else {
+          load_segs[n_load_segs].start = start;
+          load_segs[n_load_segs].end = end;
+          load_segs[n_load_segs].start2 = 0;
+          load_segs[n_load_segs].end2 = 0;
+          ++n_load_segs;
         }
-
-        break;
-#     endif
-
-      case PT_LOAD:
-        {
-          GC_has_static_roots_func callback = GC_has_static_roots;
-          if( !(p->p_flags & PF_W) ) break;
-          start = ((char *)(p->p_vaddr)) + info->dlpi_addr;
-          end = start + p->p_memsz;
-
-          if (callback != 0 && !callback(info->dlpi_name, start, p->p_memsz))
-            break;
-#         ifdef PT_GNU_RELRO
-            if (n_load_segs >= MAX_LOAD_SEGS) ABORT("Too many PT_LOAD segs");
-#           if CPP_WORDSZ == 64
-              /* FIXME: GC_push_all eventually does the correct         */
-              /* rounding to the next multiple of ALIGNMENT, so, most   */
-              /* probably, we should remove the corresponding assertion */
-              /* check in GC_add_roots_inner along with this code line. */
-              /* start pointer value may require aligning */
-              start = (ptr_t)((word)start & ~(sizeof(word) - 1));
-#           endif
-            load_segs[n_load_segs].start = start;
-            load_segs[n_load_segs].end = end;
-            load_segs[n_load_segs].start2 = 0;
-            load_segs[n_load_segs].end2 = 0;
-            ++n_load_segs;
-#         else
-            GC_add_roots_inner(start, end, TRUE);
-#         endif /* PT_GNU_RELRO */
-        }
-      break;
-      default:
-        break;
+#     else
+        GC_add_roots_inner(start, end, TRUE);
+#     endif /* !PT_GNU_RELRO */
     }
   }
 
+# ifdef PT_GNU_RELRO
+    p = info->dlpi_phdr;
+    for (i = 0; i < (int)info->dlpi_phnum; i++, p++) {
+      if (p->p_type == PT_GNU_RELRO) {
+        /* This entry is known to be constant and will eventually be    */
+        /* remapped as read-only.  However, the address range covered   */
+        /* by this entry is typically a subset of a previously          */
+        /* encountered "LOAD" segment, so we need to exclude it.        */
+        int j;
+
+        start = (ptr_t)p->p_vaddr + info->dlpi_addr;
+        end = start + p->p_memsz;
+        for (j = n_load_segs; --j >= 0; ) {
+          if ((word)start >= (word)load_segs[j].start
+              && (word)start < (word)load_segs[j].end) {
+            if (load_segs[j].start2 != 0) {
+              WARN("More than one GNU_RELRO segment per load one\n",0);
+            } else {
+              GC_ASSERT((word)end <= (word)load_segs[j].end);
+              /* Remove from the existing load segment */
+              load_segs[j].end2 = load_segs[j].end;
+              load_segs[j].end = start;
+              load_segs[j].start2 = end;
+            }
+            break;
+          }
+          if (0 == j && 0 == GC_has_static_roots)
+            WARN("Failed to find PT_GNU_RELRO segment"
+                 " inside PT_LOAD region\n", 0);
+            /* No warning reported in case of the callback is present   */
+            /* because most likely the segment has been excluded.       */
+        }
+      }
+    }
+# endif
+
   *(int *)ptr = 1;     /* Signal that we were called */
   return 0;
 }
@@ -560,7 +565,7 @@ GC_INNER GC_bool GC_register_main_static_data(void)
     /* zero (otherwise a compiler might issue a warning).               */
     return FALSE;
 # else
-    return (dl_iterate_phdr == 0); /* implicit conversion to function ptr */
+    return 0 == COVERT_DATAFLOW(dl_iterate_phdr);
 # endif
 }
 
@@ -575,6 +580,7 @@ STATIC GC_bool GC_register_dynamic_libraries_dl_iterate_phdr(void)
     {
       static GC_bool excluded_segs = FALSE;
       n_load_segs = 0;
+      load_segs_overflow = FALSE;
       if (!EXPECT(excluded_segs, TRUE)) {
         GC_exclude_static_roots_inner((ptr_t)load_segs,
                                       (ptr_t)load_segs + sizeof(load_segs));
@@ -606,7 +612,7 @@ STATIC GC_bool GC_register_dynamic_libraries_dl_iterate_phdr(void)
 
         /* Evaluate DATASTART only once.  */
         if (datastart_cached == (ptr_t)(word)-1) {
-          datastart_cached = (ptr_t)(DATASTART);
+          datastart_cached = DATASTART;
         }
         datastart = (char *)datastart_cached;
 #     else
@@ -617,19 +623,29 @@ STATIC GC_bool GC_register_dynamic_libraries_dl_iterate_phdr(void)
           static ptr_t dataend_cached = 0;
           /* Evaluate DATAEND only once. */
           if (dataend_cached == 0) {
-            dataend_cached = (ptr_t)(DATAEND);
+            dataend_cached = DATAEND;
           }
           dataend = (char *)dataend_cached;
         }
 #     else
         dataend = DATAEND;
 #     endif
+      if (NULL == *(char * volatile *)&datastart
+          || (word)datastart > (word)dataend)
+        ABORT_ARG2("Wrong DATASTART/END pair",
+                   ": %p .. %p", (void *)datastart, (void *)dataend);
 
       /* dl_iterate_phdr may forget the static data segment in  */
       /* statically linked executables.                         */
       GC_add_roots_inner(datastart, dataend, TRUE);
-#     if defined(DATASTART2)
-        GC_add_roots_inner(DATASTART2, (char *)(DATAEND2), TRUE);
+#     ifdef GC_HAVE_DATAREGION2
+        if ((word)DATASTART2 - 1U >= (word)DATAEND2) {
+                        /* Subtract one to check also for NULL  */
+                        /* without a compiler warning.          */
+          ABORT_ARG2("Wrong DATASTART/END2 pair",
+                     ": %p .. %p", (void *)DATASTART2, (void *)DATAEND2);
+        }
+        GC_add_roots_inner(DATASTART2, DATAEND2, TRUE);
 #     endif
   }
   return TRUE;
@@ -677,25 +693,42 @@ extern ElfW(Dyn) _DYNAMIC[];
 STATIC struct link_map *
 GC_FirstDLOpenedLinkMap(void)
 {
-    ElfW(Dyn) *dp;
     static struct link_map *cachedResult = 0;
 
-    if (0 == (ptr_t)_DYNAMIC) {
+    if (0 == COVERT_DATAFLOW(_DYNAMIC)) {
         /* _DYNAMIC symbol not resolved. */
         return(0);
     }
     if( cachedResult == 0 ) {
 #     if defined(NETBSD) && defined(RTLD_DI_LINKMAP)
+#       if defined(CPPCHECK)
+#         define GC_RTLD_DI_LINKMAP 2
+#       else
+#         define GC_RTLD_DI_LINKMAP RTLD_DI_LINKMAP
+#       endif
         struct link_map *lm = NULL;
-        if (!dlinfo(RTLD_SELF, RTLD_DI_LINKMAP, &lm))
-            cachedResult = lm;
+        if (!dlinfo(RTLD_SELF, GC_RTLD_DI_LINKMAP, &lm) && lm != NULL) {
+            /* Now lm points link_map object of libgc.  Since it    */
+            /* might not be the first dynamically linked object,    */
+            /* try to find it (object next to the main object).     */
+            while (lm->l_prev != NULL) {
+                lm = lm->l_prev;
+            }
+            cachedResult = lm->l_next;
+        }
 #     else
+        ElfW(Dyn) *dp;
         int tag;
+
         for( dp = _DYNAMIC; (tag = dp->d_tag) != 0; dp++ ) {
-            if( tag == DT_DEBUG ) {
-                struct link_map *lm
-                        = ((struct r_debug *)(dp->d_un.d_ptr))->r_map;
-                if( lm != 0 ) cachedResult = lm->l_next; /* might be NULL */
+            if (tag == DT_DEBUG) {
+                struct r_debug *rd = (struct r_debug *)dp->d_un.d_ptr;
+                /* d_ptr could be null if libs are linked statically. */
+                if (rd != NULL) {
+                    struct link_map *lm = rd->r_map;
+                    if (lm != NULL)
+                        cachedResult = lm->l_next; /* might be NULL */
+                }
                 break;
             }
         }
@@ -769,7 +802,7 @@ GC_INNER void GC_register_dynamic_libraries(void)
     char buf[30];
     static prmap_t * addr_map = 0;
     static int current_sz = 0;  /* Number of records currently in addr_map */
-    static int needed_sz;       /* Required size of addr_map            */
+    int needed_sz = 0;          /* Required size of addr_map            */
     int i;
     long flags;
     ptr_t start;
@@ -796,17 +829,19 @@ GC_INNER void GC_register_dynamic_libraries(void)
                    ": fd = %d, errno = %d", fd, errno);
     }
     if (needed_sz >= current_sz) {
+        GC_scratch_recycle_no_gww(addr_map,
+                                  (size_t)current_sz * sizeof(prmap_t));
         current_sz = needed_sz * 2 + 1;
                         /* Expansion, plus room for 0 record */
         addr_map = (prmap_t *)GC_scratch_alloc(
-                                (word)current_sz * sizeof(prmap_t));
+                                (size_t)current_sz * sizeof(prmap_t));
         if (addr_map == NULL)
           ABORT("Insufficient memory for address map");
     }
     if (ioctl(fd, PIOCMAP, addr_map) < 0) {
         ABORT_ARG3("/proc PIOCMAP ioctl failed",
                    ": errcode= %d, needed_sz= %d, addr_map= %p",
-                   errno, needed_sz, addr_map);
+                   errno, needed_sz, (void *)addr_map);
     };
     if (GC_n_heap_sects > 0) {
         heap_end = GC_heap_sects[GC_n_heap_sects-1].hs_start
@@ -906,7 +941,7 @@ GC_INNER void GC_register_dynamic_libraries(void)
 #   else
       char * stack_top
          = (char *)((word)GC_approx_sp() &
-                        ~(GC_sysinfo.dwAllocationGranularity - 1));
+                    ~(word)(GC_sysinfo.dwAllocationGranularity - 1));
 
       if (base == limit) return;
       if ((word)limit > (word)stack_top
@@ -957,7 +992,6 @@ GC_INNER void GC_register_dynamic_libraries(void)
   GC_INNER void GC_register_dynamic_libraries(void)
   {
     MEMORY_BASIC_INFORMATION buf;
-    size_t result;
     DWORD protect;
     LPVOID p;
     char * base;
@@ -968,7 +1002,8 @@ GC_INNER void GC_register_dynamic_libraries(void)
 #   endif
     base = limit = p = GC_sysinfo.lpMinimumApplicationAddress;
     while ((word)p < (word)GC_sysinfo.lpMaximumApplicationAddress) {
-        result = VirtualQuery(p, &buf, sizeof(buf));
+        size_t result = VirtualQuery(p, &buf, sizeof(buf));
+
 #       ifdef MSWINCE
           if (result == 0) {
             /* Page is free; advance to the next possible allocation base */
@@ -1026,29 +1061,18 @@ extern int errno;
 
 GC_INNER void GC_register_dynamic_libraries(void)
 {
-  int status;
-  ldr_process_t mypid;
-
-  /* module */
-    ldr_module_t moduleid = LDR_NULL_MODULE;
-    ldr_module_info_t moduleinfo;
-    size_t moduleinfosize = sizeof(moduleinfo);
-    size_t modulereturnsize;
-
-  /* region */
-    ldr_region_t region;
-    ldr_region_info_t regioninfo;
-    size_t regioninfosize = sizeof(regioninfo);
-    size_t regionreturnsize;
-
-  /* Obtain id of this process */
-    mypid = ldr_my_process();
+  ldr_module_t moduleid = LDR_NULL_MODULE;
+  ldr_process_t mypid = ldr_my_process(); /* obtain id of this process */
 
   /* For each module */
     while (TRUE) {
-
-      /* Get the next (first) module */
-        status = ldr_next_module(mypid, &moduleid);
+      ldr_module_info_t moduleinfo;
+      size_t modulereturnsize;
+      ldr_region_t region;
+      ldr_region_info_t regioninfo;
+      size_t regionreturnsize;
+      int status = ldr_next_module(mypid, &moduleid);
+                                /* Get the next (first) module */
 
       /* Any more modules? */
         if (moduleid == LDR_NULL_MODULE)
@@ -1064,7 +1088,7 @@ GC_INNER void GC_register_dynamic_libraries(void)
 
       /* Get the module information */
         status = ldr_inq_module(mypid, moduleid, &moduleinfo,
-                                moduleinfosize, &modulereturnsize);
+                                sizeof(moduleinfo), &modulereturnsize);
         if (status != 0 )
             ABORT("ldr_inq_module failed");
 
@@ -1084,7 +1108,7 @@ GC_INNER void GC_register_dynamic_libraries(void)
         for (region = 0; region < moduleinfo.lmi_nregion; region++) {
           /* Get the region information */
             status = ldr_inq_region(mypid, moduleid, region, &regioninfo,
-                                    regioninfosize, &regionreturnsize);
+                                    sizeof(regioninfo), &regionreturnsize);
             if (status != 0 )
                 ABORT("ldr_inq_region failed");
 
@@ -1124,15 +1148,13 @@ extern int sys_nerr;
 
 GC_INNER void GC_register_dynamic_libraries(void)
 {
-  int status;
   int index = 1; /* Ordinal position in shared library search list */
-  struct shl_descriptor *shl_desc; /* Shared library info, see dl.h */
 
   /* For each dynamic library loaded */
     while (TRUE) {
-
-      /* Get info about next shared library */
-        status = shl_get(index, &shl_desc);
+      struct shl_descriptor *shl_desc; /* Shared library info, see dl.h */
+      int status = shl_get(index, &shl_desc);
+                                /* Get info about next shared library   */
 
       /* Check if this is the end of the list or if some error occurred */
         if (status != 0) {
@@ -1179,18 +1201,24 @@ GC_INNER void GC_register_dynamic_libraries(void)
 # include <sys/errno.h>
   GC_INNER void GC_register_dynamic_libraries(void)
   {
+      int ldibuflen = 8192;
+
+      for (;;) {
         int len;
-        char *ldibuf;
-        int ldibuflen;
         struct ld_info *ldi;
+#       if defined(CPPCHECK)
+          char ldibuf[ldibuflen];
+#       else
+          char *ldibuf = alloca(ldibuflen);
+#       endif
 
-        ldibuf = alloca(ldibuflen = 8192);
-
-        while ( (len = loadquery(L_GETINFO,ldibuf,ldibuflen)) < 0) {
+        len = loadquery(L_GETINFO, ldibuf, ldibuflen);
+        if (len < 0) {
                 if (errno != ENOMEM) {
                         ABORT("loadquery failed");
                 }
-                ldibuf = alloca(ldibuflen *= 2);
+                ldibuflen *= 2;
+                continue;
         }
 
         ldi = (struct ld_info *)ldibuf;
@@ -1203,6 +1231,8 @@ GC_INNER void GC_register_dynamic_libraries(void)
                                 TRUE);
                 ldi = len ? (struct ld_info *)((char *)ldi + len) : 0;
         }
+        break;
+      }
   }
 #endif /* AIX */
 
@@ -1275,8 +1305,6 @@ STATIC void GC_dyld_image_add(const struct GC_MACH_HEADER *hdr,
   const struct GC_MACH_SECTION *sec;
   const char *name;
   GC_has_static_roots_func callback = GC_has_static_roots;
-  char secnam[16];
-  const char *fmt;
   DCL_LOCK_STATE;
 
   if (GC_no_dls) return;
@@ -1308,9 +1336,12 @@ STATIC void GC_dyld_image_add(const struct GC_MACH_HEADER *hdr,
 
   /* Sections constructed on demand.    */
   for (j = 0; j < sizeof(GC_dyld_add_sect_fmts) / sizeof(char *); j++) {
-    fmt = GC_dyld_add_sect_fmts[j];
+    const char *fmt = GC_dyld_add_sect_fmts[j];
+
     /* Add our manufactured aligned BSS sections.       */
     for (i = 0; i <= L2_MAX_OFILE_ALIGNMENT; i++) {
+      char secnam[16];
+
       (void)snprintf(secnam, sizeof(secnam), fmt, (unsigned)i);
       secnam[sizeof(secnam) - 1] = '\0';
       sec = GC_GETSECTBYNAME(hdr, SEG_DATA, secnam);
@@ -1340,8 +1371,6 @@ STATIC void GC_dyld_image_remove(const struct GC_MACH_HEADER *hdr,
   unsigned long start, end;
   unsigned i, j;
   const struct GC_MACH_SECTION *sec;
-  char secnam[16];
-  const char *fmt;
 
   for (i = 0; i < sizeof(GC_dyld_sections)/sizeof(GC_dyld_sections[0]); i++) {
     sec = GC_GETSECTBYNAME(hdr, GC_dyld_sections[i].seg,
@@ -1361,8 +1390,11 @@ STATIC void GC_dyld_image_remove(const struct GC_MACH_HEADER *hdr,
 
   /* Remove our on-demand sections.     */
   for (j = 0; j < sizeof(GC_dyld_add_sect_fmts) / sizeof(char *); j++) {
-    fmt = GC_dyld_add_sect_fmts[j];
+    const char *fmt = GC_dyld_add_sect_fmts[j];
+
     for (i = 0; i <= L2_MAX_OFILE_ALIGNMENT; i++) {
+      char secnam[16];
+
       (void)snprintf(secnam, sizeof(secnam), fmt, (unsigned)i);
       secnam[sizeof(secnam) - 1] = '\0';
       sec = GC_GETSECTBYNAME(hdr, SEG_DATA, secnam);
@@ -1417,12 +1449,13 @@ GC_INNER void GC_init_dyld(void)
      This WILL properly register already linked libraries and libraries
      linked in the future.
   */
-
-  _dyld_register_func_for_add_image(GC_dyld_image_add);
-  _dyld_register_func_for_remove_image(GC_dyld_image_remove);
-      /* Ignore 2 compiler warnings here: passing argument 1 of       */
-      /* '_dyld_register_func_for_add/remove_image' from incompatible */
-      /* pointer type.                                                */
+  _dyld_register_func_for_add_image(
+        (void (*)(const struct mach_header*, intptr_t))GC_dyld_image_add);
+  _dyld_register_func_for_remove_image(
+        (void (*)(const struct mach_header*, intptr_t))GC_dyld_image_remove);
+                        /* Structure mach_header64 has the same fields  */
+                        /* as mach_header except for the reserved one   */
+                        /* at the end, so these casts are OK.           */
 
   /* Set this early to avoid reentrancy issues. */
   initialized = TRUE;
@@ -1458,6 +1491,21 @@ GC_INNER GC_bool GC_register_main_static_data(void)
 
 #endif /* DARWIN */
 
+#if defined(HAIKU)
+# include <kernel/image.h>
+
+  GC_INNER void GC_register_dynamic_libraries(void)
+  {
+    image_info info;
+    int32 cookie = 0;
+
+    while (get_next_image_info(0, &cookie, &info) == B_OK) {
+      ptr_t data = (ptr_t)info.data;
+      GC_add_roots_inner(data, data + info.data_size, TRUE);
+    }
+  }
+#endif /* HAIKU */
+
 #elif defined(PCR)
 
 # include "il/PCR_IL.h"
