@@ -2,7 +2,7 @@
 ;;; control.thread-pool - thread pool
 ;;;
 ;;;  Copyright (c) 2003-2007 Time Intermedia Corporation, All rights reserved.
-;;;  Copyright (c) 2010-2014  Shiro Kawai  <shiro@acm.org>
+;;;  Copyright (c) 2010-2018  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;  Redistribution and use in source and binary forms, with or without
 ;;;  modification, are permitted provided that the following conditions
@@ -38,7 +38,7 @@
 (define-module control.thread-pool
   (use srfi-1)
   (use srfi-19)
-  (use util.queue)
+  (use data.queue)
   (use util.match)
   (use gauche.threads)
   (use gauche.record)
