@@ -1,7 +1,7 @@
 ;;;
 ;;; gauche.cgen.literal - static literal data
 ;;;
-;;;   Copyright (c) 2004-2014  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2004-2018  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -38,13 +38,16 @@
   (use gauche.parameter)
   (use gauche.sequence)
   (use gauche.cgen.unit)
-  (use gauche.experimental.app)
   (use util.match)
   (export <cgen-literal> cgen-c-name cgen-cexpr cgen-make-literal
           cgen-literal-static?
           define-cgen-literal cgen-literal
           cgen-allocate-static-datum
-          c-safe-string-literal         ;useful to emit C string constant
+
+          make-serializable-extended-pair
+
+          ;; Deprecated - use cgen-safe-string
+          c-safe-string-literal
           )
   )
 (select-module gauche.cgen.literal)
@@ -156,14 +159,14 @@
               (if (eq? category 'constant) "SCM_CGEN_CONST " "")
               name)
       (dolist [dl dls]
-        (for-each (cut print "#if "<>) (~ dl'cpp-conditions))
+        (for-each (cut print "#if "<>) (reverse (~ dl'cpp-conditions)))
         (format #t "  ~a ~a[~a];\n" (~ dl'c-type) (~ dl'c-member-name)
                 (~ dl'count))
         (for-each (cut print "#endif /*"<>"*/") (~ dl'cpp-conditions)))
       (format #t "} ~a = " name)))
 
   (define (emit-initializers dl)
-    (for-each (cut print "#if "<>) (~ dl'cpp-conditions))
+    (for-each (cut print "#if "<>) (reverse (~ dl'cpp-conditions)))
     (print "  {   /* "(~ dl'c-type)" "(~ dl'c-member-name)" */")
     (dolist [thunk (reverse (~ dl'init-thunks))]
       (if (string? thunk)
@@ -468,23 +471,13 @@
       :c-name (cgen-allocate-static-datum
                'constant 'ScmString
                (format "  SCM_STRING_CONST_INITIALIZER(~a, ~a, ~a)"
-                       (c-safe-string-literal value)
+                       (cgen-safe-string value)
                        (string-size value) (string-length value)))
       :value value))
   )
 
-(define (c-safe-string-literal value)
-  (with-string-io value
-    (lambda ()
-      (display "\"")
-      (port-for-each
-       (^b (if (or (= #x20 b) (= #x21 b) ; #x22 = #\"
-                   (<= #x23 b #x3e)      ; #x3f = #\?  - avoid trigraph trap
-                   (<= #x40 b #x5b)      ; #x5c = #\\
-                   (<= #x5d b #x7e))
-             (write-byte b)
-             (format #t "\\~3,'0o" b))) read-byte)
-      (display "\""))))
+;; For the backward compatibility
+(define c-safe-string-literal cgen-safe-string)
 
 ;; symbol ------------------------------------------------------
 
@@ -528,9 +521,15 @@
    )
   (make (value)
     (cond
-     [(fixnum? value)
+     ;; We don't use fixnum?, since we may be cross-compiling on 64bit
+     ;; machine for 32bit machine.  This is the range of 30bit fixnum.
+     [(< (- (%expt 2 29)) value (- (%expt 2 29) 1))
       (make <cgen-scheme-integer> :value value :c-name #f)]
-     [(< (- (%expt 2 31)) value (- (%expt 2 32)))
+     ;; Integers that doesn't fit in 30bit fixnum but the literal number
+     ;; can fit in C 32bit integers.  Note: We can use both signed and
+     ;; unsigned literals, so the upper bound is 2^32-1, while
+     ;; the lower bound is -2^31.
+     [(< (- (%expt 2 31)) value (- (%expt 2 32) 1))
       (make <cgen-scheme-integer> :value value
             :c-name (cgen-allocate-static-datum))]
      [else
@@ -606,6 +605,10 @@
 
 ;; pair ---------------------------------------------------------
 
+;; NB: We don't serialize extended pairs automatically.  Pair attributes
+;; can contain arbitrary information, some of which may be transient, and
+;; serializing everything isn't usually what you want.
+
 (define-cgen-literal <cgen-scheme-pair> <pair>
   ((car :init-keyword :car)
    (cdr :init-keyword :cdr))
@@ -626,6 +629,40 @@
       ))
   )
 
+;; If you do need to serialize extended pair, replace it
+;; with <serializable-extended-pair>, and set its attr slot
+;; only with necessary info.
+
+(define-class <serializable-extended-pair> ()
+  ((car :init-keyword :car)
+   (cdr :init-keyword :cdr)
+   (attrs :init-keyword :attrs :init-value '())))
+
+;; This always create serializable extended pair, even PAIR is just an
+;; ordinary pair, so that you can call pair-attribute-set! afterwards.
+(define (make-serializable-extended-pair ca cd attrs)
+  (make <serializable-extended-pair> :car ca :cdr cd :attrs attrs))
+
+(define-cgen-literal <cgen-scheme-extended-pair> <serializable-extended-pair>
+  ((car :init-keyword :car)
+   (cdr :init-keyword :cdr)
+   (attrs :init-keyword :attrs))
+  (make (value)
+    (let* ([ca (cgen-literal (~ value'car))]
+           [cd (cgen-literal (~ value'cdr))]
+           [attrs (cgen-literal (~ value'attrs))])
+      (make <cgen-scheme-extended-pair>
+        :value value :car ca :cdr cd :attrs attrs
+        :c-name (cgen-allocate-static-datum))))
+  (static (self) #f)
+  (init (self)
+    (print "  " (cgen-c-name self)
+           " = Scm_MakeExtendedPair("
+           (cgen-cexpr (~ self'car)) ", "
+           (cgen-cexpr (~ self'cdr)) ", "
+           (cgen-cexpr (~ self'attrs)) ");"))
+  )
+
 ;; vector -------------------------------------------------------
 
 ;; NB: ScmVector has variable part, so we can't directly initialize it.
@@ -690,13 +727,13 @@
   (rxmatch->string #/<(.\d+)vector>/ (symbol->string (class-name class)) 1))
 
 (define (uvector-class->c-name class)
-  #`"Scm_,(string-upcase (uvector-class->tag-name class))VectorClass")
+  #"Scm_~(string-upcase (uvector-class->tag-name class))VectorClass")
 
 (define (uvector-class->type-enum class)
   (global-variable-ref
    (find-module 'gauche)
    (string->symbol
-    #`"SCM_UVECTOR_,(string-upcase (uvector-class->tag-name class))")))
+    #"SCM_UVECTOR_~(string-upcase (uvector-class->tag-name class))")))
 
 (define (uvector-class->c-type-name class)
   (cond
