@@ -1,7 +1,7 @@
 ;;;
 ;;; gauce.cgen.stub - stub forms parsing and code generation
 ;;;
-;;;   Copyright (c) 2000-2014  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2000-2018  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -37,8 +37,8 @@
   (use srfi-42)
   (use util.match)
   (use text.tr)
+  (use text.tree)
   (use file.util)
-  (use util.list)
   (use gauche.parameter)
   (use gauche.sequence)
   (use gauche.mop.instance-pool)
@@ -48,6 +48,7 @@
   (use gauche.cgen.cise)
   (use gauche.cgen.tmodule)
   (export <cgen-stub-unit> <cgen-stub-error>
+          cgen-stub-cise-ambient
           cgen-genstub
           cgen-stub-parser cgen-stub-parse-form)
   )
@@ -166,7 +167,7 @@
 ;;             results.
 ;;        a string : becomes the body of C code.
 ;;
-;;   define-cgeneric name c-name property-clause ...)
+;;   define-cgeneric name c-name property-clause ...
 ;;
 ;;      Defines generic function.   C-name specifies a C variable name
 ;;      that keeps the generic function structure.  One or more of
@@ -235,8 +236,8 @@
 (define (cgen-genstub stubfile :key (predef-syms '()))
   (unless (file-is-readable? stubfile)
     (error <cgen-stub-error> "Couldn't read input file:" stubfile))
-  (let1 prefix
-      (cgen-safe-name-friendly (path-sans-extension (sys-basename stubfile)))
+  (let1 prefix ($ cgen-safe-name-friendly $ path-sans-extension
+                  $ sys-basename stubfile)
     (parameterize ([cgen-current-unit
                     (make <cgen-stub-unit>
                       :name (path-sans-extension (sys-basename stubfile))
@@ -249,12 +250,16 @@
        (for-each cgen-define predef-syms)
        (cgen-include "<gauche.h>")
        (with-input-from-file stubfile
-         (cut port-fold
+         (cut generator-fold
               ;; We treat the initial raw strings specially---they will be
               ;; in decl part.  The flag decl-strings? tracks that.
               (^[form decl-strings?]
                 (cond [(and decl-strings? (string? form)) (cgen-decl form) #t]
-                      [else (cgen-stub-parse-form form) #f]))
+                      [else
+                       (guard (e [else
+                                  ($ raise $ make-compound-condition e
+                                     $ make <compile-error-mixin> :expr form)])
+                         (cgen-stub-parse-form form) #f)]))
               #t read))
        (cgen-emit-c (cgen-current-unit))))))
 
@@ -284,12 +289,15 @@
    (handler :init-keyword :handler)))
 
 (define-macro (define-form-parser name args . body)
-  `(make <form-parser> :name ',name :args ',args :handler (^ ,args ,@body)))
+  `(make <form-parser> :name ',name :args ',args
+         :handler (^ (&whole ,@args) ,@body)))
 
 (define-method invoke ((self <form-parser>) form)
   (define (badform)
     (errorf <cgen-stub-error> "stub: malformed ~a: ~s" (~ self'name) form))
   (apply (~ self'handler)
+         ;; the first (implicit) argument is form itself, bound to &whole
+         form
          ;; need to check if given form matches args
          (let loop ([llist (~ self'args)]
                     [form  (cdr form)])
@@ -319,7 +327,18 @@
   (eval `(define-cise-expr ,@args) (current-module)))
 
 (define-form-parser define-cfn args
-  (cgen-body (cise-render-to-string `(define-cfn ,@args) 'toplevel)))
+  (parameterize ([cise-ambient (cise-ambient-copy (cise-ambient) '())])
+    (cgen-body (cise-render-to-string `(define-cfn ,@args) 'toplevel))
+    (cgen-decl (cise-ambient-decl-strings (cise-ambient)))))
+
+;; extra check of valid clauses
+(define (check-clauses directive name clauses valid-keys)
+  (and-let1 bad (find (^c (or (not (pair? c))
+                              (not (memq (car c) valid-keys))))
+                      clauses)
+    (errorf <cgen-stub-error>
+            "Invalid clause for ~s ~s: Expected a list starting with either one of ~s, but got: ~s"
+            directive name valid-keys bad)))
 
 ;;===================================================================
 ;; Type handling
@@ -525,6 +544,7 @@
    (return-type       :initform #f :init-keyword :return-type)
       ;; return type given by ::<type>.
    (decls             :initform '())
+      ;; reverse list of C declaration that should come before body stmts.
    (stmts             :initform '())
       ;; reverse list of C stmt lines.
    (c++-handlers      :initform '())
@@ -535,13 +555,59 @@
    (flags             :initform '() :init-keyword :flags)
       ;; list of keywords to modify code generation.  currently
       ;; :fast-flonum and :constant are supported.
+   (forward-decls     :initform '())
+      ;; list of strings for code that must be emitted before the
+      ;; procedure definition
+   (source-info       :initform #f :init-keyword :source-info)
+      ;; (file::<string> line-no::<integer>), if known.
+      ;; Will be saved in pair attributes of info.
+   (bind-info         :initform #f :init-keyword :bind-info)
+      ;; (module-name::<symbol> proc-name::<symbol>), if known.
+      ;; Will be saved in pair attributes of info.
+   (info              :initform #f)
+      ;; cgen-literal of ScmObj to be set in the 'info' slot of ScmProcedure.
    ))
 
 (define (get-arg cproc arg) (find (^x (eq? arg (~ x'name))) (~ cproc'args)))
 (define (push-stmt! cproc stmt) (push! (~ cproc'stmts) stmt))
+(define (push-decl! cproc decl) (push! (~ cproc'decls) decl))
 
 (define-generic c-stub-name )
 
+;; API
+;; This parameter holds <procstub> during processing the body.
+(define current-procstub (make-parameter #f))
+
+;; API
+;; We use customized cise context while generating stubs
+(define (cgen-stub-cise-ambient :optional (orig-ambient (cise-ambient)))
+  (rlet1 ctx (cise-ambient-copy orig-ambient '())
+    ;; Override "return"
+    ($ cise-register-macro! 'return
+       (^[form env]
+         (match form
+           [(_)         `(goto SCM_STUB_RETURN)]
+           [(_ e0)       `(begin (set! SCM_RESULT ,e0)
+                                 (goto SCM_STUB_RETURN))]
+           [(_ e0 e1)    `(begin (set! SCM_RESULT0 ,e0)
+                                 (set! SCM_RESULT1 ,e1)
+                                 (goto SCM_STUB_RETURN))]
+           [(_ e0 e1 e2) `(begin (set! SCM_RESULT0 ,e0)
+                                 (set! SCM_RESULT1 ,e1)
+                                 (set! SCM_RESULT2 ,e2)
+                                 (goto SCM_STUB_RETURN))]
+           [_ (error "Too many values to return")]))
+       ctx)))
+
+(define-syntax with-cise-ambient
+  (syntax-rules ()
+    [(_ procstub body ...)
+     (parameterize ([cise-ambient (cgen-stub-cise-ambient (cise-ambient))]
+                    [current-procstub procstub])
+       body ...
+       (push! (~ procstub'forward-decls)
+              (cise-ambient-decl-strings (cise-ambient))))]))
+
 ;;-----------------------------------------------------------------
 ;; (define-cproc scheme-name (argspec) body)
 ;;
@@ -581,6 +647,19 @@
                       :proc-name (make-literal (x->string scheme-name))
                       :return-type rettype :flags flags
                       :args args
+                      ;; Heuristics - Ideally &whole has source info, but
+                      ;; precomp may have reconstructed the form and the
+                      ;; original source info may be lost.  We try to find
+                      ;; relevant info from a few subforms.
+                      ;; Once we fix precomp, we can just say
+                      ;; (debug-source-info &whole).
+                      :source-info (or (debug-source-info &whole)
+                                       (debug-source-info (cdr &whole))
+                                       (debug-source-info (caddr &whole)))
+                      :bind-info (and-let* ([m (current-tmodule)]
+                                            [mod-name (~ m'name)]
+                                            [ (symbol? mod-name) ])
+                                   (list mod-name scheme-name))
                       :keyword-args keyargs
                       :num-reqargs nreqs
                       :num-optargs nopts
@@ -588,10 +667,82 @@
                       :allow-other-keys? other-keys?)
           (process-body cproc body)
           (check-fast-flonum cproc args)
+          (assign-proc-info! cproc)
           (cgen-add! cproc))))))
 
 (define-method c-stub-name ((cproc <cproc>)) #`",(~ cproc'c-name)__STUB")
 
+;; Get stub name of the named cproc in the current unit.
+(define (cproc-stub-c-name scheme-name)
+  (if-let1 cproc (find (^p (eq? (~ p'scheme-name) scheme-name))
+                       (get-stubs <cproc>))
+    (c-stub-name cproc)
+    (error "cproc-stub-c-name: No such cproc: " scheme-name)))
+
+;; In cise, you can use .cproc macro to get cproc stub as ScmObj.
+(define-cise-expr .cproc
+  [(_ name)
+   (unless (symbol? name) (error ".cproc requires a symbol, but got:" name))
+   `(SCM_OBJ (& (C: ,(cproc-stub-c-name name))))])
+
+;; Allow to call Scheme procedure from <procstub> body.
+;;
+;;  (.funcall/rec scheme-proc-name arg ...)
+;;  => static ScmObj tmp = SCM_UNDEFINED;
+;;     SCM_BIND_PROC(tmp, "scheme-proc-name", current_module);
+;;     Scm_ApplyRec(tmp, args);
+;;
+;;  (.funcall/cps scheme-proc-name arg ...)
+;;  => static ScmObj tmp = SCM_UNDEFINED;
+;;     SCM_BIND_PROC(tmp, "scheme-proc-name", current_module);
+;;     Scm_VMApplyRec(tmp, args);
+
+;; returns tmp-var name (symbol)
+;; module can be #f (then the tmodule is used), or a symbol
+(define (%insert-funcall-decls who scheme-proc-name module)
+  (if-let1 cproc (current-procstub)
+    (rlet1 tmpvar (gensym "proc")
+      (let ([ctmpvar (cgen-safe-name (symbol->string tmpvar))]
+            [modexp  (if module
+                       #"Scm_FindModule(SCM_SYMBOL(SCM_INTERN(\"~module\")),\
+                                        SCM_FIND_MODULE_CREATE)"
+                       #"SCM_MODULE(~(tmodule-cname (current-tmodule)))")])
+        (push-decl! cproc #"static ScmObj ~ctmpvar = SCM_UNDEFINED;")
+        (push-decl! cproc #"SCM_BIND_PROC(~ctmpvar, \
+                             \"~scheme-proc-name\", \
+                             ~modexp);")))
+    (errorf "~a form is not in define-cproc" who)))
+
+(define (%expand-funcall-apply applyproc tmpvar args)
+  (define (suffix n) (string->symbol #"~|applyproc|~|n|"))
+  (match args
+    [()           `(,(suffix 0) ,tmpvar)]
+    [(a)          `(,(suffix 1) ,tmpvar ,a)]
+    [(a b)        `(,(suffix 2) ,tmpvar ,a ,b)]
+    [(a b c)      `(,(suffix 3) ,tmpvar ,a ,b ,c)]
+    [(a b c d)    `(,(suffix 4) ,tmpvar ,a ,b ,c ,d)]
+    [(a b c d e)  `(,(suffix 5) ,tmpvar ,a ,b ,c ,d ,e)]
+    ;; TODO: for more than 5 args, emit list generation code
+    ))
+
+(define-cise-expr .funcall/rec
+  [(_ (module-name scheme-proc-name) arg ...)
+   (let1 tmpvar (%insert-funcall-decls '.funcall/rec
+                                       scheme-proc-name module-name)
+     (%expand-funcall-apply 'Scm_ApplyRec tmpvar arg))]
+  [(_ scheme-proc-name arg ...)
+   (let1 tmpvar (%insert-funcall-decls '.funcall/rec scheme-proc-name #f)
+     (%expand-funcall-apply 'Scm_ApplyRec tmpvar arg))])
+
+(define-cise-expr .funcall/cps
+  [(_ (module-name scheme-proc-name) arg ...)
+   (let1 tmpvar (%insert-funcall-decls '.funcall/cps
+                                       scheme-proc-name module-name)
+     (%expand-funcall-apply 'Scm_VMApply tmpvar arg))]
+  [(_ scheme-proc-name arg ...)
+   (let1 tmpvar (%insert-funcall-decls '.funcall/cps scheme-proc-name #f)
+     (%expand-funcall-apply 'Scm_VMApply tmpvar arg))])
+
 ;; create arg object.  used in cproc and cmethod
 (define (make-arg class argname count . rest)
   (define (grok-argname argname)
@@ -753,7 +904,7 @@
                          (append (~ cproc'args) (~ cproc'keyword-args)))]
               [form (if (eq? (~ cproc'return-type) '<void>)
                       `((,s ,@args))
-                      `((result (,s ,@args))))]
+                      `((return (,s ,@args))))] ; this is the overridden "return" - see cgen-stub-cise-ambient above
               [rettype (if (pair? (~ cproc'return-type))
                          '<top>
                          (~ cproc'return-type))])
@@ -808,21 +959,26 @@
     (push-stmt! cproc "{")
     (push-stmt! cproc #`",(~ rettype'c-type) SCM_RESULT;")
     (push-stmt! cproc #`"SCM_RESULT = ,c-func-name(,(args));")
+    (push-stmt! cproc "goto SCM_STUB_RETURN;") ; avoid 'label not used' error
+    (push-stmt! cproc "SCM_STUB_RETURN:") ; label
     (push-stmt! cproc (cgen-return-stmt (cgen-box-expr rettype "SCM_RESULT")))
     (push-stmt! cproc "}"))
-  (match form
-    [([? check-expr expr])
-     (let1 rt (~ cproc'return-type)
-       (if rt
-         (process-call-spec cproc `(,rt ,expr)) ; for transition
-         (typed-result *scm-type* expr)))]
-    [('<void> [? check-expr expr])
-     (push-stmt! cproc #`",expr(,(args));")
-     (push-stmt! cproc "SCM_RETURN(SCM_UNDEFINED);")]
-    [(typename [? check-expr expr])
-     (unless (and (symbol? typename) (check-expr expr)) (err))
-     (typed-result (name->type typename) expr)]
-    [else (err)]))
+  ($ with-cise-ambient cproc
+     (match form
+       [([? check-expr expr])
+        (let1 rt (~ cproc'return-type)
+          (if rt
+            (process-call-spec cproc `(,rt ,expr)) ; for transition
+            (typed-result *scm-type* expr)))]
+       [('<void> [? check-expr expr])
+        (push-stmt! cproc #`",expr(,(args));")
+        (push-stmt! cproc "goto SCM_STUB_RETURN;") ; avoid 'label not used' error
+        (push-stmt! cproc "SCM_STUB_RETURN:") ; label
+        (push-stmt! cproc "SCM_RETURN(SCM_UNDEFINED);")]
+       [(typename [? check-expr expr])
+        (unless (and (symbol? typename) (check-expr expr)) (err))
+        (typed-result (name->type typename) expr)]
+       [else (err)])))
 
 (define-method process-body-spec ((cproc <procstub>) form)
   (define (err) (error <cgen-stub-error> "malformed 'body' spec:" form))
@@ -842,19 +998,22 @@
       (push-stmt! cproc "{")
       (push-stmt! cproc #`",(~ rettype'c-type) SCM_RESULT;")
       (push-stmt! cproc #`" SCM_RESULT = (,expr);")
+      (push-stmt! cproc "goto SCM_STUB_RETURN;") ; avoid 'label not used' error
+      (push-stmt! cproc "SCM_STUB_RETURN:") ; label
       (push-stmt! cproc (cgen-return-stmt (cgen-box-expr rettype "SCM_RESULT")))
       (push-stmt! cproc "}")))
-  (match form
-    [('<void> . stmts)
-     (error <cgen-stub-error> "<void> type isn't allowed in 'expr' directive:" form)]
-    [([? symbol? rettype] expr)
-     (typed-result (name->type rettype) expr)]
-    [(expr)
-     (let1 rt (~ cproc'return-type)
-       (if rt
-         (process-expr-spec cproc `(,rt ,expr)) ; for transition
-         (typed-result *scm-type* expr)))]
-    [else (error <cgen-stub-error> "malformed 'expr' spec:" form)]))
+  ($ with-cise-ambient cproc
+     (match form
+       [('<void> . stmts)
+        (error <cgen-stub-error> "<void> type isn't allowed in 'expr' directive:" form)]
+       [([? symbol? rettype] expr)
+        (typed-result (name->type rettype) expr)]
+       [(expr)
+        (let1 rt (~ cproc'return-type)
+          (if rt
+            (process-expr-spec cproc `(,rt ,expr)) ; for transition
+            (typed-result *scm-type* expr)))]
+       [else (error <cgen-stub-error> "malformed 'expr' spec:" form)])))
 
 (define-method process-catch-spec ((cproc <procstub>) form)
   (match form
@@ -886,6 +1045,8 @@
     (push-stmt! cproc "{")
     (push-stmt! cproc #`",(~ rettype'c-type) SCM_RESULT;")
     (for-each expand-stmt stmts)
+    (push-stmt! cproc "goto SCM_STUB_RETURN;") ; avoid 'label not used' error
+    (push-stmt! cproc "SCM_STUB_RETURN:") ; label
     (push-stmt! cproc (cgen-return-stmt (cgen-box-expr rettype "SCM_RESULT")))
     (push-stmt! cproc "}"))
   (define (typed-results rettypes stmts)
@@ -902,6 +1063,8 @@
                              (cgen-box-expr rettype #`"SCM_RESULT,i"))
                            rettypes)
            ",")
+        (push-stmt! cproc "goto SCM_STUB_RETURN;") ; avoid 'label not used' error
+        (push-stmt! cproc "SCM_STUB_RETURN:") ; label
         (push-stmt! cproc
                     (case nrets
                       [(0) (cgen-return-stmt "Scm_Values(SCM_NIL)")]
@@ -913,37 +1076,92 @@
                       [else (cgen-return-stmt
                              #`"Scm_Values(Scm_List(,results,, NULL))")]))
         )))
-  (match rettype
-    [#f             ; the default case; we assume it is <top>.
-     (typed-result *scm-type* body)]
-    ['<void>        ; no results
-     (for-each expand-stmt body)
-     (push-stmt! cproc "SCM_RETURN(SCM_UNDEFINED);")]
-    [[? symbol? t]  ; single result
-     (typed-result (name->type t) body)]
-    [[? list? ts]   ; multiple values
-     (typed-results (map name->type ts) body)]
-    [_ (error <cgen-stub-error> "invalid cproc return type:" rettype)]))
+  ($ with-cise-ambient cproc
+     (match rettype
+       [#f             ; the default case; we assume it is <top>.
+        (typed-result *scm-type* body)]
+       ['<void>        ; no results
+        (for-each expand-stmt body)
+        (push-stmt! cproc "goto SCM_STUB_RETURN;") ; avoid 'label not used' error
+        (push-stmt! cproc "SCM_STUB_RETURN:") ; label
+        (push-stmt! cproc "SCM_RETURN(SCM_UNDEFINED);")]
+       [[? symbol? t]  ; single result
+        (typed-result (name->type t) body)]
+       [[? list? ts]   ; multiple values
+        (typed-results (map name->type ts) body)]
+       [_ (error <cgen-stub-error> "invalid cproc return type:" rettype)])))
+
+(define-method assign-proc-info! ((proc <cproc>))
+  (define (arginfo arg) (~ arg'name))
+  (let* ([qargs (filter (cut is-a? <> <required-arg>) (~ proc'args))]
+         [oargs (filter (cut is-a? <> <optional-arg>) (~ proc'args))]
+         [kargs (filter (cut is-a? <> <keyword-arg>) (~ proc'args))]
+         [rarg  (filter (cut is-a? <> <rest-arg>) (~ proc'args))]
+         [aarg  (filter (cut is-a? <> <optarray-arg>) (~ proc'args))]
+         [all-args `(,@(map arginfo qargs)
+                     ,@(cond-list
+                        [(pair? oargs) @ `(:optional ,@(map arginfo oargs))]
+                        [(pair? kargs) @ `(:key ,@(map arginfo kargs))]
+                        [(pair? rarg) @ `(:rest ,(arginfo (car rarg)))]
+                        [(pair? aarg) @ `(:optarray ,(arginfo (car aarg)))]))])
+    (set! (~ proc'info)
+          (make-literal
+           (if (or (~ proc'source-info) (~ proc'bind-info))
+             ($ make-serializable-extended-pair (~ proc'scheme-name) all-args
+                (cond-list
+                 [(~ proc'source-info) => (cut cons 'source-info <>)]
+                 [(~ proc'bind-info)   => (cut cons 'bind-info <>)]))
+             (cons (~ proc'scheme-name) all-args))))))
 
 ;;;
 ;;; Emitting code
 ;;;
 
+(define-method cgen-emit-decl ((proc <procstub>))
+  (dolist [s (~ proc'forward-decls)] (p s)))
+
+(define-method cgen-emit-decl ((cproc <cproc>))
+  (next-method)
+  ;; We emit forward definitions of C function names and stub record definitions
+  ;; first, so that the function body can refer to the stub record.
+  (p "static ScmObj "(~ cproc'c-name)"(ScmObj*, int, void*);")
+  (let1 flags (~ cproc'flags)
+    (format #t "static SCM_DEFINE_SUBR~a(" (if (null? flags) "" "X"))
+    (format #t "~a, ~a, ~a,"
+            (c-stub-name cproc)                       ; cvar
+            (~ cproc'num-reqargs)                     ; req
+            (cond [(zero? (~ cproc'num-optargs)) 0]
+                  [(not (null? (~ cproc'keyword-args))) 1]
+                  [(~ cproc'have-rest-arg?) (~ cproc'num-optargs)]
+                  [else (+ (~ cproc'num-optargs) 1)])); opt
+    (unless (null? flags)                             ; cst
+      (if (memq :constant flags) (display "1, ") (display "0, ")))
+    (format #t "SCM_FALSE,")                          ; info - to be set in init
+    (unless (null? flags)                             ; flags
+      (if (memq :fast-flonum flags)
+        (display "SCM_SUBR_IMMEDIATE_ARG, ")
+        (display "0, ")))
+    (format #t "~a, ~a, NULL);\n"
+            (~ cproc'c-name)                          ; func
+            (cond [(~ cproc'inline-insn)
+                   => (^i (format "SCM_MAKE_INT(SCM_VM_~a)"
+                                  (string-tr (x->string i) "-" "_")))]
+                  [else "NULL"])))                    ; inliner
+  (p))
+
 (define-method cgen-emit-body ((cproc <cproc>))
   (p "static ScmObj "(~ cproc'c-name)"(ScmObj *SCM_FP, int SCM_ARGCNT, void *data_)")
   (p "{")
   ;; argument decl
   (for-each emit-arg-decl (~ cproc'args))
   (for-each emit-arg-decl (~ cproc'keyword-args))
-  (if (null? (~ cproc'keyword-args))
-    (p "  ScmObj SCM_SUBRARGS["(+ (length (~ cproc'args))
-                                  (~ cproc'num-optargs))"];")
-    (begin
-      (p "  ScmObj SCM_SUBRARGS["(+ (length (~ cproc'args))
-                                    (~ cproc'num-optargs)
-                                    -1)"];")
-      (p "  ScmObj SCM_OPTARGS = SCM_ARGREF(SCM_ARGCNT-1);")))
-  (p "  int SCM_i;")
+  (let1 arg-array-size (+ (length (~ cproc'args))
+                          (~ cproc'num-optargs)
+                          (if (null? (~ cproc'keyword-args)) 0 -1))
+    (unless (zero? arg-array-size)
+      (p "  ScmObj SCM_SUBRARGS["arg-array-size"];")))
+  (unless (null? (~ cproc'keyword-args))
+    (p "  ScmObj SCM_OPTARGS = SCM_ARGREF(SCM_ARGCNT-1);"))
   (p "  SCM_ENTER_SUBR(\""(~ cproc'scheme-name)"\");")
   ;; argument count check (for optargs)
   (when (and (> (~ cproc'num-optargs) 0)
@@ -958,9 +1176,10 @@
   ;; argument assertions & unbox op.
   (let1 k (+ (length (~ cproc'args)) (~ cproc'num-optargs)
              (if (null? (~ cproc'keyword-args)) 0 -1))
-    (p "  for (SCM_i=0; SCM_i<"k"; SCM_i++) {")
-    (p "    SCM_SUBRARGS[SCM_i] = SCM_ARGREF(SCM_i);")
-    (p "  }"))
+    (unless (zero? k)
+      (p "  for (int SCM_i=0; SCM_i<"k"; SCM_i++) {")
+      (p "    SCM_SUBRARGS[SCM_i] = SCM_ARGREF(SCM_i);")
+      (p "  }")))
   (for-each emit-arg-unbox (~ cproc'args))
   (unless (null? (~ cproc'keyword-args))
     (emit-keyword-args-unbox cproc))
@@ -968,6 +1187,7 @@
   (unless (null? (~ cproc'c++-handlers))
     (p "try {"))
   (p "  {")
+  (for-each p (reverse (~ cproc'decls)))
   (for-each p (reverse (~ cproc'stmts)))
   (p "  }")
   (unless (null? (~ cproc'c++-handlers))
@@ -978,30 +1198,6 @@
       (p "}")))
   ;; closing the function
   (p "}")
-  (p)
-  ;; emit stub record
-  (let1 flags (~ cproc'flags)
-    (format #t "static SCM_DEFINE_SUBR~a(" (if (null? flags) "" "X"))
-    (format #t "~a, ~a, ~a,"
-            (c-stub-name cproc)                       ; cvar
-            (~ cproc'num-reqargs)                     ; req
-            (cond [(zero? (~ cproc'num-optargs)) 0]
-                  [(not (null? (~ cproc'keyword-args))) 1]
-                  [(~ cproc'have-rest-arg?) (~ cproc'num-optargs)]
-                  [else (+ (~ cproc'num-optargs) 1)])); opt
-    (unless (null? flags)                             ; cst
-      (if (memq :constant flags) (display "1, ") (display "0, ")))
-    (format #t "~a," (cgen-c-name (~ cproc'proc-name))); inf
-    (unless (null? flags)                              ; flags
-      (if (memq :fast-flonum flags)
-        (display "SCM_SUBR_IMMEDIATE_ARG, ")
-        (display "0, ")))
-    (format #t "~a, ~a, NULL);\n"
-            (~ cproc'c-name)                          ; func
-            (cond [(~ cproc'inline-insn)
-                   => (^i (format "SCM_MAKE_INT(SCM_VM_~a)"
-                                  (string-tr (x->string i) "-" "_")))]
-                  [else "NULL"])))                    ; inliner
   (p))
 
 (define-method cgen-emit-init ((cproc <cproc>))
@@ -1012,6 +1208,9 @@
        (c-stub-name cproc)
        (if (or (~ cproc'inline-insn) (memq :constant (~ cproc'flags)))
          "SCM_BINDING_INLINABLE" "0")))
+  (when (~ cproc'info)
+    (f "  ~a.common.info = ~a;"
+       (c-stub-name cproc) (cgen-c-name (~ cproc'info))))
   (next-method)
   )
 
@@ -1176,6 +1375,10 @@
                      :specializers specializers
                      :num-reqargs numargs
                      :args args
+                     ;; NB: See define-cproc about this.
+                     :source-info (or (debug-source-info &whole)
+                                      (debug-source-info (cdr &whole))
+                                      (debug-source-info (caddr &whole)))
                      :have-rest-arg? have-optarg?
                      )
         (let loop ([body body])
@@ -1233,6 +1436,7 @@
   (for-each emit-arg-unbox (~ method'args))
   ;; body
   (p "  {")
+  (for-each p (reverse (~ method'decls)))
   (for-each p (reverse (~ method'stmts)))
   (p "  }")
   (p "}")
@@ -1296,6 +1500,7 @@
 ;;   (<slot-spec> ...)
 ;;   [(allocator <proc-spec>)]
 ;;   [(printer   <proc-spec>)]
+;;   [(comparer  <proc-spec>)]
 ;;   [(direct-supers <string> ...)]
 ;;   )
 ;;
@@ -1341,6 +1546,7 @@
    (qualifiers :init-keyword :qualifiers)
    (allocator  :init-keyword :allocator :init-value #f)
    (printer    :init-keyword :printer   :init-value #f)
+   (comparer   :init-keyword :comparer  :init-value #f)
    (slot-spec  :init-keyword :slot-spec :init-value '())
    (direct-supers :init-keyword :direct-supers :init-value '())
    ))
@@ -1375,26 +1581,34 @@
        (check-arg list? slot-spec)
        (let* ([allocator (cond [(assq 'allocator more) => cadr] [else #f])]
               [printer   (cond [(assq 'printer more) => cadr] [else #f])]
+              [comparer  (cond [(assq 'comparer more) => cadr] [else #f])]
               [dsupers   (cond [(assq 'direct-supers more) => cdr] [else '()])]
               [cclass (make <cclass>
                         :scheme-name scm-name :c-type c-type :c-name c-name
                         :qualifiers quals
                         :cpa cpa :direct-supers dsupers
-                        :allocator allocator :printer printer)])
+                        :allocator allocator :printer printer
+                        :comparer comparer)])
          (set! (~ cclass'slot-spec) (process-cclass-slots cclass slot-spec))
          (cgen-add! cclass))])))
 
+(define-method c-allocator-name ((self <cclass>))
+  (let1 allocator (~ self'allocator)
+    (cond [(c-literal-expr allocator)]
+          [(not allocator) "NULL"]
+          [else #`",(~ self'c-name)_ALLOCATE"])))
+
 (define-method c-printer-name ((self <cclass>))
   (let1 printer (~ self'printer)
     (cond [(c-literal-expr printer)]
           [(not printer) "NULL"]
           [else #`",(~ self'c-name)_PRINT"])))
 
-(define-method c-allocator-name ((self <cclass>))
-  (let1 allocator (~ self'allocator)
-    (cond [(c-literal-expr allocator)]
-          [(not allocator) "NULL"]
-          [else #`",(~ self'c-name)_ALLOCATE"])))
+(define-method c-comparer-name ((self <cclass>))
+  (let1 comparer (~ self'comparer)
+    (cond [(c-literal-expr comparer)]
+          [(not comparer) "NULL"]
+          [else #`",(~ self'c-name)_COMPARE"])))
 
 (define-method c-slot-spec-name ((self <cclass>))
   (if (null? (~ self'slot-spec))
@@ -1423,14 +1637,20 @@
     (p (c-code (~ self'printer)))
     (p "}")
     (p ""))
+  (unless ((any-pred not c-literal?) (~ self'comparer))
+    (p "static int "(c-comparer-name self)"(ScmObj x, ScmObj y, int equalp)")
+    (p "{")
+    (p (c-code (~ self'comparer)))
+    (p "}")
+    (p ""))
   (emit-cpa self)
   (if (memv :base (~ self'qualifiers))
     (let1 c-type (string-trim-right (~ self'c-type))
       (unless (string-suffix? "*" c-type)
         (errorf <cgen-stub-error> "can't use C-type ~s as a base class; C-type must be a pointer type" c-type))
       (let1 c-instance-type (string-drop-right c-type 1)
-        (p "SCM_DEFINE_BASE_CLASS("(~ self'c-name)", "c-instance-type", "(c-printer-name self)", NULL, NULL, "(c-allocator-name self)", "(cpa-name self)");")))
-    (p "SCM_DEFINE_BUILTIN_CLASS("(~ self'c-name)", "(c-printer-name self)", NULL, NULL, "(c-allocator-name self)", "(cpa-name self)");"))
+        (p "SCM_DEFINE_BASE_CLASS("(~ self'c-name)", "c-instance-type", "(c-printer-name self)", "(c-comparer-name self)", NULL, "(c-allocator-name self)", "(cpa-name self)");")))
+    (p "SCM_DEFINE_BUILTIN_CLASS("(~ self'c-name)", "(c-printer-name self)", "(c-comparer-name self)", NULL, "(c-allocator-name self)", "(cpa-name self)");"))
   (p "")
   (when (pair? (~ self'slot-spec))
     (for-each emit-getter-n-setter (~ self'slot-spec))
@@ -1557,8 +1777,25 @@
 ;;    [(print print-proc)]
 ;;    [(cleanup cleanup-proc)]
 ;;
-;;  qualifier :: :private
-;;  flag :: :keep-identity | :map-nul
+;;  qualifier : :private
+;;  flag      : :keep-identity | :map-null
+;;
+;;  scm-name  : Scheme variable name; this will be bound to a newly-created
+;;              subclass of <foreign-pointer> to represent this C-ptr type.
+;;  c-type    : Name of the actual C type we wrap.
+;;  c-name    : A C variable name (must have type ScmClass *).  If :private
+;;              qualifier is given, static definition of the varaible is
+;;              generated; otherwise, the definition & declaration must
+;;              be provided elsewhere.  In initialization code, an instance
+;;              of a class (the same one bound to scm-name in the Scheme
+;;              world) will be stored in this C variable.
+;;  c-pred    : A macro name to determine if ScmObj is this type.
+;;  c-boxer   : A macro name to wrap C pointer and return ScmObj
+;;  c-unboxer : A macro name to extract C pointer from ScmObj
+;;
+;;  If :private is given, c-pred, c-boxer and c-unboxer definitions are
+;;  generated automatically.  Otherwise those definitions must be provided
+;;  elsewhere.
 
 (define-class <c-ptr> (<stub>)
   ((c-type       :init-keyword :c-type)
@@ -1577,6 +1814,7 @@
                  z scm-name)))
     (match args
       [(c-type c-varname c-pred c-boxer c-unboxer . clauses)
+       (check-clauses 'define-cptr scm-name clauses '(flags print cleanup))
        (let ([flags (or (assq-ref clauses 'flags) '())]
              [print-proc (assq-ref clauses 'print)]
              [cleanup-proc (assq-ref clauses 'cleanup)])
@@ -1593,8 +1831,8 @@
            (make-cgen-type scm-name c-type
                            (x->string scm-name) ; description
                            (x->string c-pred)
-                           (x->string c-boxer)
-                           (x->string c-unboxer))
+                           (x->string c-unboxer)
+                           (x->string c-boxer))
            (cgen-add! fptr)))]
       [_ (error <cgen-stub-error> "invalid define-cptr clause for" scm-name)])))
 
@@ -1671,7 +1909,7 @@
     ;; TODO: search path
     (error <cgen-stub-error> "couldn't find include file: " file))
   (with-input-from-file file
-    (^[] (port-for-each cgen-stub-parse-form read)))
+    (cut generator-for-each cgen-stub-parse-form read))
   )
 
 (define-form-parser initcode codes
