@@ -1,7 +1,7 @@
 ;;;
 ;;; common-macros.scm - common macros
 ;;;
-;;;   Copyright (c) 2000-2012  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2000-2018  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -38,17 +38,58 @@
 ;; on other autoloaded files, since it is easy to create circular
 ;; dependency.
 
-(select-module gauche)
+(define-module gauche.common-macros
+  (export syntax-error syntax-errorf
+          push! pop! inc! dec! update!
+          check-arg get-optional get-keyword*
+          ^ ^_ ^a ^b ^c ^d ^e ^f ^g ^h ^i ^j ^k ^l ^m ^n ^o ^p ^q
+          ^r ^s ^t ^u ^v ^w ^x ^y ^z $
+          let1 if-let1 and-let1 rlet1
+          let/cc begin0 fluid-let
+          let-values let*-values
+          values-ref values->list
+          ecase
+          dotimes doilst doplist while until
+          guard unwind-protect
+          cond-list
+          assume assume-type))
+(select-module gauche.common-macros)
 
-;;; syntax-error
-;;; syntax-errorf
-;;;   Signals an error at compile time.
+;;; syntax-error msg arg ...
+;;; syntax-errorf fmtstr arg ...
+;;;   Signal an error at compile time.
+;;;   These are typically used as a result of expansion of syntax-rules
+;;;   macro; er-macro or legacy macro can directly call error/errorf so
+;;;   there's no point to use syntax-error.  Then, the 'original attribute
+;;;   of the form contains the macro input that caused syntax error.
+;;;   We extract that and throw a compound condition, so that the
+;;;   error message will include the macro input that directly caused
+;;;   this error.
+;;;   TODO: Move this to core after 0.9.6.
 
-(define-macro (syntax-error . args)
-  (apply error (map unwrap-syntax args)))
+(define-syntax syntax-error
+  (er-macro-transformer
+   (lambda (f r c)
+     (let ([args (map unwrap-syntax (cdr f))]
+           [original ((with-module gauche.internal pair-attribute-get)
+                      f 'original #f)])
+       (if original
+         (raise (make-compound-condition
+                 (apply make-error (car args) (cdr args))
+                 (make <compile-error-mixin> :expr original)))
+         (apply error args))))))
 
-(define-macro (syntax-errorf . args)
-  (apply errorf (map unwrap-syntax args)))
+(define-syntax syntax-errorf
+  (er-macro-transformer
+   (lambda (f r c)
+     (let ([args (map unwrap-syntax (cdr f))]
+           [original ((with-module gauche.internal pair-attribute-get)
+                      f 'original #f)])
+       (if original
+         (raise (make-compound-condition
+                 (make-error (apply format/ss (car args) (cdr args)))
+                 (make <compile-error-mixin> :expr original)))
+         (apply errorf args))))))
 
 ;;;-------------------------------------------------------------
 ;;; generalized set! family
@@ -180,6 +221,11 @@
     [(if-let1 var exp then . else)
      (let ((var exp)) (if var then . else))]))
 
+(define-syntax and-let1                 ;returns #f if test evaluates to #f
+  (syntax-rules ()
+    [(and-let1 var test exp exp2 ...)
+     (let ((var test)) (and var (begin exp exp2 ...)))]))
+
 (define-syntax let/cc                   ;as in PLT
   (syntax-rules ()
     [(let/cc var . body)
@@ -243,6 +289,40 @@
                (lambda () ,@body)
                ,swap))))))
 
+;; srfi-11, r7rs
+(define-syntax let*-values
+  (syntax-rules ()
+    [(_ () . body)
+     (let () . body)]
+    [(_ ((formals init) . rest) . body)
+     (receive formals init
+       (let*-values rest . body))]))
+
+(define-syntax let-values
+  (er-macro-transformer
+   (^[f r c]
+     ;; we rename all the formals and rebind
+     (let* ([bindings (cadr f)]
+            [body (cddr f)]
+            [formals  (map car bindings)]
+            [inits    (map cadr bindings)]
+            [formals* (map (^f (map* (^x (gensym (x->string x)))
+                                     (^t (if (null? t) t (gensym (x->string t))))
+                                     f))
+                           formals)]
+            [rebinds (append-map
+                      (^[f f*]
+                        (map* list
+                              (^[a b] (if (null? a) '() (list (list a b))))
+                              f f*))
+                      formals formals*)])
+       ;; TRANSIENT: Avoid using quasirename until 0.9.6 release, for
+       ;; it autoloads macroutil.scm -> util/match.scm and create
+       ;; circular dependency.
+       `(,(r'let*-values) ,(map list formals* inits)
+         (,(r'let) ,rebinds
+          ,@body))))))
+
 ;;;-------------------------------------------------------------
 ;;; applications
 
@@ -266,10 +346,17 @@
 ;;  ($ f a b $* g c d $*) => (lambda args (apply f a b (apply g c d args)))
 ;;  ($ f a b $ g c d $*) => (lambda args (f a b (apply g c d args)))
 
-(define-syntax $
+;; Kludge: We already have binding of '$' in gauche module, created for
+;; autoload.  Using (define-syntax $ ...) here makes a separate binding in
+;; gauche.common-macros, and that makes the literal comparison of '$'
+;; fail, since '$' in here refers to gauche.common-macros#$, while
+;; the macro use environment will refer to gauche#$.  We use very ugly
+;; hack here to workaround the issue; needs fundamental fix later.
+(define-syntax %$
   (syntax-rules ()
     [($ x . xs) (%$-split (x . xs) () ())]
     [($) (syntax-error "invalid $ form" ($))]))
+(set! $ %$) ; DON'T DO THIS.  Only works in the current version.
 
 ;; ($ a b $ c d $*) => ($* (c d) $ (a b))
 ;; ($ a b $ c d $* e f) => ((e f) $* (c d) $ (a b))
@@ -332,29 +419,67 @@
 (define-syntax dotimes
   (syntax-rules ()
     [(_ (var n res) . body)
-     (do ((limit n)
-          (var 0 (+ var 1)))
-         ((>= var limit) res)
+     (do ([limit n]
+          [var 0 (+ var 1)])
+         [(>= var limit) res]
        . body)]
     [(_ (var n) . body)
-     (do ((limit n)
-          (var 0 (+ var 1)))
-         ((>= var limit))
+     (do ([limit n]
+          [var 0 (+ var 1)])
+         [(>= var limit) (undefined)]
        . body)]
+    [(_ (n) . body)
+     (let1 i n
+       (cond [(<= i 0) (undefined)]
+             [(infinite? i)
+              (do () (#f) . body)] ;avoid unnecessary flonum calculation
+             [else
+              (do ([i i (- i 1)])
+                  [(<= i 0) (undefined)]
+                . body)]))]
     [(_ . other)
      (syntax-error "malformed dotimes" (dotimes . other))]))
 
 (define-syntax dolist
   (syntax-rules ()
     [(_ (var lis res) . body)
-     (begin (for-each (lambda (var) . body) lis)
-            (let ((var '())) res))      ;bound var for CL compatibility
-     ]
+     (do ([p lis (cdr p)])
+         [(null? p)
+          (let1 var '() res)]      ;bound var for CL compatibility
+       (let1 var (car p) . body))]
     [(_ (var lis) . body)
-     (begin (for-each (lambda (var) . body) lis) '())]
+     (do ([p lis (cdr p)])
+         [(null? p) '()]
+       (let1 var (car p) . body))]
+    [(_ (lis) . body)
+     (dolist (tmp lis) . body)]
     [(_ . other)
      (syntax-error "malformed dolist" (dolist . other))]))
 
+(define-syntax doplist
+  (syntax-rules ()
+    [(_ ((k v) plis default) . body)
+     (do ([p plis (cddr p)])
+         [(cond [(null? p) #t]
+                [(null? (cdr p))
+                 (let ([k (car p)]
+                       [v default])
+                   . body)]
+                [else #f])]
+       (let ([k (car p)]
+             [v (cadr p)])
+         . body))]
+    [(_ ((k v) plis) . body)
+     (do ([p plis (cddr p)])
+         [(cond [(null? p) #t]
+                [(null? (cdr p)) (error "plist is not even:" plis)]
+                [else #f])]
+       (let ([k (car p)]
+             [v (cadr p)])
+         . body))]
+    [(_ . other)
+     (syntax-error "malformed doplist" (doplist . other))]))
+
 (define-syntax while
   (syntax-rules (=>)
     [(_ expr guard => var . body)
@@ -416,6 +541,8 @@
 ;;;-------------------------------------------------------------
 ;;; guard (srfi-34)
 
+(define %reraise (with-module gauche.internal %reraise))
+
 (define-syntax guard
   (syntax-rules ()
     [(guard (var . clauses) . body)
@@ -429,7 +556,8 @@
 (define-syntax %guard-rec
   (syntax-rules (else =>)
     [(%guard-rec var exc)
-     (raise exc)]
+     ;; exception handler can return to the caller
+     (%reraise exc)]
     [(%guard-rec var exc (else . exprs))
      (begin . exprs)]
     [(%guard-rec var exc (test => proc) . more)
@@ -446,14 +574,55 @@
 
 ;;;-------------------------------------------------------------
 ;;; unwind-protect
+;;;
 
+;; We set up exit-handler in the dynamic extent of BODY (but not in HANDLER),
+;; since if BODY calls exit, the error handlers won't be called---the dynamic
+;; environment is rewound upon exit, but that merely reset the error handlers.
+;;
+;; An alternative idea is to treat exit as if it's another kind of a condition,
+;; so that guard clauses are invoked.  We tried it, but the problem is how to
+;; deal with "ignore-errors" idiom, e.g. (guard (e [else #f]) body).  The exit
+;; condition shouldn't be stopped in such a way.
+;;
+;; TODO: Current definition doesn't work when unwind-protect is used
+;; within a thread that is terminated; thread termination isn't a condition
+;; either.
 (define-syntax unwind-protect
   (syntax-rules ()
     [(unwind-protect body handler ...)
-     (let ((h (lambda () handler ...)))
-       (receive r (guard (e (else (h) (raise e))) body)
-         (h)
-         (apply values r)))]
+     (let ([x (exit-handler)]
+           [h (lambda () handler ...)]
+           [done #f])
+       (with-error-handler
+           (lambda (e)
+             (exit-handler x)
+             (cond
+              [(condition-has-type? e <serious-condition>)
+               (unless done (set! done #t) (h))
+               ;; NB: We don't know E is thrown by r7rs#raise or
+               ;; r7rs#raise-continuable, but gauche#raise can handle both
+               ;; case.
+               (raise e)]
+              [else
+               ;; exception handler can return to the caller
+               (%reraise e)]))
+         (lambda ()
+           (receive r
+               (dynamic-wind
+                 (lambda ()
+                   (when done
+                     (error "Attempt to reenter obsoleted dynamic environment"))
+                   (exit-handler (lambda (code fmt args)
+                                   (set! done #t)
+                                   (h)
+                                   (x code fmt args))))
+                 (lambda () body)
+                 (lambda () (exit-handler x)))
+             (set! done #t)
+             (h)
+             (apply values r)))
+         :rewind-before #t))]
     [(unwind-protect . other)
      (syntax-error "malformed unwind-protect" (unwind-protect . other))]))
 
@@ -494,3 +663,25 @@
             [r (cond-list . rest)])
        (if tmp (cons (begin . expr) r) r))]
     ))
+
+;;;----------------------------------------------------------------
+;;; assume (srfi-145) and co.
+;;;
+
+;; We might add run-time optimization switch to expand assume to nothing.
+(define-syntax assume
+  (syntax-rules ()
+    [(_ expr)
+     (unless expr
+       (error (format "Invalid assumption: ~s" 'expr)))]
+    [(_ expr message ...)
+     (unless expr
+       (error (format "Invalid assumption: ~s:" 'expr) message ...))]))
+
+(define-syntax assume-type
+  (syntax-rules ()
+    [(_ expr type)
+     (let1 v expr
+       (unless (is-a? v type)
+         (type-error 'expr type v)))]
+    ))
