@@ -1,7 +1,7 @@
 ;;;
 ;;; condutil.scm - condition primitives.  autoloaded.
 ;;;
-;;;   Copyright (c) 2004-2014  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2004-2018  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -97,31 +97,42 @@
 ;; we extend srfi-35 to allow #f as predicate and accessors, as well as
 ;; omitting accessors.
 
-(define-macro (define-condition-type name super pred . field-specs)
-  (define (badfield-error field)
-    (error "bad field spec for define-condition-type:" field))
-  (define (scan-specs specs slots readers)
-    (match specs
-      [() (emit-defs slots readers)]
-      [((field #f) . rest)
-       (scan-specs rest (cons field slots) readers)]
-      [((field) . rest)
-       (scan-specs rest (cons field slots) readers)]
-      [((field reader) . rest)
-       (scan-specs rest (cons field slots)
-                   (cons `(define (,reader obj) (condition-ref obj ',field))
-                         readers))]
-      [_ (badfield-error (car specs))]))
-  (define (emit-defs slots readers)
-    `(begin
-       (define-class ,name (,super)
-         ,(map (^s `(,s :init-keyword ,(make-keyword s))) slots)
-         :metaclass <condition-meta>)
-       ,@readers
-       ,@(if pred
-           `((define (,pred obj) (condition-has-type? obj ,name)))
-           '())))
-  (scan-specs field-specs '() '()))
+(define-syntax define-condition-type
+  (er-macro-transformer
+   (^[f r c]
+     (match (cdr f)
+       [(name super pred . field-specs)
+        (define (badfield-error field)
+          (error "bad field spec for define-condition-type:" field))
+        (define (scan-specs specs slots readers)
+          (match specs
+            [() (emit-defs slots readers)]
+            [((field #f) . rest)
+             (scan-specs rest (cons field slots) readers)]
+            [((field) . rest)
+             (scan-specs rest (cons field slots) readers)]
+            [((field reader) . rest)
+             (scan-specs rest (cons field slots)
+                         (cons (quasirename r
+                                 (define (,reader obj)
+                                   (condition-ref obj ',field)))
+                               readers))]
+            [_ (badfield-error (car specs))]))
+        (define (emit-defs slots readers)
+          (quasirename r
+            (begin
+              (define-class ,name (,super)
+                ,(map (^s (quasirename r (,s :init-keyword ',(make-keyword s))))
+                      slots)
+                :metaclass <condition-meta>)
+              ,@readers
+              ,@(if pred
+                  (quasirename r
+                    ((define (,pred obj)
+                       (condition-has-type? obj ,name))))
+                  '()))))
+        (scan-specs field-specs '() '())]
+       ))))
 
 (define-syntax condition
   (syntax-rules ()
@@ -140,16 +151,6 @@
     ((condition-sub type inits (other . more))
      (syntax-error "malformed condition field initializer:" other))))
 
-;; A trick to allow slot-ref to be used for compound condition.
-(define-method slot-missing ((class <condition-meta>)
-                             (cc <compound-condition>)
-                             slot)
-  (let loop ((members (ref cc '%conditions)))
-    (cond ((null? members) (slot-missing class cc slot))
-          ((slot-exists? (car members) slot)
-           (slot-ref (car members) slot))
-          (else (loop (cdr members))))))
-
 ;; Aliases for srfi-35/srfi-36 compatibility
 (define &condition   <condition>)
 (define &message     <message-condition>)
