@@ -1,7 +1,7 @@
 ;;;
 ;;; procedure.scm - auxiliary procedure utilities.  to be autoloaded.
 ;;;
-;;;   Copyright (c) 2000-2012  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2000-2018  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -32,17 +32,17 @@
 ;;;
 
 (define-module gauche.procedure
-  (use srfi-1)
   (export compose .$ complement pa$ map$ for-each$ apply$
           count$ fold$ fold-right$ reduce$ reduce-right$
           filter$ partition$ remove$ find$ find-tail$
           any$ every$ delete$ member$ assoc$
           any-pred every-pred
-          applicable? arity procedure-arity-includes?
+          arity procedure-arity-includes?
           <arity-at-least> arity-at-least? arity-at-least-value
-          case-lambda ~ ref* disasm
+          source-code source-location disasm
           generator-fold generator-fold-right
           generator-for-each generator-map
+          generator-find
           ;; for the backward compatibility
           port-fold port-fold-right port-for-each port-map
           ))
@@ -52,16 +52,14 @@
 ;; Combinator utilities -----------------------------------------
 
 (define (pa$ fn . args)                  ;partial apply
-  (lambda more-args (apply fn (append args more-args))))
+  (^ more-args (apply fn (append args more-args))))
 
 (define (compose . fns)
-  (cond ((null? fns) values)
-        ((null? (cdr fns)) (car fns))
-        ((null? (cddr fns))
-         (lambda args
-           (call-with-values (lambda () (apply (cadr fns) args)) (car fns))))
-        (else
-         (compose (car fns) (apply compose (cdr fns))))))
+  (cond [(null? fns) values]
+        [(null? (cdr fns)) (car fns)]
+        [(null? (cddr fns))
+         (^ args (call-with-values (^[] (apply (cadr fns) args)) (car fns)))]
+        [else (compose (car fns) (apply compose (cdr fns)))]))
 
 (define .$ compose)                     ;experimental
 
@@ -69,10 +67,10 @@
 
 (define (complement fn)
   (case (arity fn) ;; some optimization
-    ((0) (lambda () (not (fn))))
-    ((1) (lambda (x) (not (fn x))))
-    ((2) (lambda (x y) (not (fn x y))))
-    (else (lambda args (not (apply fn args))))))
+    [(0) (^[] (not (fn)))]
+    [(1) (^[x] (not (fn x)))]
+    [(2) (^[x y] (not (fn x y)))]
+    [else (^ args (not (apply fn args)))]))
 
 (define (map$ proc)      (pa$ map proc))
 (define (for-each$ proc) (pa$ for-each proc))
@@ -81,13 +79,13 @@
 ;; partial evaluation version of srfi-1 procedures
 (define (count$ pred) (pa$ count pred))
 (define (fold$ kons . maybe-knil)
-  (lambda lists (apply fold kons (append maybe-knil lists))))
+  (^ lists (apply fold kons (append maybe-knil lists))))
 (define (fold-right$ kons . maybe-knil)
-  (lambda lists (apply fold-right kons (append maybe-knil lists))))
+  (^ lists (apply fold-right kons (append maybe-knil lists))))
 (define (reduce$ f . maybe-ridentity)
-  (lambda args (apply reduce f (append maybe-ridentity args))))
+  (^ args (apply reduce f (append maybe-ridentity args))))
 (define (reduce-right$ f . maybe-ridentity)
-  (lambda args (apply reduce-right f (append maybe-ridentity args))))
+  (^ args (apply reduce-right f (append maybe-ridentity args))))
 (define (filter$ pred) (pa$ filter pred))
 (define (partition$ pred) (pa$ partition pred))
 (define (remove$ pred) (pa$ remove pred))
@@ -101,22 +99,18 @@
 
 
 (define (any-pred . preds)
-  (lambda args
-    (let loop ((preds preds))
-      (cond ((null? preds) #f)
-            ((apply (car preds) args))
-            (else (loop (cdr preds)))))))
+  (^ args (let loop ([preds preds])
+            (cond [(null? preds) #f]
+                  [(apply (car preds) args)]
+                  [else (loop (cdr preds))]))))
 
 (define (every-pred . preds)
   (if (null? preds)
-      (lambda args #t)
-      (lambda args
-        (let loop ((preds preds))
-          (cond ((null? (cdr preds))
-                 (apply (car preds) args))
-                ((apply (car preds) args)
-                 (loop (cdr preds)))
-                (else #f))))))
+    (^ args #t)
+    (^ args (let loop ([preds preds])
+              (cond [(null? (cdr preds)) (apply (car preds) args)]
+                    [(apply (car preds) args) (loop (cdr preds))]
+                    [else #f])))))
 
 ;; Curryable procedures ------------------------------------
 
@@ -138,21 +132,6 @@
      (define name (curry-lambda formals . body))]))
 |#
 
-;; Applicability -------------------------------------------
-
-;; NB: This takes a list of classes.  But what if we support eqv-specilizer?
-;; One idea is to let the caller wrap a concrete instance.  We'll see...
-(define (applicable? proc . arg-types)
-  (define method-applicable?
-    (with-module gauche.object method-applicable-for-classes?))
-  (let1 c (class-of proc)
-    (cond [(eq? c <procedure>)
-           (procedure-arity-includes? proc (length arg-types))]
-          [(eq? c <generic>)
-           (any (^m (apply method-applicable? m arg-types)) (~ proc'methods))]
-          [(eq? c <method>)  (apply method-applicable? m arg-types)]
-          [else (apply applicable? object-apply c arg-types)])))
-
 ;; Procedure arity -----------------------------------------
 
 (define-class <arity-at-least> ()
@@ -169,9 +148,14 @@
 
 (define (arity proc)
   (cond [(or (is-a? proc <procedure>) (is-a? proc <method>))
-         (if (ref proc 'optional)
-           (make <arity-at-least> :value (ref proc 'required))
-           (ref proc 'required))]
+         (if-let1 clambda (case-lambda-info proc)
+           (map (^[info] (if (cadr info)
+                           (make <arity-at-least> :value (car info))
+                           (car info)))
+                clambda)
+           (if (ref proc 'optional)
+             (make <arity-at-least> :value (ref proc 'required))
+             (ref proc 'required)))]
         [(is-a? proc <generic>) (map arity (ref proc 'methods))]
         [else (errorf "cannot get arity of ~s" proc)]))
 
@@ -185,40 +169,54 @@
       (any check a)
       (check a))))
 
-;; ~, ref*  ----------------------------------------------------
-;;  (~ a b c d) => (ref (ref (ref a b) c) d)
-;;  TODO: This is here because case-lambda used to be here as well.
-;;  This should really belong to src/objlib.scm.  However, we can't
-;;  do that in 0.9.2, since src/objlib.scm needs to be complied with
-;;  0.9.1.  Move this after 0.9.2 release.
-(define ~
-  (getter-with-setter
-   (case-lambda
-     [(obj selector) (ref obj selector)]
-     [(obj selector . more) (apply ~ (ref obj selector) more)])
-   (case-lambda
-     [(obj selector val) ((setter ref) obj selector val)]
-     [(obj selector selector2 . rest)
-      (apply (setter ~) (ref obj selector) selector2 rest)])))
+;; source-code -------------------------------------------------
+;; Take the source code and location from a procedure.
+;;
+;; Where is source code saved?
+;;  (~ <compiled-code>'debug-info) -> definition -> source-info
+;;  In case if the source is a result of macro expansion, the original
+;;  source is attached in the 'original pair attribute of the source
+;;  code.  The `source-code' procedure traces the very origin of the source
+;;  (via compiled-code-definition).
+;;  Note that precompiled closure doesn't have `definition' info, hence
+;;  source code is not available.
+;;
+;; Where is source location saved?
+;;  For SUBRs and precompiled closures: The stub generator and precompiler
+;;  sets the procedure signature to the (~ <procedure>'info) slot.  The
+;;  signature has 'source-info attribute attached.
+;;  Other closures: They still have signature in 'info, but no source-info
+;;  is attached.  Instead you should take the source code and then look
+;;  at its source-info.
 
-(define ref* ~)                         ;for the backward compatibility
+(define (source-code proc)
+  (and (closure? proc)
+       ((with-module gauche.internal compiled-code-definition)
+        (closure-code proc))))
+
+(define (source-location proc)
+  (define (extract x)
+    (and (pair? x)
+         ($ (with-module gauche.internal pair-attribute-get)
+            x 'source-info #f)))
+  (or (extract (~ proc'info))
+      (extract (source-code proc))))
 
 ;; disassembler ------------------------------------------------
-;; I'm not sure whether this should be here or not, but fot the time being...
+;; I'm not sure whether this should be here or not, but for the time being...
 
 (define (disasm proc)
   (define dump (with-module gauche.internal vm-dump-code))
-  (define (dump-case-lambda min-reqargs proc-array)
+  (define (dump-case-lambda infos)
     (print "CASE-LAMBDA")
-    (let1 len (vector-length proc-array)
-      (do ([i 0 (+ i 1)])
-          ((= i len) #t)
-        (when (closure? (vector-ref proc-array i))
-          (if (= i (- len 1))
-            (print ";;; numargs >= " (+ i min-reqargs))
-            (print ";;; numargs = " (+ i min-reqargs)))
-          (dump (closure-code (vector-ref proc-array i)))))))
-
+    (dolist [info infos]
+      (apply [^(reqargs optarg proc)
+               (when (closure? proc)
+                 (if optarg
+                   (print ";;; numargs > " reqargs)
+                   (print ";;; numargs = " reqargs))
+                 (dump (closure-code proc)))]
+             info)))
   (cond
    [(closure? proc) (print "CLOSURE " proc) (dump (closure-code proc))]
    [(is-a? proc <method>)
@@ -231,17 +229,7 @@
       (print ">> method " m)
     (cond [(method-code m) => dump]
           [else (print "(defined in C)")]))]
-   [(and (subr? proc)
-         (let1 info (procedure-info proc)
-           ;; NB: Detect case-lamdba.  The format of procedure-info of
-           ;; case-lambda is tentative, and may be changed in future.
-           ;; See the comment of make-case-lambda-dispatcher in intlib.stub.
-           (and (pair? info)
-                (pair? (cdr info))
-                (integer? (cadr info))
-                (pair? (cddr info))
-                (vector? (caddr info))
-                (apply dump-case-lambda (cdr info)))))]
+   [(case-lambda-info proc) => dump-case-lambda]
    [else (print "Disassemble not applicable for " proc)])
   (values))
 
@@ -315,6 +303,12 @@
             (reverse r)
             (loop (cons (apply fn items) r))))))))
 
+(define (generator-find pred gen)
+  (let loop ([item (gen)])
+    (cond [(eof-object? item) #f]
+          [(pred item) item]
+          [else (loop (gen))])))
+
 ;; For the backward compatibility
 (define port-fold generator-fold)
 (define port-fold-right generator-fold-right)
