@@ -1,7 +1,7 @@
 ;;;
 ;;; process.scm - process interface
 ;;;
-;;;   Copyright (c) 2000-2016  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2000-2018  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -42,12 +42,14 @@
   (use srfi-13)
   (use srfi-14)
   (export <process> <process-abnormal-exit>
-          run-process process? process-alive? process-pid
+          run-process do-process do-process!
+          process? process-alive? process-pid
           process-command process-input process-output process-error
+          process-upstreams
           process-wait process-wait-any process-exit-status
           process-send-signal process-kill process-stop process-continue
           process-list
-          run-process-pipeline
+          run-pipeline do-pipeline
           ;; process ports
           open-input-process-port   open-output-process-port
           call-with-input-process   call-with-output-process
@@ -56,6 +58,8 @@
           process-output->string    process-output->string-list
           ;; shell utilities
           shell-escape-string shell-tokenize-string
+          ;; deprecated
+          run-process-pipeline
           ))
 (select-module gauche.process)
 
@@ -63,7 +67,7 @@
 (autoload gauche.charconv
           wrap-with-input-conversion wrap-with-output-conversion)
 (autoload gauche.uvector
-          write-block)
+          write-uvector)
 
 (define-class <process> ()
   ((pid       :init-value -1 :getter process-pid)
@@ -76,6 +80,9 @@
    (error     :allocation :virtual :slot-ref (^o (process-output o 2)))
    (extra-inputs  :initform '())
    (extra-outputs :initform '())
+   (upstreams :init-value '()) ; pipeline upstream #<process>es
+
+   ;; class slot - keep reference to all processes whose status is unclaimed
    (processes :allocation :class :initform '())
   ))
 
@@ -87,8 +94,15 @@
 ;; to an output port).
 ;; For the backward compatibility, (process-input p) and (process-output p)
 ;; returns pipes connected to child's stdin and stdout, respectively.
+;; NB: If P is the result of run-pipeline, (process-input p 'stdin) returns
+;; the input pipe to the head of the pipeline, not the input pipe of
+;; the process represented by P (which is, of course, connected to the
+;; previous process and not available).
 (define (process-input p :optional (name 'stdin))
-  (cond [(assv name (~ p'in-pipes)) => cdr] [else #f]))
+  (if (and (eq? name 'stdin)
+           (not (null? (~ p'upstreams))))
+    (process-input (car (~ p'upstreams))) ; the input of the whole pipeline
+    (cond [(assv name (~ p'in-pipes)) => cdr] [else #f])))
 
 (define (process-output p :optional (name 'stdout))
   (cond [(assv name (~ p'out-pipes)) => cdr] [else #f]))
@@ -116,6 +130,25 @@
     (%run-process-old command args) ;; backward compatibility
     (%run-process-new command args)))
 
+;; A typical use case---run process synchronously, returns #t for
+;; success, #f for failure.
+(define (do-process command . args)
+  (let* ([eflag (case (get-keyword :on-abnormal-exit args #f)
+                  [(#f) #f]
+                  [(:error) #t]
+                  [else => (cut error
+                                "Value for on-abnormal-exit argument \
+                                 must be either #f or :error, but got:" <>)])]
+         [p (apply run-process command (delete-keyword :on-abnormal-exit args))])
+    (process-wait p #f eflag)
+    (zero? (process-exit-status p))))
+
+;; Similar to do-process, but raise an error on abnormal exit.
+;; This use case is typical, so we add a separate API.
+(define (do-process! command . args)
+  (apply do-process command :on-abnormal-exit :error 
+         (delete-keyword :on-abnormal-exit args)))
+
 ;; Note: I/O redirection
 ;;  'Redirects' keyword argument is a generic way to wire child's I/Os.
 ;;  It takes a list of <io-spec>s, where each <io-spec> can be one of
@@ -236,9 +269,9 @@
                       :null, integer file descriptor or file input port, \
                       but got ~s for file descriptor ~s"
                      a2 (cadr arg)))]
-      [(<<) (unless (string? a2)
-              (errorf "input redirection '<<' requires a string, but got ~s \
-                       for file descriptor ~s" a2 (cadr arg)))]
+      [(<<) (unless (or (string? a2) (uvector? a2))
+              (errorf "input redirection '<<' requires a string or a uvector, \
+                       but got ~s for file descriptor ~s" a2 (cadr arg)))]
       [(<<<) #t]                        ;anything is ok
       [(> >>)
        (unless (or (string? a2) (symbol? a2) (eq? a2 :null) (integer? a2)
@@ -363,12 +396,13 @@
          (letrec ([write-arg (^o (unwind-protect
                                      (cond [(eq? dir '<<<) (write arg o)]
                                            [(string? arg) (display arg o)]
-                                           [else (write-block arg o)])
+                                           [else (write-uvector arg o)])
                                    (close-output-port o)))])
            (cond-expand
             [gauche.sys.threads
              (receive (in out) (sys-pipe)
                (push! iomap `(,fd . ,in))
+               (push! toclose in)
                (thread-start! (make-thread (cut write-arg out))))]
             [else
              ;; If we can't use threads, we fall back to using temporary
@@ -428,6 +462,8 @@
        (process-pid process)))
 (define (process-list) (class-slot-ref <process> 'processes))
 
+(define (process-upstreams p) (~ p'upstreams))
+
 ;;-----------------------------------------------------------------
 ;; wait
 ;;
@@ -440,6 +476,11 @@
              (slot-set! process 'processes
                         (delete process (slot-ref process 'processes)))
              (when raise-error? (%check-normal-exit process))
+             ;; If we're the end of the pipeline, salvage status of
+             ;; the upstream processes.  We won't raise error on non-zero
+             ;; exit in those processes, but we do care nohang? flag.
+             (dolist [p (~ process 'upstreams)]
+               (process-wait p nohang?))
              #t)))
     #f))
 
@@ -476,11 +517,9 @@
 ;; We might adopt scsh-like process forms eventually, but finding an
 ;; optimal DSL takes time.  Meanwhile, this intermediate-level API
 ;; would cover typical use case...
-(define (run-process-pipeline commands
-                              :key (input #f) (output #f) (error #f)
-                              (wait #f)
-                              (sigmask #f) (directory #f)
-                              (detached #f))
+(define (run-pipeline commands
+                      :key (input #f) (output #f) (error #f)
+                      (wait #f) (sigmask #f) (directory #f) (detached #f))
   (when (null? commands)
     (error "At least one command is required to run-command-pipeline"))
   (and-let1 offending (any (^c (and (not (pair? c)) (list c))) commands)
@@ -488,21 +527,40 @@
             (car offending) commands))
   (let* ([pipe-pairs (map (^_ (call-with-values sys-pipe cons))
                           (cdr commands))]
-         [cmds (map (^[cmdline in out w]
+         [cmds (map (^[cmdline in out]
                       `(,cmdline :input ,in :output ,out :error ,error
-                                 :wait ,w :sigmask ,sigmask
+                                 :sigmask ,sigmask
                                  :directory ,directory :detached ,detached))
                     commands
                     (cons input (map car pipe-pairs))
-                    (fold-right cons (list output) (map cdr pipe-pairs))
-                    `(,@(make-list (- (length commands) 1) #f) ,wait))]
+                    (fold-right cons (list output) (map cdr pipe-pairs)))]
          ;; We have to close output pipe after spawning the process, for
          ;; the process that reads from the pipe would wait until all the
          ;; ends are closed.  We have to treat the last command separately,
          ;; for we might :wait #t for it.
          [ps (map (cut apply run-process <>) (drop-right cmds 1))])
     (dolist [p pipe-pairs] (close-output-port (cdr p)))
-    (append ps (list (apply run-process (last cmds))))))
+    ;; Keep upstream processes in 'upstreams' slot.
+    (rlet1 p (apply run-process (last cmds))
+      (set! (~ p'upstreams) ps)
+      (when wait (process-wait p)))))
+
+(define (do-pipeline commands . args)
+  (let* ([eflag (case (get-keyword :on-abnormal-exit args #f)
+                  [(#f) #f]
+                  [(:error) #t]
+                  [else => (cut error
+                                "Value for on-abnormal-exit argument \
+                                 must be either #f or :error, but got:" <>)])]
+         [p (apply run-pipeline commands
+                   (delete-keyword :on-abnormal-exit args))])
+    (process-wait p #f eflag)
+    (zero? (process-exit-status p))))
+
+;; For the backward compatiblity.  DEPRECATED.
+(define (run-process-pipeline . args) ; returns list of processes.
+  (let1 p (apply run-pipeline args)
+    (append (~ p'upstreams) (list p))))
 
 ;;===================================================================
 ;; Process ports
@@ -518,13 +576,13 @@
 (define (open-input-process-port command :key (input *nulldev*)
                                  ((:error err) #f) (host #f)
                                  :allow-other-keys rest)
-  (let1 p (%apply-run-process command input :pipe err host)
+  (let1 p (%apply-run-process command input :pipe err host rest)
     (values (wrap-input-process-port p rest) p)))
 
 (define (call-with-input-process command proc :key (input *nulldev*)
                                  ((:error err) #f) (host #f) (on-abnormal-exit :error)
                                  :allow-other-keys rest)
-  (let* ((p (%apply-run-process command input :pipe err host))
+  (let* ((p (%apply-run-process command input :pipe err host rest))
          (i (wrap-input-process-port p rest)))
     (unwind-protect (proc i)
       (begin
@@ -540,14 +598,14 @@
 (define (open-output-process-port command :key (output *nulldev*)
                                   ((:error err) #f) (host #f)
                                   :allow-other-keys rest)
-  (let1 p (%apply-run-process command :pipe output err host)
+  (let1 p (%apply-run-process command :pipe output err host rest)
     (values (wrap-output-process-port p rest) p)))
 
 (define (call-with-output-process command proc :key (output *nulldev*)
                                   ((:error err) #f) (host #f)
                                   (on-abnormal-exit :error)
                                   :allow-other-keys rest)
-  (let* ((p (%apply-run-process command :pipe output err host))
+  (let* ((p (%apply-run-process command :pipe output err host rest))
          (o (wrap-output-process-port p rest)))
     (unwind-protect (proc o)
       (begin
@@ -563,7 +621,7 @@
 (define (call-with-process-io command proc :key ((:error err) #f)
                               (host #f) (on-abnormal-exit :error)
                               :allow-other-keys rest)
-  (let* ((p (%apply-run-process command :pipe :pipe err host))
+  (let* ((p (%apply-run-process command :pipe :pipe err host rest))
          (i (wrap-input-process-port p rest))
          (o (wrap-output-process-port p rest)))
     (unwind-protect (proc i o)
@@ -651,8 +709,8 @@
           (let1 c (peek-char)
             (cond [(eof-object? c) (list c0)]
                   [(char-whitespace? c) (skip-ws) (list c0)]
-                  [(eqv? c #\') (cons c0 (read-sq))]
-                  [(eqv? c #\") (cons c0 (read-dq))]
+                  [(eqv? c #\') (read-char) (cons c0 (read-sq))]
+                  [(eqv? c #\") (read-char) (cons c0 (read-dq))]
                   [(eqv? c #\\) (read-char) (cons c0 (read-escaped))]
                   [(#[|&\;<>()$`] c) (err-meta c)]
                   [else (read-char) (cons c0 (read-unquoted c))])))
@@ -741,21 +799,22 @@
 ;; rule is too broken to use reliably.  Another possibility is to implement
 ;; much of high-level /bin/sh functionalities in Scheme, so that we can
 ;; provide consistent behavior.  Something to think about.
-(define (%apply-run-process command stdin stdout stderr host)
+(define (%apply-run-process command stdin stdout stderr host opts)
   (apply run-process
          (cond [(string? command)
-                (cond-expand [gauche.os.windows `("cmd" "/c" ,command)]
-                             [else        `("/bin/sh" "-c" ,command)])]
+                (cond-expand [gauche.os.windows `("cmd.exe" "/c" ,command)]
+                             [else              `("/bin/sh" "-c" ,command)])]
                [(list? command) command]
                [else (error "Bad command spec" command)])
          :input stdin :output stdout :host host
-         (cond [(string? stderr) `(:error ,stderr)]
-               [else '()])))
+         (cond [(string? stderr) `(:error ,stderr ,@opts)]
+               [else opts])))
 
 ;; Possibly wrap the process port by a conversion port
 (define (wrap-input-process-port process opts)
   (let-keywords opts ([encoding #f]
-                      [conversion-buffer-size 0])
+                      [conversion-buffer-size 0]
+                      . opts)
     (if encoding
       (wrap-with-input-conversion (process-output process) encoding
                                   :buffer-size conversion-buffer-size)
@@ -763,7 +822,8 @@
 
 (define (wrap-output-process-port process opts)
   (let-keywords opts ([encoding #f]
-                      [conversion-buffer-size 0])
+                      [conversion-buffer-size 0]
+                      . opts)
     (if encoding
       (wrap-with-output-conversion (process-input process) encoding
                                   :buffer-size conversion-buffer-size)
