@@ -1,7 +1,7 @@
 ;;;
 ;;; http.scm - HTTP 1.1
 ;;;
-;;;   Copyright (c) 2000-2014  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2000-2018  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -44,8 +44,6 @@
 ;; The features required for typical client usage are implemented first.
 
 (define-module rfc.http
-  (use srfi-1)
-  (use srfi-2)
   (use srfi-11)
   (use srfi-13)
   (use rfc.822)
@@ -57,11 +55,11 @@
   (use gauche.sequence)
   (use gauche.uvector)
   (use util.match)
-  (use util.list)
   (use text.tree)
   (export <http-error>
           http-user-agent make-http-connection reset-http-connection
           http-compose-query http-compose-form-data
+          http-status-code->description
 
           http-proxy http-request
           http-null-receiver http-string-receiver http-oport-receiver
@@ -85,7 +83,11 @@
           mime-compose-parameters
           mime-parse-content-type)
 (autoload rfc.tls
-          make-tls tls-connect tls-input-port tls-output-port tls-close)
+          make-tls tls-destroy tls-connect tls-input-port
+          tls-output-port tls-close)
+(autoload gauche.process
+          run-process process-input process-output process-error
+          process-wait process-kill)
 
 (autoload file.util file-size find-file-in-paths null-device)
 
@@ -101,14 +103,13 @@
 
 ;; default string to be used for user-agent.
 (define http-user-agent
-  (make-parameter #`"gauche.http/,(gauche-version)"))
+  (make-parameter #"gauche.http/~(gauche-version)"))
 
 ;; global proxy settings.  can be overridden by :proxy keyword
 ;; argument.
 (define http-proxy (make-parameter #f))
 
 ;; The default redirect handler
-;;
 (define http-default-redirect-handler
   (make-parameter
    (^[method code headers body]
@@ -227,7 +228,6 @@
                            auth-user
                            auth-password
                            (proxy (http-proxy))
-                           extra-headers
                            (user-agent (http-user-agent))
                            (secure #f)
                            (receiver (http-string-receiver))
@@ -235,6 +235,12 @@
                            ((:request-encoding enc) (gauche-character-encoding))
                       :allow-other-keys opts)
 
+  (define extra-headers
+    ($ concatenate $ reverse
+       $ rlet1 z '()
+       (doplist [(k v) opts]
+                (when v (push! z (list k v))))))
+
   (define conn (ensure-connection server auth-handler auth-user auth-password
                                   proxy secure extra-headers))
   (define redirector (if no-redirect
@@ -243,7 +249,6 @@
                          [(#t) (http-default-redirect-handler)]
                          [(#f) #f]
                          [else => identity])))
-  (define options `(:user-agent ,user-agent ,@(http-auth-headers conn) ,@opts))
   (define no-body-replies '("204" "304"))
 
   (define (get-body iport method code headers receiver)
@@ -257,16 +262,16 @@
                                         '(:proxy-connection keep-alive)
                                         '(:connection keep-alive))]
                [#t @ `(:host ,host :user-agent ,user-agent
-                       ,@(http-auth-headers conn) ,@opts)]))
+                       ,@(http-auth-headers conn) ,@extra-headers)]))
 
   ;; If we decide to give up redirection, we read from already-retrieved
   ;; body of 3xx reply.  This modifies reply headers if necessary.
   (define (redirect-headers body rep-headers)
     (if body
-      `(:content-length ,(string-size body)
-                        ,@(delete-keywords '(:content-length
-                                             :content-transfer-encoding)
-                                           rep-headers))
+      `(("content-length" ,(string-size body))
+        ,@(remove (^p (member (car p) '("content-length"
+                                        "content-transfer-encoding")))
+                  rep-headers))
       rep-headers))
 
   ;; returns either one of:
@@ -280,9 +285,11 @@
         (let* ([body (get-body in method code rep-headers
                                (http-string-receiver))]
                [verdict (consider-redirect method code rep-headers body)])
+          ;; consider-redirect returns either #f (don't redirect) or
+          ;; (METHOD . LOCATION).
           (if verdict
             `(redirect-to ,(car verdict) ,(cdr verdict))
-            (let1 hdrs (redirect-headers body rep-headers)
+            (let1 hdrs (redirect-headers body rep-headers) ;giving up
               `(reply ,code ,hdrs
                       ,(and body
                             (receive-body (open-input-string body) code
@@ -313,6 +320,7 @@
              (loop (cons uri history)
                    (~ (redirect-connection! conn proto new-server)'server)
                    method path*))])))))
+
 ;;
 ;; Pre-defined receivers
 ;;
@@ -430,7 +438,7 @@
       (let* ([size (string-size body)]
              [hdrs `(("content-length" ,(x->string size))
                      ("mime-version" "1.0")
-                     ("content-type" ,#`"multipart/form-data; boundary=\",boundary\"")
+                     ("content-type" ,#"multipart/form-data; boundary=\"~|boundary|\"")
                      ,@(alist-delete "content-type" hdrs equal?))]
              [body-sink (header-sink hdrs)]
              [port (body-sink size)])
@@ -492,15 +500,14 @@
    (socket :init-value #f)              ; A <socket> for persistent connection.
                                         ; If it is shutdown by the server,
                                         ; the APIs attempt to reconnect.
-   (secure-agent  :init-value #f)       ; When using secure connection via
-                                        ; tls, this slot holds its handle.
+   (secure-agent  :init-value #f)       ; See "secure agent handling" below
    (persistent    :init-keyword :persistent)   ; true for persistent connection.
    (auth-handler  :init-keyword :auth-handler  :init-value #f)
    (auth-user     :init-keyword :auth-user     :init-value #f)
    (auth-password :init-keyword :auth-password :init-value #f)
    (proxy         :init-keyword :proxy)
    (extra-headers :init-keyword :extra-headers)
-   (secure        :init-keyword :secure) ; boolean
+   (secure        :init-keyword :secure) ; either #f, tls or stunnel
    ))
 
 (define (make-http-connection server :key
@@ -522,15 +529,21 @@
 ;; This modifies CONN.
 (define (redirect-connection! conn proto new-server)
   (let1 orig-server (~ conn'server)
+    ;; Unless we're connecting the same server with same secure level,
+    ;; we reset the connection.
     (unless (and (string=? orig-server new-server)
-                 (eq? (~ conn'secure) (equal? proto "https")))
+                 (or (and (equal? proto "https") (~ conn'secure))
+                     (and (not (equal? proto "https")) (not (~ conn'secure)))))
       (shutdown-secure-agent conn)
       (and-let* ([s (~ conn'socket)])
         (socket-shutdown s)
         (socket-close s)
         (set! (~ conn'socket) #f))
       (set! (~ conn'server) new-server)
-      (set! (~ conn'secure) (equal? proto "https"))))
+      ;; NB: If the server redirects http connection to https, we don't have
+      ;; a chance for the caller to select alternative TLS flavor---for the
+      ;; time being we blindly assume tls.
+      (set! (~ conn'secure) (and (equal? proto "https") 'tls))))
   conn)
 
 ;;==============================================================
@@ -549,12 +562,12 @@
   (define (esc s) (uri-encode-string (x->string s) :encoding encoding))
   (define (query-1 n&v)
     (match n&v
-      [(name value) #`",(esc name)=,(esc value)"]
+      [(name value) #"~(esc name)=~(esc value)"]
       [_ (error "Invalid request-uri form:" params)]))
   (define (query) (string-concatenate (intersperse "&" (map query-1 params))))
   (cond [(not path) (query)]
         [(null? params) path]
-        [else #`",|path|?,(query)"]))
+        [else #"~|path|?~(query)"]))
 
 ;; multipart/form-data composition [RFC2388]
 ;; <params> : (<param> ...)
@@ -602,6 +615,61 @@
     (mime-compose-message-string (map translate-param params))
     (mime-compose-message (map translate-param params) port)))
 
+;;==============================================================
+;; status codes
+;;
+
+(define *status-code-map*
+  (hash-table 'eqv?
+              '(100 . "Continue")
+              '(101 . "Switching Protocols")
+              '(200 . "OK")
+              '(201 . "Created")
+              '(202 . "Accepted")
+              '(203 . "Non-Authoritative Information")
+              '(204 . "No Content")
+              '(205 . "Reset Content")
+              '(206 . "Partial Content")
+              '(300 . "Multiple Choices")
+              '(301 . "Moved Permanently")
+              '(302 . "Found")
+              '(303 . "See Other")
+              '(304 . "Not Modified")
+              '(305 . "Use Proxy")
+              '(306 . "(Unused)")
+              '(307 . "Temporary Redirect")
+              '(400 . "Bad Request")
+              '(401 . "Unauthorized")
+              '(402 . "Payment Required")
+              '(403 . "Forbidden")
+              '(404 . "Not Found")
+              '(405 . "Method Not Allowed")
+              '(406 . "Not Acceptable")
+              '(407 . "Proxy Authentication Required")
+              '(408 . "Request Timeout")
+              '(409 . "Conflict")
+              '(410 . "Gone")
+              '(411 . "Length Required")
+              '(412 . "Precondition Failed")
+              '(413 . "Request Entity Too Large")
+              '(414 . "Request-URI Too Long")
+              '(415 . "Unsupported Media Type")
+              '(416 . "Requested Range Not Satisfiable")
+              '(417 . "Expectation Failed")
+              '(500 . "Internal Server Error")
+              '(501 . "Not Implemented")
+              '(502 . "Bad Gateway")
+              '(503 . "Service Unavailable")
+              '(504 . "Gateway Timeout")
+              '(505 . "HTTP Version Not Supported")
+              ))
+
+;; API
+;; code can be an integer or a string, e.g. "200"
+;; returns #f for unknown code
+(define (http-status-code->description code)
+  (hash-table-get *status-code-map* (x->integer code) #f))
+
 ;;==============================================================
 ;; internal utilities
 ;;
@@ -612,18 +680,6 @@
     [(path n&v ...) (http-compose-query path n&v enc)]
     [_ (error "Invalid request-uri form for http request API:" request-uri)]))
 
-(define (canonical-body request-body extra-headers enc)
-  (cond [(not request-body) (values #f extra-headers)]
-        [(string? request-body) (values request-body extra-headers)]
-        [(list? request-body)
-         (receive (body boundary) (http-compose-form-data request-body #f enc)
-           (values body
-                   `(:mime-version "1.0"
-                     :content-type
-                     ,#`"multipart/form-data; boundary=,boundary"
-                     ,@(delete-keyword! :content-type extra-headers))))]
-        [else (error "Invalid request-body format:" request-body)]))
-
 ;; Always returns a connection object.
 (define (ensure-connection server auth-handler auth-user auth-password
                            proxy secure extra-headers)
@@ -641,19 +697,36 @@
       (check-override auth-password)
       (check-override proxy)
       (check-override extra-headers)
-      (check-override secure))))
+      (set! (~ conn'secure)
+            (and (not (undefined? secure))
+                 (case secure
+                   [(#f) #f]
+                   [(#t tls) 'tls]
+                   [(stunnel) 'stunnel]
+                   [else (error "`secure' argument for http connection must be \
+                                 either boolean, 'tls or 'stunnel, but got:"
+                                secure)]))))))
 
 (define (reset-http-connection conn)
   (shutdown-socket-connection conn)
   (shutdown-secure-agent conn))
 
 (define (start-socket-connection conn)
-  (let1 server (or (~ conn'proxy) (~ conn'server))
+  ;; If address is given ipv6 format such as "[::1]:port", we have to
+  ;; use "::1" part as the hostname, excluding [].
+  (define (parse-address addr)
+    (rxmatch-case addr
+      [#/^unix:(\/.*)$/ (_ path) (values #f #f path)] ;unix domain
+      [#/^\[([a-fA-F\d:]+)\](?::(\d+))?$/ (_ host port) (values host port #f)]
+      [#/^([^:]+)(?::(\d+))?$/ (_ host port) (values host port #f)]
+      [else (error "Unrecognized http server address:" addr)]))
+  (receive (host port path) (parse-address (or (~ conn'proxy) (~ conn'server)))
     (set! (~ conn'socket)
-          (cond [(#/([^:]+):(\d+)/ server)
-                 => (^m (make-client-socket (m 1) (x->integer (m 2))))]
-                [else
-                 (make-client-socket server (if (~ conn'secure) 443 80))]))))
+          (if path
+            (make-client-socket 'unix path)
+            (make-client-socket host (if port
+                                       (x->integer port)
+                                       (if (~ conn'secure) 443 80)))))))
 
 (define (shutdown-socket-connection conn)
   (when (~ conn'socket)
@@ -663,15 +736,16 @@
     (set! (~ conn'socket) #f)))
 
 (define (with-connection conn proc)
-  (unless (~ conn'persistent)
-    (unless (~ conn'socket) (start-socket-connection conn))
-    (when (~ conn'secure) (start-secure-agent conn)))
   (unwind-protect
-      (apply proc (if (~ conn'secure)
-                    `(,(tls-input-port (~ conn'secure-agent))
-                      ,(tls-output-port (~ conn'secure-agent)))
-                    `(,(socket-input-port (~ conn'socket))
-                      ,(socket-output-port (~ conn'socket)))))
+      (begin
+        (unless (~ conn'persistent)
+          (unless (~ conn'socket) (start-socket-connection conn))
+          (when (~ conn'secure) (start-secure-agent conn)))
+        (apply proc (if (~ conn'secure)
+                      `(,(secure-agent-input-port conn)
+                        ,(secure-agent-output-port conn))
+                      `(,(socket-input-port (~ conn'socket))
+                        ,(socket-output-port (~ conn'socket))))))
     (unless (~ conn'persistent)
       (when (~ conn'secure) (shutdown-secure-agent conn))
       (shutdown-socket-connection conn))))
@@ -703,7 +777,7 @@
 
 ;; send
 (define (send-request out method uri sender headers enc)
-  (define request-line #`",method ,uri HTTP/1.1\r\n")
+  (define request-line #"~method ~uri HTTP/1.1\r\n")
   (define request-headers
     ($ map (cut map (^s (if (keyword? s) (keyword->string s) (x->string s))) <>)
        $ slices headers 2))
@@ -801,25 +875,99 @@
 ;; secure agent handling
 ;;
 
+;; We provide two options to handle secure connection.
+;;  tls     - using Gauche's rfc.tls module (default)
+;;  stunnel - using external stunnel process
+
 (define (shutdown-secure-agent conn)
   (when (~ conn'secure-agent)
-    (tls-close (~ conn'secure-agent))
-    (tls-destroy (~ conn'secure-agent))
+    (ecase (~ conn'secure)
+      [(tls)
+       (tls-close (~ conn'secure-agent))
+       (tls-destroy (~ conn'secure-agent))]
+      [(stunnel)
+       (close-output-port (process-input (~ conn'secure-agent)))
+       (or (process-wait (~ conn'secure-agent) #t)
+           (begin (sys-nanosleep #e1e8)    ;0.1s
+                  (process-wait (~ conn'secure-agent) #t))
+           (begin (sys-nanosleep #e1e8)    ;0.1s
+                  (process-kill (~ conn'secure-agent))))])
     (set! (~ conn'secure-agent) #f)))
 
 (define (start-secure-agent conn)
-  (unless (http-secure-connection-available?)
-    (error "Secure connection is not available on this platform"))
   (when (~ conn'secure-agent) (shutdown-secure-agent conn))
-  (let1 tls (make-tls)
-    (tls-connect tls (socket-fd (~ conn'socket)))
-    (set! (~ conn'secure-agent) tls)))
+  (ecase (~ conn'secure)
+    [(tls)
+     (let1 tls (make-tls :server-name (~ conn'server))
+       (set! (~ conn'secure-agent) tls)
+       (tls-connect tls (socket-fd (~ conn'socket))))]
+    [(stunnel)
+     (let* ([rhost      (or (~ conn'proxy) (~ conn'server))]
+            [rhost:port (if (string-index rhost #\:) rhost #"~|rhost|:https")])
+       (if-let1 p ((probe-stunnel) rhost:port)
+         (set! (~ conn'secure-agent) p)
+         (error "Couldn't start stunnel subprocess")))]))
+
+(define (secure-agent-input-port conn)
+  (ecase (~ conn'secure)
+    [(tls)     (tls-input-port (~ conn'secure-agent))]
+    [(stunnel) (process-output (~ conn'secure-agent))]))
+
+(define (secure-agent-output-port conn)
+  (ecase (~ conn'secure)
+    [(tls)     (tls-output-port (~ conn'secure-agent))]
+    [(stunnel) (process-input (~ conn'secure-agent))]))
 
 ;; for external api
-(define (http-secure-connection-available?)
-  (cond-expand
-   [gauche.net.tls #t]
-   [else #f]))
+(define (http-secure-connection-available? :optional (type #t))
+  (case type
+    [(#t tls)
+     (cond-expand
+      [gauche.net.tls #t]
+      [else #f])]
+    [(stunnel)
+     (boolean (probe-stunnel))]
+    [else
+     (error "Unknown secure connection type (must be tls, stunnel or #t):"
+            type)]))
+
+(define probe-stunnel
+  (let1 result #f
+    (define (run-stunnel3 path)
+      (lambda (host:port)
+        (run-process `(,path "-c" "-r" ,host:port) :input :pipe :output :pipe
+                     :error :null :wait #f)))
+    (define (run-stunnel4 path)
+      (lambda (host:port)
+        (receive (in out) (sys-pipe)
+          (rlet1 p (run-process `(,path "-fd" 3)
+                                :redirects `((< 0 stdin)
+                                             (> 1 stdout)
+                                             (> 2 :null)
+                                             (< 3 ,(port-file-number in)))
+                                :wait #f)
+            (format out "client = yes\n")
+            (format out "connect = ~a" host:port)
+            (close-output-port out)))))
+
+    (lambda (:key (force #f))
+      (if (and result (not force))
+        result
+        (and-let* ([path (or (find-file-in-paths "stunnel4") ;ubuntu has this
+                             (find-file-in-paths "stunnel"))]
+                   [p (run-process `(,path "-version")
+                                   :error :pipe :output :pipe)]
+                   [vers (read-line (process-error p))])
+          (process-wait p)
+          (rlet1 proc (cond
+                       [(eof-object? vers) (run-stunnel3 path)]
+                       [(#/stunnel (\d+)\.\d+/ vers)
+                        => (^m (if (>= (x->integer (m 1)) 4)
+                                 (run-stunnel4 path)
+                                 (run-stunnel3 path)))]
+                       [(#/exec failed/i vers) #f]
+                       [else (run-stunnel3 path)])
+            (set! result proc)))))))
 
 ;;==============================================================
 ;; authentication handling
@@ -834,7 +982,7 @@
   (and-let* ([user (~ conn 'auth-user)]
              [pass (or (~ conn 'auth-password) "")])
     `(:authorization ,($ format "Basic ~a"
-                         $ base64-encode-string #`",|user|:,|pass|"))))
+                         $ base64-encode-string #"~|user|:~|pass|"))))
 
 (define http-default-auth-handler
   (make-parameter http-basic-auth-handler))
