@@ -1,7 +1,7 @@
 ;;;
 ;;; cgi.scm - CGI utility
 ;;;
-;;;   Copyright (c) 2000-2014  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2000-2018  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -394,14 +394,15 @@
   (let ([ct (or content-type
                 (and (not location) "text/html"))]
         [r '()])
-    (when status   (push! r #`"Status: ,status\r\n"))
-    (when ct       (push! r #`"Content-type: ,ct\r\n"))
-    (when location (push! r #`"Location: ,location\r\n"))
+    (when status   (push! r #"Status: ~status\r\n"))
+    (when ct       (push! r #"Content-type: ~ct\r\n"))
+    (when location (push! r #"Location: ~location\r\n"))
     (dolist [cookie cookies]
-      (push! r #`"Set-cookie: ,cookie\r\n"))
+      (push! r #"Set-cookie: ~cookie\r\n"))
     (dolist [p (slices rest 2)]
       (when (pair? (cdr p))
-        (push! r #`",(car p): ,(cadr p)\r\n")))
+        (let1 hdrname (if (keyword? (car p)) (keyword->string (car p)) (car p))
+          (push! r #"~|hdrname|: ~(cadr p)\r\n"))))
     (push! r "\r\n")
     (reverse r)))
 
