@@ -1,7 +1,7 @@
 ;;;
 ;;; gauche.generator - generator framework
 ;;;
-;;;   Copyright (c) 2011-2016  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2011-2018  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -47,11 +47,13 @@
           generator->list generator->reverse-list
           generator->vector generator->vector!
           generator->string
+          generator->uvector generator->uvector!
+          generator->bytevector generator->bytevector!
           generator-any generator-every generator-unfold
-          generator-count
+          generator-count generator-map->list
 
           null-generator gcons* gappend gflatten
-          gconcatenate gmerge
+          gconcatenate gmerge ggroup
           circular-generator gunfold giota grange gindex gselect ginterval
           gmap gmap-accum gfilter gremove gdelete gdelete-neighbor-dups
           gfilter-map gstate-filter gbuffer-filter
@@ -61,7 +63,6 @@
           ;; srfi-121 compatibility
           generator make-iota-generator make-range-generator
           make-coroutine-generator bytevector->generator
-          make-bits-generator
           make-for-each-generator make-unfold-generator gcombine
           ))
 (select-module gauche.generator)
@@ -82,7 +83,11 @@
     [(_ exp body ...) (rlet1 tmp exp body ...)]))
 
 ;; Avoid circular dependency during build
-(autoload gauche.uvector uvector-ref u8vector?)
+(autoload gauche.uvector uvector-ref uvector-set! uvector-length
+          make-uvector u8vector?
+          <u8vector> <s8vector> <u16vector> <s16vector>
+          <u32vector> <s32vector> <u64vector> <s64vector>
+          <f16vector> <f32vector> <f64vector>)
 
 ;;;
 ;;; Converters and constructors
@@ -345,9 +350,9 @@
                   (begin0 e2 (set! e2 (gen2))))
                 (if (eof-object? e2)
                   (begin0 e1 (set! e1 (gen1)))
-                  (if (prec e1 e2)
-                    (begin0 e1 (set! e1 (gen1)))
-                    (begin0 e2 (set! e2 (gen2)))))))))]
+                  (if (prec e2 e1)
+                    (begin0 e2 (set! e2 (gen2)))
+                    (begin0 e1 (set! e1 (gen1)))))))))]
     [(prec . gens)
      (apply gmerge prec (map (cut apply gmerge prec <>) (slices gens 2)))]))
 
@@ -666,6 +671,33 @@
         (cond [(eof-object? v) (- k at)]
               [else (vector-set! vec k v) (loop (+ k 1))])))))
 
+(define (generator->uvector gen :optional (n #f) (class <u8vector>))
+  (define (gather i r)
+    (if (eqv? i n)
+      (values r i)
+      (let1 v (gen)
+        (if (eof-object? v)
+          (values r i)
+          (gather (+ i 1) (cons v r))))))
+  (receive (r k) (gather 0 '())
+    (rlet1 vec (make-uvector class k)
+      (do ([i (- k 1) (- i 1)]
+           [r r       (cdr r)])
+          [(null? r)]
+        (uvector-set! vec i (car r))))))
+(define (generator->bytevector gen :optional (n #f))
+  (generator->uvector gen n <u8vector>))
+
+(define (generator->uvector! vec at gen)
+  (let1 len (uvector-length vec)
+    (let loop ([k at])
+      (let1 v (if (>= k len) (eof-object) (gen))
+        (cond [(eof-object? v) (- k at)]
+              [else (uvector-set! vec k v) (loop (+ k 1))])))))
+(define (generator->bytevector! vec at gen)
+  (check-arg u8vector? vec)
+  (generator->uvector! vec at gen))
+  
 (define (generator->string gen :optional (n #f))
   (with-output-to-string
     (^[]
@@ -714,12 +746,20 @@
 (define (bytevector->generator bv :optional (start 0) (end (uvector-length bv)))
   (unless (u8vector? bv) (error "u8vector required, but got:" bv))
   (uvector->generator bv start end))
-(define (make-bits-generator n) (bits->generator n))
 (define (make-for-each-generator for-each coll)
   (generate (^[yield] (for-each yield coll))))
 (define (make-unfold-generator p f g seed) (gunfold p f g seed))
 (define (gcombine proc seed gen . gens) (apply gmap-accum proc seed gen gens))
 
+;; srfi-158 auxiliary procedures
+(define (ggroup gen k :optional padding)
+  (if (undefined? padding)
+    (gslices gen k)
+    (gslices gen k #t padding)))
+
+(define (generator-map->list proc gen . more)
+  (generator->list (apply gmap proc gen more)))
+
 ;; TODO:
 ;;  (gen-ec (: i ...) ...)
 ;;    srfi-42-ish macro to create generator.  it's not "eager", so
