@@ -16,39 +16,29 @@
 ;; which is autoloaded.  The user doesn't need to see this change.
 
 (define-module srfi-1
-  (export xcons list-tabulate circular-list
-          not-pair?
-          list=
-          first second third fourth fifth sixth seventh eighth
-          ninth tenth car+cdr take-right drop-right
-          take! drop-right!
-          zip unzip1 unzip2 unzip3 unzip4 unzip5
-          unfold pair-fold unfold-right
-          pair-fold-right
-          map! pair-for-each map-in-order
-          partition!
-          list-index
-          take-while drop-while take-while! span break span! break!
-          alist-cons
+  ;; The export list includes the procedures defined in #<module gauche>
+  ;; in order to make (import (srfi 1)) work for r7rs programs.
+  (export xcons cons* make-list list-tabulate list-copy circular-list iota
+          proper-list? circular-list? dotted-list?
+          not-pair? null-list? list=
+          first second third fourth fifth sixth seventh eighth ninth tenth
+          car+cdr take drop take-right drop-right
+          take! drop-right! split-at split-at! last last-pair
+          length+ concatenate reverse append! concatenate! reverse!
+          append-reverse append-reverse!
+          zip unzip1 unzip2 unzip3 unzip4 unzip5 count
+          fold unfold pair-fold reduce fold-right unfold-right
+          pair-fold-right reduce-right append-map append-map!
+          map! pair-for-each filter-map map-in-order
+          filter partition remove filter! partition! remove!
+          find find-tail any every list-index take-while drop-while
+          take-while! span break span! break!
+          delete delete-duplicates delete! delete-duplicates!
+          alist-cons alist-copy alist-delete alist-delete!
           lset<= lset= lset-adjoin lset-union lset-union!
           lset-intersection lset-intersection! lset-difference
           lset-difference! lset-xor lset-xor!
           lset-diff+intersection lset-diff+intersection!
-
-          ;; These are defined in gauche module, but they're not exported.
-          ;; We need to export them here so that (use srfi-1 :only (fold))
-          ;; will work.
-          null-list? cons* last member
-          take drop take-right drop-right take! drop-right!
-          delete delete! delete-duplicates delete-duplicates!
-          assoc alist-copy alist-delete alist-delete!
-          any every filter filter! remove remove! filter-map
-          fold fold-right count reduce reduce-right
-          append-reverse append-reverse!
-          concatenate concatenate!
-          append-map append-map!
-          find find-tail
-          split-at split-at! partition iota
           ))
 (select-module srfi-1)
 
