@@ -1,7 +1,7 @@
 /*
  * core.c - core kernel interface
  *
- *   Copyright (c) 2000-2016  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2000-2018  Shiro Kawai  <shiro@acm.org>
  *
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -98,6 +98,7 @@ extern void Scm_Init_libfmt(void);
 extern void Scm_Init_libio(void);
 extern void Scm_Init_liblazy(void);
 extern void Scm_Init_liblist(void);
+extern void Scm_Init_libmac(void);
 extern void Scm_Init_libmisc(void);
 extern void Scm_Init_libmod(void);
 extern void Scm_Init_libnum(void);
@@ -184,6 +185,7 @@ void Scm_Init(const char *signature)
     Scm_Init_libio();
     Scm_Init_liblazy();
     Scm_Init_liblist();
+    Scm_Init_libmac();
     Scm_Init_libmisc();
     Scm_Init_libmod();
     Scm_Init_libnum();
@@ -513,12 +515,21 @@ init_cond_features()
 
         /* TLS/SSL.  This nees to be in the core in order to switch
            code _before_ loading rfc.tls */
+#if defined(GAUCHE_USE_AXTLS) || defined(GAUCHE_USE_MBEDTLS)
+        { "gauche.net.tls", "rfc.tls" },
+#endif
 #if defined(GAUCHE_USE_AXTLS)
-        { "gauche.net.tls", "rfc.tls" },
         { "gauche.net.tls.axtls", "rfc.tls" },
-#elif defined(GAUCHE_USE_OPENSSL)
-        { "gauche.net.tls", "rfc.tls" },
-        { "gauche.net.tls.openssl", "rfc.tls" },
+#endif
+#if defined(GAUCHE_USE_MBEDTLS)
+        /* NB: Kludge - mbedTLS is implemented in a separate module rfc.tls.mbed,
+           but we don't autoload it with cond-expand gauche.net.tls.mbedtls.
+           If the gauche is compiled with mbedTLS support but the target system
+           lacks mbed DSO, using rfc.tls.mbed causes runtime error.  We don't
+           want that merely by checking feature identifier.  We set autoload
+           in rfc.tls, so the runtime error only occur when the program actually
+           try to use <mbed-tls>. */
+        { "gauche.net.tls.mbedtls", NULL },
 #endif
         { NULL, NULL }
     };
@@ -653,9 +664,14 @@ void Scm_SimpleMain(int argc, const char *argv[],
     ScmModule *user = Scm_UserModule();
     ScmObj mainproc = Scm_GlobalVariableRef(user, SCM_SYMBOL(SCM_INTERN("main")), 0);
     if (SCM_PROCEDUREP(mainproc)) {
-        ScmObj r = Scm_ApplyRec1(mainproc, args);
-        if (SCM_INTP(r)) Scm_Exit(SCM_INT_VALUE(r));
-        else             Scm_Exit(70);
+        static ScmObj run_main_proc = SCM_UNDEFINED;
+        SCM_BIND_PROC(run_main_proc, "run-main", Scm_GaucheInternalModule());
+        SCM_ASSERT(SCM_PROCEDUREP(run_main_proc));
+
+        ScmEvalPacket epak;
+        int r = Scm_Apply(run_main_proc, SCM_LIST2(mainproc, args), &epak);
+        SCM_ASSERT(r == 1 && SCM_INTP(epak.results[0]));
+        Scm_Exit(SCM_INT_VALUE(epak.results[0]));
     } else {
         Scm_Exit(70);
     }
