@@ -1,7 +1,7 @@
 /*
  * error.c - error handling
  *
- *   Copyright (c) 2000-2016  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2000-2018  Shiro Kawai  <shiro@acm.org>
  *
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -94,7 +94,7 @@ static void message_print(ScmObj obj, ScmPort *port, ScmWriteContext *ctx)
 {
     ScmClass *k = Scm_ClassOf(obj);
     Scm_Printf(port, "#<%A \"%30.1A\">",
-               Scm__InternalClassName(k),
+               Scm_ShortClassName(k),
                SCM_ERROR_MESSAGE(obj));
 }
 
@@ -106,7 +106,7 @@ static ScmObj message_allocate(ScmClass *klass, ScmObj initargs)
 }
 
 /* See comment on gauche/exception.h about hack in 'message' slot.
-   TODO: Remove this hack on 1.0 release. */
+   TRANSIENT: Remove this hack on 1.0 release. */
 static ScmObj message_get(ScmMessageCondition *obj)
 {
     ScmObj msglist = obj->message;
@@ -369,7 +369,7 @@ static ScmClass *compound_cpl[] = {
 static void compound_print(ScmObj obj, ScmPort *port, ScmWriteContext *ctx)
 {
     ScmClass *k = Scm_ClassOf(obj);
-    Scm_Printf(port, "#<%A", Scm__InternalClassName(k));
+    Scm_Printf(port, "#<%A", Scm_ShortClassName(k));
     ScmCompoundCondition *c = SCM_COMPOUND_CONDITION(obj);
     ScmObj cp;
     SCM_FOR_EACH(cp, c->conditions) {
@@ -566,17 +566,17 @@ ScmObj Scm_ConditionTypeName(ScmObj c)
     if (!SCM_CONDITIONP(c)) return SCM_MAKE_STR("(not a condition)");
 
     if (!SCM_COMPOUND_CONDITION_P(c)) {
-        sname = Scm__InternalClassName(Scm_ClassOf(c));
+        sname = Scm_ShortClassName(Scm_ClassOf(c));
     } else {
         ScmObj h = SCM_NIL, t = SCM_NIL, cp;
         SCM_FOR_EACH(cp, SCM_COMPOUND_CONDITION(c)->conditions) {
             ScmObj cc = SCM_CAR(cp);
             if (SCM_MIXIN_CONDITION_P(cc)) continue;
-            SCM_APPEND1(h, t, Scm__InternalClassName(Scm_ClassOf(cc)));
+            SCM_APPEND1(h, t, Scm_ShortClassName(Scm_ClassOf(cc)));
         }
         if (SCM_NULLP(h)) {
             /* not usual, but tolerate */
-            sname = Scm__InternalClassName(Scm_ClassOf(c));
+            sname = Scm_ShortClassName(Scm_ClassOf(c));
         } else {
             sname = Scm_StringJoin(h, &cond_name_delim, SCM_STRING_JOIN_INFIX);
         }
@@ -662,7 +662,7 @@ void Scm_Error(const char *msg, ...)
     SCM_ERROR_DOUBLE_FAULT_CHECK(vm);
     ScmObj ostr;
     SCM_ERROR_MESSAGE_FORMAT(ostr, msg);
-    ScmObj e = Scm_MakeError(Scm_GetOutputString(SCM_PORT(ostr), 0));
+    ScmObj e = Scm_MakeError(Scm_GetOutputString(SCM_PORT(ostr), TRUE));
     Scm_VMThrowException2(vm, e, SCM_RAISE_NON_CONTINUABLE);
     Scm_Panic("Scm_Error: Scm_VMThrowException returned.  something wrong.");
 }
@@ -734,7 +734,7 @@ void Scm_SysError(const char *msg, ...)
 
     ScmObj ostr;
     SCM_SYSERROR_MESSAGE_FORMAT(ostr, msg, en);
-    ScmObj e = Scm_MakeSystemError(Scm_GetOutputString(SCM_PORT(ostr), 0), en);
+    ScmObj e = Scm_MakeSystemError(Scm_GetOutputString(SCM_PORT(ostr), TRUE), en);
     Scm_VMThrowException2(vm, e, SCM_RAISE_NON_CONTINUABLE);
     Scm_Panic("Scm_Error: Scm_VMThrowException returned.  something wrong.");
 }
@@ -746,7 +746,8 @@ void Scm_SysError(const char *msg, ...)
  */
 void Scm_TypeError(const char *what, const char *expected, ScmObj got)
 {
-    Scm_Error("%s expected for %s, but got %S", expected, what, got);
+    Scm_Error("%s is supposed to be of type %s, but got %S",
+              what, expected, got);
 }
 
 
@@ -768,7 +769,7 @@ void Scm_PortError(ScmPort *port, int reason, const char *msg, ...)
     ScmObj ostr;
     if (en != 0) SCM_SYSERROR_MESSAGE_FORMAT(ostr, msg, en);
     else         SCM_ERROR_MESSAGE_FORMAT(ostr, msg);
-    ScmObj smsg = Scm_GetOutputString(SCM_PORT(ostr), 0);
+    ScmObj smsg = Scm_GetOutputString(SCM_PORT(ostr), TRUE);
 
     ScmClass *peclass;
     switch (reason) {
@@ -903,9 +904,9 @@ ScmObj Scm_RaiseCondition(ScmObj condition_type, ...)
 }
 
 /*
- * Show stack trace.
- *   stacklite - return value of Scm_VMGetStackLite
- *   maxdepth - maximum # of stacks to be shown.
+ * Show stack or call trace.
+ *   stacklite - return value of Scm_VMGetStackLite or Scm_VMGetCallTraceLite
+ *   maxdepth - maximum # of entries to be shown.
  *              0 to use the default.  -1 for unlimited.
  *   skip     - ignore this number of frames.  Useful to call this from
  *              a Scheme error handling routine, in order to skip the
@@ -937,6 +938,12 @@ void Scm_DumpStackTrace(ScmVM *vm, ScmPort *port)
     SCM_PUTZ("Stack Trace:\n", -1, port);
     SCM_PUTZ("_______________________________________\n", -1, port);
     Scm_ShowStackTrace(port, stack, 0, 0, 0, 0);
+    if (vm->callTrace) {
+        ScmObj trace = Scm_VMGetCallTraceLite(vm);
+        SCM_PUTZ("Call Trace:\n", -1, port);
+        SCM_PUTZ("_______________________________________\n", -1, port);
+        Scm_ShowStackTrace(port, trace, 0, 0, 0, 0);
+    }
     SCM_FLUSH(port);
 }
 
