@@ -1,7 +1,7 @@
 /*
  * gauche.h - Gauche scheme system header
  *
- *   Copyright (c) 2000-2016  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2000-2018  Shiro Kawai  <shiro@acm.org>
  * 
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -60,6 +60,7 @@
 #include <signal.h>
 #include <string.h>
 #include <errno.h>
+#include <stdint.h>
 #include <gauche/int64.h>
 #include <gauche/float.h>
 
@@ -374,9 +375,6 @@ SCM_EXTERN ScmChar Scm_CharDowncase(ScmChar ch);
 SCM_EXTERN ScmChar Scm_CharTitlecase(ScmChar ch);
 SCM_EXTERN ScmChar Scm_CharFoldcase(ScmChar ch);
 
-SCM_EXTERN void Scm__InstallCharconvHooks(ScmChar (*u2c)(int),
-                                          int (*c2u)(ScmChar));
-
 #if   defined(GAUCHE_CHAR_ENCODING_EUC_JP)
 #include "gauche/char_euc_jp.h"
 #elif defined(GAUCHE_CHAR_ENCODING_UTF_8)
@@ -387,6 +385,15 @@ SCM_EXTERN void Scm__InstallCharconvHooks(ScmChar (*u2c)(int),
 #include "gauche/char_none.h"
 #endif
 
+/* Character lexer category.  See 7.1.1 of R7RS */
+typedef enum {
+    SCM_CHAR_INITIAL,
+    SCM_CHAR_SUBSEQUENT,
+    SCM_CHAR_SIGN_SUBSEQUENT,
+} ScmCharLexerCategory;
+
+SCM_EXTERN int Scm_CharLexerCategoryP(ScmChar c, ScmCharLexerCategory cat);
+
 /*
  * HEAP ALLOCATED OBJECTS
  *
@@ -518,7 +525,7 @@ typedef struct ScmMacroRec     ScmMacro;
 typedef struct ScmPromiseRec   ScmPromise;
 typedef struct ScmRegexpRec    ScmRegexp;
 typedef struct ScmRegMatchRec  ScmRegMatch;
-typedef struct ScmWriteControlsRec  ScmWriteControls;  /* see writer.h */
+typedef struct ScmWriteControlsRec  ScmWriteControls;  /* see writerP.h */
 typedef struct ScmWriteContextRec   ScmWriteContext;   /* see writerP.h */
 typedef struct ScmWriteStateRec     ScmWriteState;     /* see wrtierP.h */
 typedef struct ScmAutoloadRec  ScmAutoload;
@@ -616,11 +623,13 @@ SCM_EXTERN ScmObj Scm_VMDynamicWindC(ScmSubrProc *before,
 SCM_EXTERN ScmObj Scm_VMWithErrorHandler(ScmObj handler, ScmObj thunk);
 SCM_EXTERN ScmObj Scm_VMWithGuardHandler(ScmObj handler, ScmObj thunk);
 SCM_EXTERN ScmObj Scm_VMWithExceptionHandler(ScmObj handler, ScmObj thunk);
+SCM_EXTERN ScmObj Scm_VMReraise(ScmObj condition);
 
 /* Miscellaneous stuff */
 SCM_EXTERN int    Scm_VMGetNumResults(ScmVM *vm);
 SCM_EXTERN ScmObj Scm_VMGetResult(ScmVM *vm);
 SCM_EXTERN ScmObj Scm_VMGetStackLite(ScmVM *vm);
+SCM_EXTERN ScmObj Scm_VMGetCallTraceLite(ScmVM *vm);
 SCM_EXTERN ScmObj Scm_VMGetStack(ScmVM *vm);
 
 /* A box is to keep a reference.  It is mainly used for mutable local variables.
@@ -811,6 +820,11 @@ SCM_EXTERN void Scm_InitStaticClassWithMeta(ScmClass *klass,
                                             ScmObj supers,
                                             ScmClassStaticSlotSpec *slots,
                                             int flags);
+SCM_EXTERN ScmObj Scm_ShortClassName(ScmClass *klass); /* strip '<' and '>' */
+
+/* Use this in 'compare' slot to allow Scheme method to define
+   compare/equal? behavior thru object-compare/object-equal? */
+SCM_EXTERN int Scm_ObjectCompare(ScmObj x, ScmObj y, int equalp);
 
 /* OBSOLETE */
 SCM_EXTERN void Scm_InitBuiltinClass(ScmClass *c, const char *name,
@@ -1167,9 +1181,9 @@ SCM_EXTERN ScmObj Scm_PairAttrSet(ScmPair *pair, ScmObj key, ScmObj value);
  * CHARACTERS
  */
 
-/* Illegal character handling mode.  Used in some APIs that handles
-   character conversion, such as input ports and string-incomplete->complete.
-*/
+/* OBSOLETED */
+/* This kind of thing is now handled by string-incomplete->complete
+   in libstr.scm. */
 typedef enum {
     SCM_ILLEGAL_CHAR_REJECT,    /* Refuse to handle illegal chars.  For ports
                                    this means raising an error.  For string
@@ -1277,10 +1291,11 @@ struct ScmProcedureRec {
     unsigned int locked   : 1;     /* setter locked? (see below) */
     unsigned int currying : 1;     /* autocurrying */
     unsigned int constant : 1;     /* constant procedure. see below. */
-    unsigned int reserved : 2;     /* unused yet. */
-    ScmObj info;                   /* source code info */
+    unsigned int leaf     : 1;     /* leaf procedure/method */
+    unsigned int reserved : 1;     /* unused yet. */
+    ScmObj info;                   /* source code info (see below) */
     ScmObj setter;                 /* setter, if exists. */
-    ScmObj inliner;                /* inliner information.  see below. */
+    ScmObj inliner;                /* inliner information (see below) */
 };
 
 /* About locked slot:
@@ -1322,6 +1337,63 @@ struct ScmProcedureRec {
    or the compiler configuration (e.g. internal encoding).
  */
 
+/* About 'leaf' flag:
+   For METHOD, this flag indicates the method doesn't refer to next-method
+   argument at all, so we can skip creating next-method instance when
+   making a call.
+   For CLOSURE, we *plan* to use this to indicate the closure body doesn't
+   make a call to another procedures, to allow certain optimizations.
+ */
+
+/* About 'info' slot:
+   This is a sort of the kitchen sink slot, keeping whatever miscellaneous
+   information as our implementation evolves.  Since this can be a part of
+   statically allocated structure, we can't change its format in a way
+   that breaks the backward compatibility.
+
+   SUBR, CLOSURE:
+           This slot may contain one of this:
+           - Signature: For example, the subr `cons' has (cons obj1 obj2)
+             in it.  The first pair may have the following pair attributes.
+             
+               `source-info'   (<filename> <lineno>)
+                   The source location the procedure is defined, if known.
+                   This info can be retrieved with (source-location PROC).
+               `bind-info'     (<module-name> <var-name>)
+                   The proc is bound to <var-name> in a module named
+                   <module-name>, and it's inlinable binding.  When the
+                   compiler can pre-calculate the proc to be called in a
+                   code, it can replace the original code with a global
+                   variable reference to <var-name>.  (We can't directly
+                   insert reference to the proc, for it may not be
+                   serializable for AOT compilation).
+
+           - Subr's name, as a string or a symbol.  This is the old format.
+             It may also the case that subr is created from C function
+             Scm_MakeSubr(), for it's cumbersome in C routine to construct
+             the signature list.  Accept it, but not recommended to use
+             this format in the new code.
+           - #f.  Indicates there's no useful info.
+
+   GENERIC:
+           This slot contains the "name" of the gf, which is a symbol.
+           A kludge: For setter gf, which can be created indirectly
+           via (define-method (setter GF) ...), we use a weird name
+           |setter of GF|.  This is a quick hack to make it work, but ideally
+           we should accept a list (setter GF) as the name.  Anticipate
+           this change in future.
+           Furthermore, in order to hold source-info, we might just make
+           it a pair, e.g. (NAME) or ((setter NAME)).
+
+   METHOD:
+           This slot contains (<name> <specializer> ...),
+           where <name> is the name of the generic function, and
+           <specializer>s are the name of classes.
+
+   NEXT_METHOD:
+           This slot isn't used.
+ */
+
 /* About procedure inliner:
    This slot holds information to inline procedures.  The value of this slot
    can be one of the following kinds:
@@ -1338,6 +1410,18 @@ struct ScmProcedureRec {
    <vector>: Procedures defined with define-inline have this.  The vector
       encodes intermediate form (IForm) of the procedure code, which will be
       expanded into the caller.
+
+   <macro>:  A compiler macro.  The macro expander is invoked with the
+      original source and macro-use environment, just like the ordinary macro
+      call.  The expander must return an Sexpr.  If the expander returns
+      the input as is, it indicates expansion is not possible and the form
+      is compiled as the ordinary procedure call.
+
+   <procedure>: A procedural inliner.  It has signature Sexpr,[IForm] -> IForm,
+      where Sexpr is the original source of call size (just for debug info) and
+      input [IForm] is the IForm for list of arguments.  See compiler-1.scm.
+      It returns the modified IForm.  It can return #<undef>, to indicate
+      inlining isn't possible.
  */
 
 /* procedure type */
@@ -1358,6 +1442,7 @@ enum ScmProcedureType {
 #define SCM_PROCEDURE_SETTER(obj)   SCM_PROCEDURE(obj)->setter
 #define SCM_PROCEDURE_INLINER(obj)  SCM_PROCEDURE(obj)->inliner
 #define SCM_PROCEDURE_SETTER_LOCKED(obj) SCM_PROCEDURE(obj)->locked
+#define SCM_PROCEDURE_LEAF(obj)     SCM_PROCEDURE(obj)->leaf
 
 SCM_CLASS_DECL(Scm_ProcedureClass);
 #define SCM_CLASS_PROCEDURE    (&Scm_ProcedureClass)
@@ -1378,15 +1463,18 @@ SCM_CLASS_DECL(Scm_ProcedureClass);
     SCM_PROCEDURE(obj)->locked = FALSE,                 \
     SCM_PROCEDURE(obj)->currying = FALSE,               \
     SCM_PROCEDURE(obj)->constant = FALSE,               \
+    SCM_PROCEDURE(obj)->leaf = FALSE,                   \
     SCM_PROCEDURE(obj)->reserved = 0,                   \
     SCM_PROCEDURE(obj)->info = inf,                     \
     SCM_PROCEDURE(obj)->setter = SCM_FALSE,             \
     SCM_PROCEDURE(obj)->inliner = SCM_FALSE
 
-#define SCM__PROCEDURE_INITIALIZER(klass, req, opt, typ, cst, inf, inl)  \
-    { { klass }, (req), (opt), (typ), FALSE, FALSE, cst, 0,              \
+/* This is internal - should never be used directly */
+#define SCM__PROCEDURE_INITIALIZER(klass, req, opt, typ, cst, lef, inf, inl) \
+    { { klass }, (req), (opt), (typ), FALSE, FALSE, cst, lef, 0,             \
       (inf), SCM_FALSE, (inl) }
 
+SCM_EXTERN ScmObj Scm_CopyProcedure(ScmProcedure *proc);
 SCM_EXTERN ScmObj Scm_CurryProcedure(ScmObj proc, ScmObj *given,
                                      int ngiven, int foldlen);
 
@@ -1400,6 +1488,8 @@ struct ScmClosureRec {
 #define SCM_CLOSUREP(obj) \
     (SCM_PROCEDUREP(obj)&&(SCM_PROCEDURE_TYPE(obj)==SCM_PROC_CLOSURE))
 #define SCM_CLOSURE(obj)           ((ScmClosure*)(obj))
+#define SCM_CLOSURE_CODE(obj)      SCM_CLOSURE(obj)->code
+#define SCM_CLOSURE_ENV(obj)       SCM_CLOSURE(obj)->env
 
 SCM_EXTERN ScmObj Scm_MakeClosure(ScmObj code, ScmEnvFrame *env);
 
@@ -1429,7 +1519,7 @@ struct ScmSubrRec {
 #define SCM__DEFINE_SUBR_INT(cvar, req, opt, cst, inf, flags, func, inliner, data) \
     ScmSubr cvar = {                                                        \
         SCM__PROCEDURE_INITIALIZER(SCM_CLASS_STATIC_TAG(Scm_ProcedureClass),\
-            req, opt, SCM_PROC_SUBR, cst, inf, inliner),                    \
+             req, opt, SCM_PROC_SUBR, cst, 0, inf, inliner),                \
         flags, (func), (data)                                               \
     }
 
@@ -1463,6 +1553,8 @@ struct ScmGenericRec {
     ScmObj (*fallback)(ScmObj *argv, int argc, ScmGeneric *gf);
     void *data;
     ScmInternalMutex lock;
+    void *dispatcher;           /* To accelerate dispatching. See dispatch.c
+                                   TRANSIENT: Move this up in 1.0 release */
 };
 
 SCM_CLASS_DECL(Scm_GenericClass);
@@ -1474,9 +1566,11 @@ SCM_CLASS_DECL(Scm_GenericClass);
 #define SCM_DEFINE_GENERIC(cvar, cfunc, data)                           \
     ScmGeneric cvar = {                                                 \
         SCM__PROCEDURE_INITIALIZER(SCM_CLASS_STATIC_TAG(Scm_GenericClass),\
-                                   0, 0, SCM_PROC_GENERIC, 0,           \
+                                   0, 0, SCM_PROC_GENERIC, 0, 0,        \
                                    SCM_FALSE, NULL),                    \
-        SCM_NIL, 0, cfunc, data                                         \
+        SCM_NIL, 0, cfunc, data,                                        \
+        SCM_INTERNAL_MUTEX_INITIALIZER,                                 \
+        NULL                                                            \
     }
 
 SCM_EXTERN void Scm_InitBuiltinGeneric(ScmGeneric *gf, const char *name,
@@ -1506,11 +1600,12 @@ SCM_CLASS_DECL(Scm_MethodClass);
 #define SCM_METHODP(obj)           SCM_ISA(obj, SCM_CLASS_METHOD)
 #define SCM_METHOD(obj)            ((ScmMethod*)obj)
 #define SCM_METHOD_LOCKED(obj)     SCM_METHOD(obj)->common.locked
+#define SCM_METHOD_LEAF_P(obj)     SCM_METHOD(obj)->common.leaf
 
 #define SCM_DEFINE_METHOD(cvar, gf, req, opt, specs, func, data)        \
     ScmMethod cvar = {                                                  \
         SCM__PROCEDURE_INITIALIZER(SCM_CLASS_STATIC_TAG(Scm_MethodClass),\
-                                   req, opt, SCM_PROC_METHOD, 0,        \
+                                   req, opt, SCM_PROC_METHOD, 0, 0,     \
                                    SCM_FALSE, NULL),                    \
         gf, specs, func, data, NULL                                     \
     }
@@ -1587,8 +1682,19 @@ SCM_EXTERN ScmObj Scm_MakeSyntax(ScmSymbol *name, ScmObj handler);
 SCM_CLASS_DECL(Scm_MacroClass);
 #define SCM_CLASS_MACRO            (&Scm_MacroClass)
 
-SCM_EXTERN ScmObj Scm_MakeMacro(ScmSymbol *name, ScmObj transformer);
+/* TRANSIENT: Scm_MakeMacroFull is to keep ABI compatibility.  Will be gone
+   in 1.0.  */
+#if GAUCHE_API_0_95
+SCM_EXTERN ScmObj Scm_MakeMacro(ScmObj name, ScmObj transformer,
+                                ScmObj src, ScmObj describer);
+#define Scm_MakeMacroFull(a,b,c,d) Scm_MakeMacro(a,b,c,d)
+#else
+SCM_EXTERN ScmObj Scm_MakeMacroFull(ScmObj name, ScmObj transformer,
+                                    ScmObj src, ScmObj describer);
+ScmObj Scm_MakeMacro(ScmSymbol *name, ScmObj transformer);
+#endif
 SCM_EXTERN ScmObj Scm_MacroTransformer(ScmMacro *mac);
+SCM_EXTERN ScmObj Scm_MacroName(ScmMacro *mac);
 
 SCM_EXTERN ScmObj Scm_MakeMacroTransformer(ScmSymbol *name,
                                            ScmObj proc);
@@ -1693,6 +1799,8 @@ SCM_EXTERN void Scm_ShowStackTrace(ScmPort *out, ScmObj stacklite,
                                    int maxdepth, int skip, int offset,
                                    int format);
 
+SCM_EXTERN void Scm_SetCallTraceSize(u_long size);
+
 /* TRANSIENT: Scm_ReportErrr2 is to keep ABI compatibility.  Will be gone
    in 1.0.  */
 #if    GAUCHE_API_0_95
@@ -1766,6 +1874,7 @@ SCM_CLASS_DECL(Scm_SysSigsetClass);
 
 SCM_EXTERN ScmObj Scm_SysSigsetOp(ScmSysSigset*, ScmObj, int);
 SCM_EXTERN ScmObj Scm_SysSigsetFill(ScmSysSigset*, int);
+SCM_EXTERN void   Scm_SigFillSetMostly(sigset_t *set);
 SCM_EXTERN ScmObj Scm_GetSignalHandler(int);
 SCM_EXTERN ScmObj Scm_GetSignalHandlerMask(int);
 SCM_EXTERN ScmObj Scm_GetSignalHandlers(void);
