@@ -1,7 +1,7 @@
 /*
- * class.h - Gauche object system private header
+ * class.h - Gauche object system header
  *
- *   Copyright (c) 2000-2014  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2000-2018  Shiro Kawai  <shiro@acm.org>
  *
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -99,18 +99,38 @@ SCM_CLASS_DECL(Scm_AccessorMethodClass);
 #define SCM_ACCESSOR_METHOD(obj)     ((ScmAccessorMethod*)obj)
 #define SCM_ACCESSOR_METHOD_P(obj)   SCM_ISA(obj, SCM_CLASS_SLOT_ACCESSOR)
 
+/* Instance allocation API
+ *   Because of some historical context, the API names are somewhat
+ *   confusing.  We have several layer of 'allocate' API.
+ *
+ *   (1) Scheme 'allocate-instance' generic function and Scm_Allocate().
+ *     This is a higher layer.  It takes a class and a plist of initargs.
+ *     It uses internal dispatch mechanism to call proper concrete
+ *     allocate function, then sets up slots to the sane values.
+ *     NB: C Scm_Allocate() only calls "base" method, i.e. the one
+ *     through ScmClass->allocate(), and may throw an error if no allocator
+ *     is set.  Scheme version may be dispatched to Scheme-defined method.
+ *
+ *   (2) static *_allocate functions
+ *     These are the 'methods' stored in ScmClass->allocate, and does
+ *     specific allocation and setup for the class.
+ *     Usually calls SCM_NEW_INSTANCE in it.
+ *
+ *   (3) SCM_NEW_INSTANCE, Scm_NewInstance
+ *     The bottom layer.  Allocates memory, and if it's for Scheme
+ *     instances, allocates slot vector as well.
+ *     The code must always use the macro version SCM_NEW_INSTANCE.
+ */
+
+
 /* cliche in allocate method */
-#define SCM_ALLOCATE(klassname, klass) \
-    ((klassname*)Scm_AllocateInstance(klass, sizeof(klassname)))
+#define SCM_NEW_INSTANCE(klassname, klass) \
+    ((klassname*)Scm_NewInstance(klass, sizeof(klassname)))
 
 /* some internal methods */
 
-SCM_EXTERN ScmObj Scm_ObjectAllocate(ScmClass *klass, ScmObj initargs);
-SCM_EXTERN ScmObj Scm_AllocateInstance(ScmClass *klass, int coresize);
-SCM_EXTERN ScmObj Scm__AllocateAndInitializeInstance(ScmClass *klass,
-                                                     ScmObj *inits,
-                                                     int numInits,
-                                                     u_long flags);
+SCM_EXTERN ScmObj Scm_Allocate(ScmClass *klass, ScmObj initargs);
+SCM_EXTERN ScmObj Scm_NewInstance(ScmClass *klass, int coresize);
 SCM_EXTERN ScmObj Scm_ComputeCPL(ScmClass *klass);
 SCM_EXTERN int    Scm_MethodApplicableForClasses(ScmMethod *m,
                                                  ScmClass *types[],
@@ -156,11 +176,10 @@ SCM_EXTERN ScmObj Scm_VMTouchInstance(ScmObj obj);
 SCM_EXTERN void   Scm_DeleteDirectSubclass(ScmClass *super, ScmClass *sub);
 SCM_EXTERN void   Scm_DeleteDirectMethod(ScmClass *super, ScmMethod *m);
 
-SCM_EXTERN ScmObj Scm__InternalClassName(ScmClass *klass);
-
 SCM_EXTERN ScmGeneric Scm_GenericApplyGeneric;
 SCM_EXTERN ScmGeneric Scm_GenericObjectHash;
 SCM_EXTERN ScmGeneric Scm_GenericObjectApply;
+SCM_EXTERN ScmGeneric Scm_GenericObjectEqualP;
 SCM_EXTERN ScmGeneric Scm_GenericObjectSetter;
 SCM_EXTERN ScmGeneric Scm_GenericChangeClass;
 
@@ -168,6 +187,13 @@ SCM_EXTERN ScmObj Scm_UpdateDirectMethod(ScmMethod *m,
                                          ScmClass *oldk,
                                          ScmClass *newk);
 
+/* TRANSIENT: Obsoleted. */
+SCM_EXTERN ScmObj Scm_ObjectAllocate(ScmClass *klass, ScmObj initargs);
+/* TRANSIENT: Obsoleted.  Use SCM_NEW_INSTANCE */
+#define SCM_ALLOCATE(klassname, klass)  SCM_NEW_INSTANCE(klassname, klass)
+/* TRANSIENT: Obsoleted.  Use Scm_NewInstance*/
+SCM_EXTERN ScmObj Scm_AllocateInstance(ScmClass *klass, int coresize);
+
 SCM_DECL_END
 
 #endif /* GAUCHE_CLASS_H */
