@@ -1,7 +1,7 @@
 /*
  * pthread.h - pthread primitives
  *
- *   Copyright (c) 2000-2014  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2000-2018  Shiro Kawai  <shiro@acm.org>
  *
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -125,4 +125,7 @@ typedef pthread_mutex_t ScmInternalFastlock;
 #define SCM_INTERNAL_FASTLOCK_DESTROY(fl) SCM_INTERNAL_MUTEX_DESTROY(fl)
 #endif /*!HAVE_PTHREAD_SPINLOCK_T*/
 
+/* Issues a full memory barrier */
+#define SCM_INTERNAL_SYNC()                 __sync_synchronize()
+
 #endif /* GAUCHE_PTHREAD_H */
