@@ -1,7 +1,7 @@
 /*
  * system.h - Gauche system interface
  *
- *   Copyright (c) 2000-2014  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2000-2018  Shiro Kawai  <shiro@acm.org>
  *
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -93,9 +93,15 @@ SCM_EXTERN ScmObj Scm_DirName(ScmString *filename);
 SCM_EXTERN ScmObj Scm_BaseName(ScmString *filename);
 
 /* struct stat */
+#ifdef GAUCHE_WINDOWS
+typedef struct __stat64 ScmStat;
+#else  /*!GAUCHE_WINDOWS*/
+typedef struct stat ScmStat;
+#endif /*!GAUCHE_WINDOWS*/
+
 typedef struct ScmSysStatRec {
     SCM_HEADER;
-    struct stat statrec;
+    ScmStat statrec;
 } ScmSysStat;
 
 SCM_CLASS_DECL(Scm_SysStatClass);
@@ -147,13 +153,18 @@ SCM_EXTERN ScmObj Scm_Int64SecondsToTime(ScmInt64 sec);
 SCM_EXTERN ScmObj Scm_RealSecondsToTime(double sec);
 SCM_EXTERN ScmObj Scm_TimeToSeconds(ScmTime *t);
 
-#if !defined(HAVE_STRUCT_TIMESPEC)
-struct timespec {
+/* struct timespec compatibility handling.  MinGW32 runtime v3.21, at least, has
+   incompatible struct timespec. */
+#if defined(HAVE_STRUCT_TIMESPEC) && (!defined(GAUCHE_WINDOWS) || defined(__MINGW64_VERSION_MAJOR))
+typedef struct timespec ScmTimeSpec;
+#else  /*!(defined(HAVE_STRUCT_TIMESPEC) && (!defined(GAUCHE_WINDOWS) || defined(__MINGW64_VERSION_MAJOR)))*/
+typedef struct ScmTimeSpecRec {
     time_t tv_sec;
-    long   tv_usec;
-};
-#endif /*!HAVE_STRUCT_TIMESPEC*/
-SCM_EXTERN struct timespec *Scm_GetTimeSpec(ScmObj t, struct timespec *spec);
+    long   tv_nsec;
+} ScmTimeSpec;
+#endif /*!(defined(HAVE_STRUCT_TIMESPEC) && (!defined(GAUCHE_WINDOWS) || defined(__MINGW64_VERSION_MAJOR)))*/
+
+SCM_EXTERN ScmTimeSpec *Scm_GetTimeSpec(ScmObj t, ScmTimeSpec *spec);
 
 /* sched_yield */
 SCM_EXTERN void   Scm_YieldCPU(void);
@@ -172,6 +183,8 @@ SCM_CLASS_DECL(Scm_SysTmClass);
 
 SCM_EXTERN ScmObj Scm_MakeSysTm(struct tm *);
 
+SCM_EXTERN int    Scm_NanoSleep(const ScmTimeSpec *req,
+                                ScmTimeSpec *rem);
 
 /*==============================================================
  * Groups and users
@@ -271,11 +284,13 @@ typedef struct ScmHeaderRec ScmSysFdset;
 
 SCM_EXTERN int    Scm_Mkstemp(char *tmpl);
 SCM_EXTERN ScmObj Scm_SysMkstemp(ScmString *tmpl);
+SCM_EXTERN ScmObj Scm_SysMkdtemp(ScmString *tmpl);
 SCM_EXTERN ScmObj Scm_Environ(void);
 SCM_EXTERN const char *Scm_GetEnv(const char *name);
 SCM_EXTERN void Scm_SetEnv(const char *name, const char *value, int overwrite);
 SCM_EXTERN void Scm_UnsetEnv(const char *name);
 SCM_EXTERN void Scm_ClearEnv(void);
+SCM_EXTERN int  Scm_AvailableProcessors(void);
 
 /*==============================================================
  * Windows-specific utility functions
@@ -291,6 +306,8 @@ SCM_EXTERN ScmObj Scm_MakeWinProcess(HANDLE h);
 SCM_EXTERN int    Scm_WinProcessP(ScmObj p);
 SCM_EXTERN pid_t  Scm_WinProcessPID(ScmObj p);
 SCM_EXTERN HANDLE Scm_WinProcess(ScmObj p);
+
+SCM_EXTERN ScmObj Scm_WinGetPipeName(HANDLE h);
 #endif /* GAUCHE_WINDOWS */
 
 #endif /* GAUCHE_SYSTEM_H */
