@@ -1,7 +1,7 @@
 /*
  * vm.h - Virtual machine
  *
- *   Copyright (c) 2000-2014  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2000-2018  Shiro Kawai  <shiro@acm.org>
  *
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -45,10 +45,10 @@
 
 #define SCM_PCTYPE ScmWord*
 
-#if defined(ITIMER_PROF) && defined(SIGPROF)
+#if (defined(ITIMER_PROF) && defined(SIGPROF)) || defined(GAUCHE_WINDOWS)
 /* define if you want to use profiler */
 #define GAUCHE_PROFILE
-#endif /* defined(ITIMER_PROF) && defined(SIGPROF) */
+#endif /* (defined(ITIMER_PROF) && defined(SIGPROF)) || defined(GAUCHE_WINDOWS) */
 
 
 #ifdef __GNUC__
@@ -73,6 +73,9 @@ SCM_EXTERN void Scm_VMFlushFPStack(ScmVM *vm);
 /* Actual structure is defined in code.h */
 typedef struct ScmCompiledCodeRec ScmCompiledCode;
 
+/* Actual structure is defined in priv/vmP.h */
+typedef struct ScmCallTraceRec ScmCallTrace;
+
 /*
  * Environment frame
  *
@@ -176,12 +179,8 @@ SCM_EXTERN ScmObj Scm_MakeSyntacticClosure(ScmObj env,
  *   It shouldn't be used directly from applications.
  */
 
-typedef struct ScmIdentifierRec {
-    SCM_HEADER;
-    ScmSymbol *name;
-    ScmModule *module;
-    ScmObj env;
-} ScmIdentifier;
+/* The definition is in gauche/priv/identifierP.h (hidden) */
+typedef struct ScmIdentifierRec ScmIdentifier;
 
 SCM_CLASS_DECL(Scm_IdentifierClass);
 #define SCM_CLASS_IDENTIFIER    (&Scm_IdentifierClass)
@@ -189,12 +188,26 @@ SCM_CLASS_DECL(Scm_IdentifierClass);
 #define SCM_IDENTIFIER(obj)     ((ScmIdentifier*)(obj))
 #define SCM_IDENTIFIERP(obj)    SCM_XTYPEP(obj, SCM_CLASS_IDENTIFIER)
 
-SCM_EXTERN ScmObj Scm_MakeIdentifier(ScmSymbol *name, ScmModule *mod,
+/* Create an identifier.
+   NAME can be a symbol or an identifier.  
+   MOD is the toplevel module to close; can be NULL to use the current module.
+   ENV is the local local environment (list of frames) to close. */
+SCM_EXTERN ScmObj Scm_MakeIdentifier(ScmObj name,
+                                     ScmModule *mod,
                                      ScmObj env);
-SCM_EXTERN ScmObj Scm_CopyIdentifier(ScmIdentifier *id);
-SCM_EXTERN int    Scm_IdentifierBindingEqv(ScmIdentifier *id, ScmSymbol *sym,
+/* Returns the minimal local environment (list of frames) where this 
+   identifier is bound.  If the identifier isn't bound locally, it would be ().
+   The returned value may not be the same as ENV argument passed to the
+   constructor; we truncate irrelevant frames. */
+SCM_EXTERN ScmObj Scm_IdentifierEnv(ScmIdentifier *id);
+SCM_EXTERN ScmObj Scm_WrapIdentifier(ScmIdentifier *id);
+SCM_EXTERN int    Scm_IdentifierBindingEqv(ScmIdentifier *id,
+                                           ScmSymbol *sym,
                                            ScmObj env);
-SCM_EXTERN int    Scm_FreeVariableEqv(ScmObj var, ScmObj sym, ScmObj env);
+
+SCM_EXTERN ScmIdentifier *Scm_OutermostIdentifier(ScmIdentifier *id);
+SCM_EXTERN ScmSymbol     *Scm_UnwrapIdentifier(ScmIdentifier *id);
+SCM_EXTERN ScmGloc       *Scm_IdentifierGlobalBinding(ScmIdentifier *id);
 
 /*
  * Escape handling
@@ -275,6 +288,9 @@ typedef struct ScmEscapePointRec {
                                    with-error-handler uses the latter model,
                                    but SRFI-34's guard needs the former model.
                                 */
+    int reraised;               /* EXPERIMENTAL: if exception is reraised,
+                                   this flag is set to TRUE and the exception
+                                   handler can return to the caller. */
 } ScmEscapePoint;
 
 /* Link management */
@@ -495,13 +511,21 @@ struct ScmVMRec {
                                    longjmp(). */
     void *escapeData[2];        /* ditto. */
 
-    /* Custom debugger */
-    ScmObj defaultEscapeHandler;
+    /* Custom debugger or error reporter */
+    ScmObj customErrorReporter; /* If set, Scm_ReportError (report-error) calls
+                                   this procedure with an exception object.
+                                   The default behavior is to show the error
+                                   type, message, and the stack trace.
+                                   Alter this only if you want to customize
+                                   it per thread.
+                                 */
+
+    /* TRANSIENT: For the ABI compatibility.  Remove on 1.0 */
+    ScmObj dummy0;
+    ScmObj dummy1;
+    ScmObj dummy2;
 
     /* Program information */
-    ScmObj dummy0;              /* for the ABI compatibility.  remove on 1.0. */
-    ScmObj dummy1;              /* ditto */
-    ScmObj dummy2;              /* ditto */
     int    evalSituation;       /* eval situation (related to eval-when) */
 
     /* Signal information */
@@ -516,14 +540,29 @@ struct ScmVMRec {
 #if defined(GAUCHE_USE_WTHREADS)
     ScmWinCleanup *winCleanup; /* mimic pthread_cleanup_* */
 #endif /*defined(GAUCHE_USE_WTHREADS)*/
+
+    u_long vmid;                /* Numerical ID, mainly for debugging aid.
+                                   Can be recycled, so don't use this to
+                                   identify thread programtically.
+                                   Set by vm_register. */
+
+    ScmCallTrace *callTrace;
 };
 
 SCM_EXTERN ScmVM *Scm_NewVM(ScmVM *proto, ScmObj name);
 SCM_EXTERN int    Scm_AttachVM(ScmVM *vm);
 SCM_EXTERN void   Scm_DetachVM(ScmVM *vm);
 SCM_EXTERN void   Scm_VMDump(ScmVM *vm);
-SCM_EXTERN void   Scm_VMDefaultExceptionHandler(ScmObj exc);
+SCM_EXTERN ScmObj Scm_VMDefaultExceptionHandler(ScmObj exc);
+/* TRANSIENT: Scm_VMThrowException2 is to keep ABI compatibility.  Will be
+   gone in 1.0 */
+#if    GAUCHE_API_0_95
+SCM_EXTERN ScmObj Scm_VMThrowException(ScmVM *vm, ScmObj exc, u_long flags);
+#define Scm_VMThrowException2(v, e, f)  Scm_VMThrowException(v, e, f)
+#else  /*!GAUCHE_API_0_95*/
 SCM_EXTERN ScmObj Scm_VMThrowException(ScmVM *vm, ScmObj exc);
+SCM_EXTERN ScmObj Scm_VMThrowException2(ScmVM *vm, ScmObj exc, u_long flags);
+#endif /*!GAUCHE_API_0_95*/
 SCM_EXTERN ScmObj Scm_VMGetSourceInfo(ScmCompiledCode *code, SCM_PCTYPE pc);
 SCM_EXTERN ScmObj Scm_VMGetBindInfo(ScmCompiledCode *code, SCM_PCTYPE pc);
 SCM_EXTERN void   Scm_VMSetResult(ScmObj obj);
@@ -678,7 +717,8 @@ enum {
     SCM_COMPILE_NO_LIFTING = (1L<<7),      /* Do not run lambda lifting pass
                                               (pass4). */
     SCM_COMPILE_INCLUDE_VERBOSE = (1L<<8), /* Report expansion of 'include' */
-    SCM_COMPILE_ENABLE_CEXPR = (1L<<9)     /* Support C-expressions by reader */
+    SCM_COMPILE_ENABLE_CEXPR = (1L<<9),    /* Support C-expressions by reader */
+    SCM_COMPILE_NOINLINE_SETTERS = (1L<<10) /* Do not inline setters */
 };
 
 #define SCM_VM_COMPILER_FLAG_IS_SET(vm, flag) ((vm)->compilerFlags & (flag))
@@ -700,4 +740,6 @@ SCM_EXTERN ScmObj Scm_CallProcedureInliner(ScmObj obj, ScmObj form, ScmObj env);
 /* This is in module.c, but it's not for public, so declaration is here. */
 SCM_EXTERN ScmModule* Scm_GaucheInternalModule(void);
 
+SCM_EXTERN ScmObj Scm__VMInsnOffsets(void);
+
 #endif /* GAUCHE_VM_H */
