@@ -1,7 +1,7 @@
 /*
  * writer.h - Writer API
  *
- *   Copyright (c) 2000-2016  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2000-2018  Shiro Kawai  <shiro@acm.org>
  *
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -56,20 +56,6 @@ enum ScmWriteCaseFlags {
                                    capital letters */
 };
 
-/* Writer control parameters */
-struct ScmWriteControlsRec {
-    SCM_HEADER;
-    int printLength;            /* -1 for no limit */
-    int printLevel;             /* -1 for no limit */
-    int printBase;              /* 2-36 */
-    int printRadix;             /* boolean, #t to print radix for all numbers */
-};
-
-SCM_CLASS_DECL(Scm_WriteControlsClass);
-#define SCM_CLASS_WRITE_CONTROLS  (&Scm_WriteControlsClass)
-#define SCM_WRITE_CONTROLS(obj)   ((ScmWriteControls*)(obj))
-#define SCM_WRITE_CONTROLS_P(obj) SCM_XTYPEP(obj, SCM_CLASS_WRITE_CONTROLS)
-
 SCM_EXTERN ScmWriteControls *Scm_MakeWriteControls(const ScmWriteControls *proto);
 SCM_EXTERN const ScmWriteControls *Scm_DefaultWriteControls(void);
 SCM_EXTERN const ScmWriteControls *Scm_GetWriteControls(ScmWriteContext *ctx,
