@@ -7,7 +7,7 @@
 
 ;; Reads Unicode data tables and generates various source files.
 
-;; This script can serve two operations.
+;; This script can serve three operations.
 ;;
 ;; (1) Generate unicode-data.scm from Unicode character database
 ;;   It is only necessary when new version of Unicode is published,
@@ -21,12 +21,20 @@
 ;;   tables, within the following two generated source files:
 ;;
 ;;   char_attr.c                - General category and case mappings.
-;;   gauche/priv/unicode_attr.h - Grapheme break, word break, normalization.
+;;   gauche/priv/unicode_attr.h - Grapheme break, word break, East-asian width
 ;;
 ;;   This is done when you build from git source tree.
 ;;
 ;;    gosh ./gen-unicode.scm --compile unicode-data.scm
 ;;
+;; (3) Fetch Unicode character database files.  This is for developer's
+;;   convenience.
+;;
+;;    gosh ./gen-unicode.scm --fetch <unicode-database-directory> [<unicode-version>
+;;   Note that the original content in <unicode-database-directory> is
+;;   overwritten.
+;;   You might need to adjust code for newer version of Unicode.
+;;   See doc/HOWTO-unicode.txt for the details.
 
 (use srfi-1)
 (use srfi-13)
@@ -39,8 +47,16 @@
 (use gauche.record)
 (use gauche.charconv)
 (use gauche.uvector)
+
+;; needs to load from source
+(add-load-path "../lib" :relative)
 (use text.unicode.ucd)
 
+;; rfc.http is only required by '--fetch' operation.  We don't want to load
+;; it during build process, so let's autoload it.
+(autoload rfc.http http-get)
+
+
 ;; We generate four kind of lookup structures.  Each structure consists
 ;; of various types of tables in order to reduce the size.
 ;;
@@ -203,6 +219,7 @@
 
 (define (main args)
   (match (cdr args)
+    [("--fetch" dir . maybe-version) (apply fetch-ucd dir maybe-version)]
     [("--import" dir ucdfile)
      (unless (file-is-directory? dir)
        (exit 1 "Directory required, but got: ~a" dir))
@@ -214,26 +231,58 @@
      (generate-tables (call-with-input-file ucdfile ucd-load-db))]
     [else
      (exit 1 "Usage:\n\
-              gen-unicode.scm --import <unicode-database-dir>\n\
+              gen-unicode.scm --fetch <unicode-database-dir> [<unicode-version>]\n\
+              gen-unicode.scm --import <unicode-database-dir> <ucd-file>\n\
               gen-unicode.scm --compile <unicode-data.scm>")])
   0)
 
+;;;
+;;; Fetching 
+;;;
+(define (fetch-ucd dir :optional (version #f))
+  (let ([path (if version
+                #"/Public/~|version|/ucd"
+                "/Public/UCD/latest/ucd")]
+        [datafiles   '("UnicodeData.txt"
+                       "SpecialCasing.txt"
+                       "PropList.txt"
+                       "EastAsianWidth.txt"
+                       "auxiliary/GraphemeBreakProperty.txt"
+                       "auxiliary/SentenceBreakProperty.txt"
+                       "auxiliary/WordBreakProperty.txt")])
+    (make-directory* (build-path dir "auxiliary"))
+    (dolist [f datafiles]
+      (let1 p (build-path dir f)
+        (display #"Getting ~|f|... ") (flush)
+        (call-with-output-file p
+          (^o (http-get "www.unicode.org" (build-path path f)
+                        :sink o :flusher (^ _ #t))))
+        (display "done\n")))))
+
 ;;;
 ;;;  Generate source file
 ;;;
 
 (define (generate-tables db)
-  (with-output-to-file "char_attr.c"
-    (^() (preamble db)
-      (generate-category-tables db)
-      (generate-case-tables db)
-      (generate-digit-value-tables db)))
-  (with-output-to-file "gauche/priv/unicode_attr.h"
-    (^() (preamble db)
-      (generate-break-tables db))))
+  (receive (char_attr.p char_attr.c) (sys-mkstemp "char_attr.c.")
+    (receive (unicode_attr.p unicode_attr.h) (sys-mkstemp "unicode_attr.h.")
+      (with-output-to-port char_attr.p
+        (^() (preamble db)
+          (generate-category-tables db)
+          (generate-case-tables db)
+          (generate-digit-value-tables db)))
+      (with-output-to-port unicode_attr.p
+        (^() (preamble db)
+          (generate-break-tables db)
+          (generate-width-tables db)))
+      (close-port char_attr.p)
+      (close-port unicode_attr.p)
+      (sys-rename char_attr.c "char_attr.c")
+      (sys-rename unicode_attr.h "gauche/priv/unicode_attr.h"))))
 
 (define (preamble db)
   (print "/* Generated automatically from Unicode character database */")
+  (print "/* See src/gen-unicode.scm for the description of data structures. */")
   (print #"/* Unicode version ~(ucd-version db).  Do not edit. */")
   )
 
@@ -484,6 +533,20 @@
     (print "}")))
 
 ;; Break property values
+;; This goes to src/gauche/priv/unicode_attr.h
+;;
+;; The data structure is two-level tables that maps codepoint 0-1FFFFF to
+;; an octet.
+;; 
+;; The first level, break_table, is 512-length byte vector.
+;; Upper 9bit of codepoint is used to index this table.  If the entry
+;; is 255, the break property of that codepoint takes the default value.
+;;
+;; Otherwise, the first table entry is the subtable, break_subtable[N][],
+;; and the lower 8bit of codepoint is used to index the subtable.
+;; it returns an octet, in which upper 4 bit is for grapheme break property
+;; and lower 4 bit is for word break property.
+
 (define (generate-break-tables db)
   (define subtable-count 0)
 
@@ -546,3 +609,73 @@
                   (format #t " ~3d," n)))
     (format #t "\n};\n"))
   )
+
+;; EastAsianWidth property values
+;; This goes to src/gauche/priv/unicode_attr.h
+;;
+;; The data structure is two-level tables that maps codepoint 0-1FFFFF to
+;; an octet.
+;; 
+;; The first level, width_table, is 512-length byte vector.
+;; Upper 9bit of codepoint is used to index this table.  If the entry
+;; is one of WIDTH_x values, all codepoints of that range has that property
+;; value.
+;;
+;; Otherwise, the entry value - NUM_WIDTH_PROPERTIES points to the subtable,
+;; width_subtable[N][].  It is a vecto of nibbles; to look up, first take
+;; 1-7bit of the codepoint and look up an octet; if the LSB of codepoint
+;; is 0, take lower nibble; otherwise, take upper nibble.
+
+(define (generate-width-tables db)
+  (define prop-count (length (ucd-east-asian-widths)))
+  (define subtable-count prop-count)
+  (define width-table (~ db'width-table))
+  
+  ;; returns subtable-number
+  (define (gen-subtable start-code)
+    (let1 e (dict-get width-table start-code 'N)
+      (if (any?-ec (: lb 256)
+                   (not (eq? (dict-get width-table (+ start-code lb) 'N) e)))
+        (rlet1 subtable-num subtable-count
+          (print "  {")
+          (do-ec (: c 128)
+                 (let ([w0 (dict-get width-table (+ start-code (* c 2)) 'N)]
+                       [w1 (dict-get width-table (+ start-code (* c 2) 1) 'N)])
+                   (print #"    WIDTH_ENTRY(WIDTH_~|w1|, WIDTH_~|w0|),")))
+          (print "  },")
+          (inc! subtable-count))
+        ;; all codepoints of this range shares the value.
+        (find-index (cut eqv? e <>) (ucd-east-asian-widths))))) 
+
+  ;; Generate table of symbols
+  (define (gen-symbol-table)
+    (for-each-with-index (^(i c) (format #t "#define WIDTH_~a ~a\n" c i))
+                         (ucd-east-asian-widths))
+    (print #"#define NUM_WIDTH_PROPERTIES ~|prop-count|")
+    (print)
+    (print "static void init_WIDTH_symbols(ScmModule *mod) {")
+    (print #"  ScmObj v = Scm_MakeVector(~|prop-count|, SCM_FALSE);")
+    (for-each-with-index
+     (^[i c]
+       (print #"  SCM_VECTOR_ELEMENT(v, ~|i|) = SCM_INTERN(\"~|c|\");"))
+     (ucd-east-asian-widths))
+    (print "  Scm_DefineConst(mod, \
+                 SCM_SYMBOL(SCM_INTERN(\"*east-asian-widths*\")), \
+                 v);")
+    (print "}"))
+
+  (print)
+  (gen-symbol-table)
+  (print)
+  (print "#define WIDTH_ENTRY(a, b)  (((a)<<4) | (b))")
+  (print)
+  (print "static unsigned char width_subtable[][128] = {")
+  (let1 nlist (list-ec (: n 512) (gen-subtable (* n 256)))
+    (print "};")
+    (print)
+    (format #t "static unsigned char width_table[] = {")
+    (do-ec (:parallel (: n nlist) (:integers i))
+           (begin (when (zero? (mod i 8)) (format #t "\n   "))
+                  (format #t " ~3d," n)))
+    (format #t "\n};\n"))
+  )
