@@ -1,7 +1,7 @@
 /*
  * hash.c - hash table implementation
  *
- *   Copyright (c) 2000-2016  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2000-2018  Shiro Kawai  <shiro@acm.org>
  *
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -834,7 +834,8 @@ ScmDictEntry *Scm_HashIterNext(ScmHashIter *iter)
 
 static void hash_print(ScmObj obj, ScmPort *port, ScmWriteContext *ctx);
 
-SCM_DEFINE_BUILTIN_CLASS(Scm_HashTableClass, hash_print, NULL, NULL, NULL,
+SCM_DEFINE_BUILTIN_CLASS(Scm_HashTableClass, hash_print, Scm_ObjectCompare,
+                         NULL, NULL,
                          SCM_CLASS_DICTIONARY_CPL);
 
 ScmObj Scm_MakeHashTableSimple(ScmHashType type, unsigned int initSize)
@@ -883,6 +884,8 @@ ScmObj Scm_HashTableRef(ScmHashTable *ht, ScmObj key, ScmObj fallback)
     else    return SCM_DICT_VALUE(e);
 }
 
+/* Returns previous value; can return SCM_UNBOUND when the association hasn't
+   been there.  Be careful not to let SCM_UNBOUND leak out to Scheme! */
 ScmObj Scm_HashTableSet(ScmHashTable *ht, ScmObj key, ScmObj value, int flags)
 {
     ScmDictEntry *e;
@@ -891,12 +894,11 @@ ScmObj Scm_HashTableSet(ScmHashTable *ht, ScmObj key, ScmObj value, int flags)
                            (intptr_t)key,
                            (flags&SCM_DICT_NO_CREATE)?SCM_DICT_GET: SCM_DICT_CREATE);
     if (!e) return SCM_UNBOUND;
-    if (e->value) {
-        if (flags&SCM_DICT_NO_OVERWRITE) return SCM_DICT_VALUE(e);
-        else return SCM_DICT_SET_VALUE(e, value);
-    } else {
+    ScmObj oldval = e->value? SCM_DICT_VALUE(e) : SCM_UNBOUND;
+    if (!(flags&SCM_DICT_NO_OVERWRITE) || SCM_UNBOUNDP(oldval)) {
         return SCM_DICT_SET_VALUE(e, value);
     }
+    return oldval;
 }
 
 ScmObj Scm_HashTableDelete(ScmHashTable *ht, ScmObj key)
