@@ -1,7 +1,7 @@
 ;;;
 ;;; libeval.scm - eval, load and related stuff
 ;;;
-;;;   Copyright (c) 2000-2016  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2000-2018  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -36,11 +36,9 @@
 (inline-stub
  (declcode (.include <gauche/vminsn.h>
                      <gauche/class.h>
-                     <gauche/priv/macroP.h>
                      <gauche/priv/readerP.h>)))
 
-(declare (keep-private-macro autoload add-load-path
-                             define-compiler-macro))
+(declare (keep-private-macro autoload add-load-path))
 
 ;;;
 ;;; Eval
@@ -115,7 +113,7 @@
         [prev-read-context (current-read-context)])
 
     (define (setup-load-context)
-      (when (port-closed? port) (error "port alrady closed:" port))
+      (when (port-closed? port) (error "port already closed:" port))
       (%port-lock! port)
       (when environment (vm-set-current-module environment))
       (current-load-port port)
@@ -182,6 +180,24 @@
 (define-cproc %load-verbose? () ::<boolean>
   (return (SCM_VM_RUNTIME_FLAG_IS_SET (Scm_VM) SCM_LOAD_VERBOSE)))
 
+;; Called from Scm_DynLoad to get initfn name, which always begins with #\_.
+;; If INITFN is given, we just add "_" in front of it.  Otherwise we
+;; derive it from the name of DSO.
+(select-module gauche.internal)
+(define (%get-initfn-name initfn dsopath)
+  (if (string? initfn)
+    (string-append "_" initfn)
+    (let* ([base (sys-basename dsopath)]
+           [stem (or (string-scan-right base #\. 'before) base)])
+      (string-append "_Scm_Init_"
+                     ;; string-map is in srfi-13, so we roll our own.
+                     ;; This looks awful, but we don't call this frequently.
+                     (list->string
+                      (map (^c (cond [(char-alphabetic? c) (char-downcase c)]
+                                     [(char-numeric? c) c]
+                                     [else #\_]))
+                           (string->list stem)))))))
+
 
 (select-module gauche)
 
@@ -243,13 +259,6 @@
                   path)])
     `',((with-module gauche.internal %add-load-path) path afterp)))
 
-;; Load path hooks
-(select-module gauche.internal)
-(define-cproc %add-load-path-hook! (proc :optional (after?::<boolean> #f))
-  ::<void> Scm_AddLoadPathHook)
-(define-cproc %delete-load-path-hook! (proc)
-  ::<void> Scm_DeleteLoadPathHook)
-
 ;; API: find-load-file
 ;;
 ;;   Core function to search specified file from the search path *PATH.
@@ -273,17 +282,27 @@
 ;;   The element in SUFFIXES is directly appended to the FILENAME;
 ;;   so usually it begins with dot.
 ;;
-;;   PATHs may contain a regular file, or an empty string "".
-;;   In which case, procedures chained to *load-path-hooks* are called
-;;   in turn.  It receives three arguments; the regular filename in PATHs
-;;   or "", the (partial) filename given to the find-load-file, and the
-;;   list of suffixes.  The hook is mainly intended to allow loading
+;;   PATHs may contain a regular file, or a path beginning with "@".
+;;   If find-load-file steps on such a path, it calls procedures chained
+;;   in *load-path-hooks* in turn.  The procedure receives three arguments:
+;;   The path (of a regular file, or beginning with "@"), the partial
+;;   filename given to the find-load-file, and the list of suffixes.
+;;   The hook is mainly intended to allow loading
 ;;   from archive files or from special location (e.g. prelinked in the
 ;;   executing binary).   If the hook procedure "finds"
 ;;   the searched file in the archive file, it should return a pair of
 ;;   the canonical filename (given filename plus suffix if applicable), and
-;;   a thunk that opens and returns a port to read the file.  If the hook
-;;   procedure doesn't find the searched file, it should return #f.
+;;   a procedure that opens and returns a port to read the file.  The procedure
+;;   receives the canonical filename.
+;;   If the hook procedure doesn't find the searched file, it should return #f.
+;;
+;;   NB: For the backward compatibility, we also call load-path-hook
+;;   if PATHS contain an empty path "".  Its use is deprecated.
+;;
+;;   NB: Prefixing path with '@' is also used in configure, to indicate
+;;   that the system path should be relative to the location of the binary.
+;;   That prefix is resolved implicitly when the code queries system
+;;   paths, so won't be confused with load paths.
 ;;
 ;;   NB: find-file-in-paths in file.util is similar to this, but this one
 ;;   captures the exact behavior of `load'.
@@ -315,7 +334,9 @@
         (list found (cdr ps))
         (do-relative (cdr ps)))]
      [(and allow-archive
-           (or (equal? (car ps) "") (file-is-regular? (car ps))))
+           (or (file-is-regular? (car ps))
+               (equal? (car ps) "")     ; DEPRECATED USE
+               (eqv? #\@ (string-ref (car ps) 0))))
       (if-let1 r (any (^p (p (car ps) filename suffixes)) *load-path-hooks*)
         (list (car r) (cdr ps) (cdr r))
         (do-relative (cdr ps)))]
@@ -337,6 +358,46 @@
          (do-absolute filename)]
         [else (do-relative paths)]))
 
+;; Load path hooks
+(select-module gauche.internal)
+(define-cproc %add-load-path-hook! (proc :optional (after?::<boolean> #f))
+  ::<void> Scm_AddLoadPathHook)
+(define-cproc %delete-load-path-hook! (proc)
+  ::<void> Scm_DeleteLoadPathHook)
+
+;; Returns a procedure suitable for load path hook that searches the hashtable
+;; LIBTAB, that is keyed by partial pathname (e.g. foo/bar.scm) and has value
+;; which is the code as a string.
+;; NB: Be very careful not to call autoloaded procedures here.  The static linked
+;; version of libgauche depends on this to load Gauche standard libraries, so
+;; triggering autoload in it is a bad idea.
+(select-module gauche.internal)
+(define (make-embedded-code-loader libtab)
+  (^[archive-path name suffixes]
+    (and-let* ([prefix (and (not (equal? archive-path ""))
+                            (eqv? #\@ (string-ref archive-path 0))
+                            (string-copy archive-path 1))]
+               [path (if (equal? prefix "")
+                       name
+                       (string-append prefix "/" name))]
+               [fn (any (^[sfx]
+                          (let1 n (string-append path sfx)
+                            (and (hash-table-exists? libtab n) n)))
+                        suffixes)]
+               [content (hash-table-get libtab fn)])
+      (cons fn
+            (^_ (open-input-string content :name (string-append "@" fn)))))))
+
+;; This allows the caller to use the load path hook feature in a specific case
+;; (embedded code) without concerning its internals.
+;; LIBTAB is the same one as make-embedded-code-loader.
+;; Returns a thunk that deletes the added hook.
+(select-module gauche.internal)
+(define-in-module gauche (add-embedded-code-loader! libtab)
+  (let1 hook (make-embedded-code-loader libtab)
+    (%add-load-path-hook! hook)
+    (^[] (%delete-load-path-hook! hook))))
+
 ;;;
 ;;; Repl
 ;;;
@@ -370,64 +431,43 @@
        (loop1)))))
 
 ;;;
-;;; Macros
+;;; Kick 'main' procedure
+;;;   Returns an integer suitable for the exit code.
+;;;   This is mainly to display proper stack trace in case 'main'
+;;;   raises an error.
+(select-module gauche.internal)
+(define (run-main main args)
+  (guard (e [else (report-error e) 70])
+    (let1 r (main args)
+      (if (fixnum? r) r 70))))
+
+;;;
+;;; Invoke other version of gosh
+;;;   This is called when gosh gets -vVERSION option.
 ;;;
-
 (select-module gauche.internal)
-;; API
-(define-in-module gauche (macroexpand form)
-  (%internal-macro-expand form (make-cenv (vm-current-module)) #f))
-;; API
-(define-in-module gauche (macroexpand-1 form)
-  (%internal-macro-expand form (make-cenv (vm-current-module)) #t))
-
-(select-module gauche)
-;; API
-(define-cproc unwrap-syntax (form) Scm_UnwrapSyntax)
-
-(select-module gauche.internal)
-(inline-stub
- (define-type <macro> "ScmMacro*" "macro"
-   "SCM_MACROP" "SCM_MACRO" "SCM_OBJ")
- )
-
-;; These are used in the compiler, and hidden inside gauche.internal.
-(define-cproc macro? (obj) ::<boolean> SCM_MACROP)
-(define-cproc syntax? (obj) ::<boolean> SCM_SYNTAXP)
-
-(define-cproc make-macro-transformer (name::<symbol> proc::<procedure>)
-  Scm_MakeMacroTransformerOld)
-
-(define-cproc %make-macro-transformer (name::<symbol>? proc)
-  Scm_MakeMacro)
-
-(define-cproc compile-syntax-rules (name ellipsis literals rules mod env)
-  Scm_CompileSyntaxRules)
-
-(define-cproc macro-transformer (mac::<macro>) Scm_MacroTransformer)
-
-(define (call-macro-expander mac expr cenv)
-  (let1 r ((macro-transformer mac) expr cenv)
-    (if (and (pair? r) (not (eq? expr r)))
-      (rlet1 p (if (extended-pair? r)
-                 r
-                 (extended-cons (car r) (cdr r)))
-        (pair-attribute-set! p 'macro-input expr))
-      r)))
-
-(define-cproc make-syntax (name::<symbol> proc)
-  Scm_MakeSyntax)
-
-(define-cproc make-syntactic-closure (env literals expr)
-  Scm_MakeSyntacticClosure)
-
-(define-cproc call-syntax-handler (syn program cenv)
-  (SCM_ASSERT (SCM_SYNTAXP syn))
-  (return (Scm_VMApply2 (-> (SCM_SYNTAX syn) handler) program cenv)))
-
-(define-cproc syntax-handler (syn)
-  (SCM_ASSERT (SCM_SYNTAXP syn))
-  (return (-> (SCM_SYNTAX syn) handler)))
+(define (%invoke-other-version version args)
+  ;; Note: We assume other versions of gauche is installed under the
+  ;; same exec_prefix: $exec_prefix/lib/gauche-ABIVERSION/VERSION/ARCH/
+  (let* ([prefix (sys-normalize-pathname
+                  (string-append (gauche-architecture-directory) "/../../..")
+                  :canonicalize #t)]
+         [globpath (string-append prefix "/gauche-*/" version "/"
+                                  (gauche-architecture) "/gosh")]
+         [goshes (glob globpath)]
+         ;; Remove -v option from args
+         [args (let loop ([args (cdr args)]
+                          [r '()])
+                 (cond [(null? args) (reverse r)]
+                       [(equal? (car args) "-v") (loop (cddr args) r)]
+                       [(#/^-v/ (car args)) (loop (cdr args) r)]
+                       [else (loop (cdr args) (cons (car args) r))]))])
+    (unless (pair? goshes)
+      (format (current-error-port)
+              "No installed Gauche with version ~a under ~a.\n"
+              version prefix)
+      (exit 1))
+    (sys-exec (car goshes) (cons (car goshes) args))))
 
 ;;;
 ;;; System termination
@@ -617,7 +657,7 @@
   (let loop ([obj obj] [r '()])
     (if (pair? obj)
       (let1 si (pair-attribute-get obj 'source-info '(#f #f))
-        (if-let1 mi (pair-attribute-get obj 'macro-input #f)
+        (if-let1 mi (pair-attribute-get obj 'original #f)
           (loop mi `((,(car si) ,(cadr si) ,obj) ,@r))
           (reverse r `((,(car si) ,(cadr si) ,obj)))))
       '())))
@@ -659,39 +699,3 @@
 (define (%vm-parameter-set! index id value)
   ((with-module gauche.internal %vm-parameter-set!) index #f value))
 
-;;;
-;;;Tentative compiler macro
-;;;
-
-;;
-;;  (define-compiler-macro <name> <transformer>)
-;;
-;;  The <transformer> is the same as macro transformers, except that
-;;  <transformer> can return the given form untouched if it gives up
-;;  expansion.
-;;
-;;  For the backward compatilibity, the following form is also recognized
-;;  as <transfomer>:
-;;
-;;    (er-transformer
-;;     (lambda (form rename compare) ...)))
-;;
-;;  This resembles er-macro-transformer, but in fact we dispatch to a
-;;  half-baked temporary implementation of macro transformer; it is
-;;  for internal optimization experiment until the 'real' macro system
-;;  is in place.   It'll be dropped once we have revised macro system,
-;;  and user code shouldn't use it.
-
-(select-module gauche)
-
-;; API
-(define-macro (define-compiler-macro name xformer-spec)
-  (if (and (= (length xformer-spec) 2)
-           (eq? (unwrap-syntax (car xformer-spec)) 'er-transformer))
-    `((with-module gauche.internal %bind-inline-er-transformer)
-      (current-module) ',name ,(cadr xformer-spec))
-    `((with-module gauche.internal %attach-inline-transformer)
-      (current-module) ',name
-      (^[form cenv]
-        ((with-module gauche.internal call-macro-expander)
-         ,xformer-spec form cenv)))))
