@@ -1,7 +1,7 @@
 ;;;
 ;;; libfmt.scm - format
 ;;;
-;;;   Copyright (c) 2013-2016  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2013-2018  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -45,11 +45,16 @@
 ;; Directive = String
 ;;         | (S flags mincol colinc minpad padchar maxcol)
 ;;         | (A flags mincol colinc minpad padchar maxcol)
+;;         | (C flags)
+;;         | (W flags)
 ;;         | (D flags mincol padchar commachar interval)
 ;;         | (B flags mincol padchar commachar interval)
 ;;         | (O flags mincol padchar commachar interval)
 ;;         | (X flags mincol padchar commachar interval)
 ;;         | (x flags mincol padchar commachar interval)
+;;         | (R flags mincol padchar commachar interval)
+;;         | (r flags mincol padchar commachar interval)
+;;         | (F flags width digits scale ovfchar padchar)
 ;;         | (* flags count)
 ;; flags : '() | (Cons #\@ flags) | (Cons #\: flags)
 
@@ -59,9 +64,10 @@
 (define formatter-lex
   (let ()
     (define (fmtstr p) (port-attribute-ref p 'format-string))
-    (define (directive? c) (string-scan "sSaAdDbBoOxXrR*" c))
+    (define (directive? c) (string-scan "sSaAcCwWdDbBoOxXrR*fF" c))
     (define directive-param-spec ; (type max-#-of-params)
-      '((S 5) (A 5) (D 4) (B 4) (O 4) (X 4) (x 4) (* 1) (R 5) (r 5)))
+      '((S 5) (A 5) (W 0) (C 0)
+        (D 4) (B 4) (O 4) (X 4) (x 4) (* 1) (R 5) (r 5) (F 5)))
     (define (flag? c) (memv c '(#\@ #\:)))
     (define (next p)
       (rlet1 c (read-char p)
@@ -170,11 +176,14 @@
 ;;      | (Seq Tree ...)
 ;;      | (S flags mincol colinc minpad padchar maxcol)
 ;;      | (A flags mincol colinc minpad padchar maxcol)
+;;      | (C flags)
+;;      | (W flags)
 ;;      | (D flags mincol padchar commachar interval)
 ;;      | (B flags mincol padchar commachar interval)
 ;;      | (O flags mincol padchar commachar interval)
 ;;      | (X flags mincol padchar commachar interval)
 ;;      | (x flags mincol padchar commachar interval)
+;;      | (F flags width digits scale ovfchar padchar)
 ;;      | (* flags count)
 ;;
 ;; argcnt : An integer if the formatter takes fixed number of arguments,
@@ -325,7 +334,8 @@
                ,@body))
            (^[argptr port ctrl] ,@body))))))
            
-;; ~S and ~A
+;; ~S, ~A, ~W
+;; NB: ~W doesn't take params, so justifying part won't be called for ~W.
 (define (make-format-expr fmtstr params flags printer)
   (if (null? params)
     (^[argptr port ctrl]
@@ -365,6 +375,17 @@
            (display " ..." port))
     (display (substring str 0 limit) port)))
 
+;; ~C
+;; The "spelling out" mode (~:C) isn't supported yet.
+(define (make-format-char fmtstr flags)
+  (define (char-formatter writer)
+    (^[argptr port ctrl]
+      (let1 c (fr-next-arg! fmtstr argptr)
+        (unless (char? c) 
+          (error "Character required for ~c format directive, but got:" c))
+        (writer c port))))
+  (char-formatter (if (has-@? flags) write display)))
+
 ;; ~D, ~B, ~O, ~X, ~nR
 (define (make-format-num fmtstr params flags radix upcase)
   (if (and (null? params) (no-flag? flags))
@@ -414,9 +435,43 @@
       (string-append (insert pre) (string point) post)
       (insert str))))
 
-;; ~R
+;; ~R (roman numerals)
 (define (make-format-r src flags upper)
   (^[argptr port ctrl] (error "not implemented yet")))
+
+;; ~F, ~E, ~G  ; we only support ~F for now
+;; kind is 'E 'F or 'G
+;; @ flag is used to force plus sign (CL)
+;; : flag is used for notational rounding (Gauche only)
+(define (make-format-flo fmtstr params flags kind)
+  (define (do-fmt n w d ovchar padchar flags port)
+    (let* ([s (number->string n 10
+                              (cond-list
+                               [(has-@? flags) 'plus]
+                               [(has-:? flags) 'notational])
+                              d)]
+           [l (string-length s)])
+      (if (< w l)
+        (if ovchar
+          (dotimes [w] (write-char ovchar port))
+          (display s port))
+        (begin
+          (dotimes [(- w l)] (write-char padchar port))
+          (display s port)))))
+  ($ with-format-params ([width 0]
+                         [digits -1]
+                         [scale 0]
+                         [ovchar #f]
+                         [padchar #\space])
+     (let1 arg (fr-next-arg! fmtstr argptr)
+       (if (real? arg)
+         (do-fmt (inexact (* arg (expt 10 scale)))
+                 width digits ovchar padchar flags port)
+         (let1 sarg (write-to-string arg display)
+           (let1 len (string-length sarg)
+             (when (< len width)
+               (dotimes [(- width len)] (write-char padchar port)))
+             (display sarg port)))))))
            
 ;; ~*
 (define (make-format-jump fmtstr params flags)
@@ -436,11 +491,14 @@
                       $ map (cut formatter-compile-rec src <>) rest)]
     [('S fs . ps) (make-format-expr src ps fs write)]
     [('A fs . ps) (make-format-expr src ps fs display)]
+    [('C fs . ps) (make-format-char src fs)]
+    [('W fs . ps) (make-format-expr src ps fs write-shared)]
     [('D fs . ps) (make-format-num src ps fs 10 #f)]
     [('B fs . ps) (make-format-num src ps fs 2 #f)]
     [('O fs . ps) (make-format-num src ps fs 8 #f)]
     [('x fs . ps) (make-format-num src ps fs 16 #f)]
     [('X fs . ps) (make-format-num src ps fs 16 #t)]
+    [('F fs . ps) (make-format-flo src ps fs 'F)]
     [((or 'R 'r) fs . ps)
      (let ([upper (eq? (car tree) 'R)])
        (if (null? ps)
