@@ -1,7 +1,7 @@
 ;;;
 ;;; libio.scm - builtin port and I/O procedures
 ;;;
-;;;   Copyright (c) 2000-2016  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2000-2018  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -328,13 +328,30 @@
                                   TRUE (& bufrec)))))
 
 ;; String ports (srfi-6)
+;;   By default, string ports are named as "(input string port)" and
+;;   "(output string port)", which aren't very informative.  The caller
+;;   can specify alternative name with :name keyword argument. NB: Currently,
+;;   port name is assumed to be a pathname if it doesn't match #/^\(.*\)$/.
+;;   This convention may be replaced by more reliable mechanism to determine
+;;   port source path.  Until then, be careful to name the ports.
 (select-module gauche)
 
-(define-cproc open-input-string (string::<string> :key (private?::<boolean> #f))
-  Scm_MakeInputStringPort)
+(define-cproc open-input-string (string::<string> :key (private?::<boolean> #f)
+                                                       name)
+  (let* ([p (Scm_MakeInputStringPort string private?)])
+    ;; NB: We don't have an API to set port name in the constructor; but
+    ;; this is a bad manner.  Don't duplicate this.
+    (unless (SCM_UNBOUNDP name)
+      (set! (-> (SCM_PORT p) name) name))
+    (return p)))
 
-(define-cproc open-output-string (:key (private?::<boolean> #f))
-  Scm_MakeOutputStringPort)
+(define-cproc open-output-string (:key (private?::<boolean> #f) name)
+  (let* ([p (Scm_MakeOutputStringPort private?)])
+    ;; NB: We don't have an API to set port name in the constructor; but
+    ;; this is a bad manner.  Don't duplicate this.
+    (unless (SCM_UNBOUNDP name)
+      (set! (-> (SCM_PORT p) name) name))
+    (return p)))
 
 (define-cproc get-output-string (oport::<output-port>) ;SRFI-6
   (return (Scm_GetOutputString oport 0)))
@@ -521,6 +538,12 @@
             (get-output-string o))
           (begin (write-char c o) (loop (+ i 1))))))))
 
+(define (write-string string :optional (port (current-output-port))
+                                       (start 0)
+                                       (end -1))
+  (display ((with-module gauche.internal %maybe-substring) string start end)
+           port))
+
 ;; Consume trailing whiespaces up to (including) first EOL.
 ;; This is mainly intended for interactive REPL,
 ;; where the input is buffered by line.  We want to ignore the
@@ -797,48 +820,88 @@
  (define-cclass <write-controls>
    "ScmWriteControls*" "Scm_WriteControlsClass"
    ("Scm_TopClass")
-   ((print-length :type <int>     :c-name "printLength"
-                  :getter "if (obj->printLength < 0) return SCM_FALSE; \
-                           else return SCM_MAKE_INT(obj->printLength);"
-                  :setter "if (SCM_INTP(value) && SCM_INT_VALUE(value) >= 0) \
-                             obj->printLength = SCM_INT_VALUE(value); \
-                           else obj->printLength = -1;")
-    (print-level  :type <int>     :c-name "printLevel"
-                  :getter "if (obj->printLevel < 0) return SCM_FALSE; \
-                           else return SCM_MAKE_INT(obj->printLevel);"
-                  :setter "if (SCM_INTP(value) && SCM_INT_VALUE(value) >= 0) \
-                             obj->printLevel = SCM_INT_VALUE(value); \
-                           else obj->printLevel = -1;")
-    (print-base   :type <int>     :c-name "printBase"
-                  :setter "if (SCM_INTP(value) && SCM_INT_VALUE(value) >= 2 \
-                               && SCM_INT_VALUE(value) <= 36) \
-                             obj->printBase = SCM_INT_VALUE(value); \
-                           else Scm_Error(\"print-base must be an integer \
-                                   between 2 and 36, but got: %S\", value);")
-    (print-radix  :type <boolean> :c-name "printRadix"
-                  :setter "obj->printRadix = !SCM_FALSEP(value);"))
+   ((length :type <int>     :c-name "printLength"
+            :getter "if (obj->printLength < 0) return SCM_FALSE; \
+                     else return SCM_MAKE_INT(obj->printLength);"
+            :setter "if (SCM_INTP(value) && SCM_INT_VALUE(value) >= 0) \
+                       obj->printLength = SCM_INT_VALUE(value); \
+                     else obj->printLength = -1;")
+    (level  :type <int>     :c-name "printLevel"
+            :getter "if (obj->printLevel < 0) return SCM_FALSE; \
+                     else return SCM_MAKE_INT(obj->printLevel);"
+            :setter "if (SCM_INTP(value) && SCM_INT_VALUE(value) >= 0) \
+                       obj->printLevel = SCM_INT_VALUE(value); \
+                     else obj->printLevel = -1;")
+    (width  :type <int>     :c-name "printWidth"
+            :getter "if (obj->printWidth < 0) return SCM_FALSE; \
+                     else return SCM_MAKE_INT(obj->printWidth);"
+            :setter "if (SCM_INTP(value) && SCM_INT_VALUE(value) >= 0) \
+                       obj->printWidth = SCM_INT_VALUE(value); \
+                     else obj->printWidth = -1;")
+    (base   :type <int>     :c-name "printBase"
+            :setter "if (SCM_INTP(value) && SCM_INT_VALUE(value) >= 2 \
+                         && SCM_INT_VALUE(value) <= 36) \
+                       obj->printBase = SCM_INT_VALUE(value); \
+                     else Scm_Error(\"print-base must be an integer \
+                                    between 2 and 36, but got: %S\", value);")
+    (radix  :type <boolean> :c-name "printRadix"
+            :setter "obj->printRadix = !SCM_FALSEP(value);")
+    (pretty :type <boolean> :c-name "printPretty"
+            :setter "obj->printPretty = !SCM_FALSEP(value);"))
    (allocator (c "write_controls_allocate")))
  ;; NB: Printer is defined in libobj.scm via write-object method
  )
 
-;; The :key and :rest combination is to catch invalid keyword error
-(define (make-write-controls :key print-length print-level
-                                  print-base print-radix
-                             :rest args)
-  (apply make <write-controls> args))
+;; TRANSIENT: The print-* keyword arguments for the backward compatibility
+(define (make-write-controls :key length level width base radix pretty
+                                  print-length print-level print-width
+                                  print-base print-radix print-pretty)
+  (define (arg k k-alt) (if (undefined? k-alt) k k-alt))
+  (make <write-controls>
+    :length (arg length print-length)
+    :level  (arg level  print-level)
+    :width  (arg width  print-width)
+    :base   (arg base   print-base)
+    :radix  (arg radix  print-radix)
+    :pretty (arg pretty print-pretty)))
 
-(define (write-controls-copy wc :key print-length print-level
-                                     print-base print-radix)
+;; Returns fresh write-controls where the specified slot value is replaced
+;; from the original WC.
+;; NB: If the specified values doesn't change the original value at all,
+;; we don't bother to create a copy.  This assumes we treat WC immutable.
+;; (Maybe we should write this in C to avoid overhead.)
+;; TRANSIENT: The print-* keyword arguments for the backward compatibility
+(define (write-controls-copy wc :key length level width base radix pretty
+                                     print-length print-level print-width
+                                     print-base print-radix print-pretty)
   (let-syntax [(select
                 (syntax-rules ()
-                  [(_ k) (if (undefined? k)
-                           (slot-ref wc 'k)
-                           k)]))]
-    (make <write-controls>
-      :print-length (select print-length)
-      :print-level  (select print-level)
-      :print-base   (select print-base)
-      :print-radix  (select print-radix))))
+                  [(_ k k-alt)
+                   (if (undefined? k)
+                     (if (undefined? k-alt)
+                       (slot-ref wc 'k)
+                       k-alt)
+                     k)]))]
+    (let ([length (select length print-length)]
+          [level  (select level  print-level)]
+          [width  (select width  print-width)]
+          [base   (select base   print-base)]
+          [radix  (select radix  print-radix)]
+          [pretty (select pretty print-pretty)])
+      (if (and (eqv? length (slot-ref wc 'length))
+               (eqv? level  (slot-ref wc 'level))
+               (eqv? width  (slot-ref wc 'width))
+               (eqv? base   (slot-ref wc 'base))
+               (eqv? radix  (slot-ref wc 'radix))
+               (eqv? pretty (slot-ref wc 'pretty)))
+        wc
+        (make <write-controls>
+          :length length
+          :level  level
+          :width  width
+          :base   base
+          :radix  radix
+          :pretty pretty)))))
 
 ;;;
 ;;; With-something
