@@ -1,7 +1,7 @@
 ;;;
 ;;; liblist.scm - builtin list procedures
 ;;;
-;;;   Copyright (c) 2000-2016  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2000-2018  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -181,18 +181,17 @@
 (define-cproc last-pair (list) :constant Scm_LastPair)
 (define-cproc list-copy (list) Scm_CopyList)
 
-(define-cproc list* (:rest args)
+(define-cproc list* (arg :rest args)
   (inliner LIST-STAR)
-  (let* ([head '()] [tail '()])
-    (when (SCM_PAIRP args)
+  (if (SCM_NULLP args)
+    (return arg)
+    (let* ([head (SCM_LIST1 arg)] [tail head])
       (dopairs [cp args]
         (unless (SCM_PAIRP (SCM_CDR cp))
-          (if (SCM_NULLP head)
-            (set! head (SCM_CAR cp))
-            (SCM_SET_CDR tail (SCM_CAR cp)))
+          (SCM_SET_CDR tail (SCM_CAR cp))
           (break))
-        (SCM_APPEND1 head tail (SCM_CAR cp))))
-    (return head)))
+        (SCM_APPEND1 head tail (SCM_CAR cp)))
+      (return head))))
 
 (define-cproc append! (:rest list)
   (let* ([h '()] [t '()])
@@ -204,7 +203,9 @@
           (set! h (SCM_CAR cp))
           (SCM_SET_CDR t (SCM_CAR cp)))
         (break))
-      (SCM_APPEND h t (SCM_CAR cp)))
+      (SCM_APPEND h t (SCM_CAR cp))
+      (unless (or (SCM_NULLP t) (SCM_NULLP (SCM_CDR t)))
+        (Scm_Error "proper list required, but got %S" (SCM_CAR cp))))
     (return h)))
 
 (define-cproc reverse! (list :optional (tail ())) Scm_Reverse2X)
@@ -242,7 +243,7 @@
         [(pair? l) #f]
         [else (error "argument must be a list, but got:" l)]))
 
-(define cons* list*)                    ;srfi-1
+(define-inline cons* list*)             ;srfi-1
 
 (define (last lis) (car (last-pair lis))) ;srfi-1
 
