@@ -1,7 +1,7 @@
 ;;;
 ;;; libnum.stub - builtin number libraries
 ;;;
-;;;   Copyright (c) 2000-2016  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2000-2018  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -37,6 +37,7 @@
  (declcode (.include <gauche/vminsn.h>
                      <gauche/bignum.h>
                      <stdlib.h>
+                     <float.h>
                      <math.h>))
  (when "!defined(M_PI)"
    (declcode "#define M_PI 3.1415926535897932384")))
@@ -97,6 +98,13 @@
 ;; default-endian is defined in Scm__InitNumber().
 (define-cproc native-endian () Scm_NativeEndian)
 
+;; DBL_EPSILON
+(define-cproc flonum-epsilon () 
+  (let* ([eps::(static ScmObj) SCM_UNBOUND])
+    (when (== eps SCM_UNBOUND)
+      (set! eps (Scm_MakeFlonum DBL_EPSILON)))
+    (return eps)))
+
 (select-module gauche.internal)
 (define-cproc %bignum-dump (obj) ::<void>
   (when (SCM_BIGNUMP obj)
@@ -175,12 +183,34 @@
 (define inexact exact->inexact)           ;R6RS
 
 (select-module scheme)
-(define-cproc number->string
-  (obj :optional (radix::<fixnum> 10) (use-upper? #f)) :fast-flonum :constant
-  (return (Scm_NumberToString obj radix
-                              (?: (SCM_FALSEP use_upperP)
-                                  0
-                                  SCM_NUMBER_FORMAT_USE_UPPER))))
+(define-cproc number->string (obj :optional (radix::<fixnum> 10)
+                                            (flags #f)
+                                            (precision::<fixnum> -1))
+  :fast-flonum :constant
+  (let* ([f::u_long 0]
+         [fmt::ScmNumberFormat]
+         [o::ScmPort* (SCM_PORT (Scm_MakeOutputStringPort TRUE))])
+    (cond [(or (SCM_FALSEP flags) (SCM_NULLP flags)) (set! f 0)]
+          [(SCM_TRUEP flags) (set! f SCM_NUMBER_FORMAT_USE_UPPER)];compatibility
+          [(SCM_PAIRP flags)
+           (unless (SCM_FALSEP (Scm_Memq 'uppercase flags))
+             (logior= f SCM_NUMBER_FORMAT_USE_UPPER))
+           (unless (SCM_FALSEP (Scm_Memq 'plus flags))
+             (logior= f SCM_NUMBER_FORMAT_SHOW_PLUS))
+           (unless (SCM_FALSEP (Scm_Memq 'radix flags))
+             (logior= f SCM_NUMBER_FORMAT_ALT_RADIX))
+           (unless (SCM_FALSEP (Scm_Memq 'notational flags))
+             (logior= f SCM_NUMBER_FORMAT_ROUND_NOTATIONAL))]
+          [else
+           (Scm_Error "flags argument must be a list of symbols (uppercase, \
+                       plus, radix, notational) or a boolean, but got: %S"
+                      flags)])
+    (Scm_NumberFormatInit (& fmt))
+    (set! (ref fmt radix) radix)
+    (set! (ref fmt flags) f)
+    (set! (ref fmt precision) precision)
+    (Scm_PrintNumber o obj (& fmt))
+    (return (Scm_GetOutputString o 0))))
 
 (define-cproc string->number (obj::<string> :optional (radix::<fixnum> 10))
   (return (Scm_StringToNumber obj radix 0)))
@@ -720,7 +750,7 @@
 ;; This can be (- (integer-length (logxor n (- n 1))) 1), but we can save
 ;; creating intermediate numbers by providing this natively.
 (define-cproc twos-exponent-factor (n) ::<int> :constant
-  (cond [(SCM_EQ n (SCM_MAKE_INT 0)) (return 0)]
+  (cond [(SCM_EQ n (SCM_MAKE_INT 0)) (return -1)]
         [(SCM_INTP n)
          (let* ([z::ScmBits (cast ScmBits (cast long (SCM_INT_VALUE n)))])
            (return (Scm_BitsLowest1 (& z) 0 SCM_WORD_BITS)))]
@@ -814,3 +844,20 @@
 
 (define-cproc magnitude (z) :fast-flonum :constant Scm_VMAbs)
 (define-cproc angle (z)     ::<double> :fast-flonum :constant Scm_Angle)
+
+;; Utility to recognize clamp mode symbols and returns C enum
+;; (Mostly used in uvector-related apis)
+
+(inline-stub
+ ;; NB: This is dupe from ext/uvector/uvector.scm
+ (define-cfn Scm_ClampMode (clamp) ::int
+   (cond [(SCM_EQ clamp 'both) (return SCM_CLAMP_BOTH)]
+         [(SCM_EQ clamp 'low)  (return SCM_CLAMP_LO)]
+         [(SCM_EQ clamp 'high) (return SCM_CLAMP_HI)]
+         [(not (or (SCM_FALSEP clamp) (SCM_UNBOUNDP clamp)))
+          (Scm_Error "clamp argument must be either 'both, 'high, 'low or #f, \
+                    but got %S" clamp)])
+   (return SCM_CLAMP_ERROR)))
+
+
+
