@@ -1,7 +1,7 @@
 /*
  * macro.c - macro implementation
  *
- *   Copyright (c) 2000-2016  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2000-2018  Shiro Kawai  <shiro@acm.org>
  *
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -36,6 +36,7 @@
 #include "gauche/code.h"
 #include "gauche/vminsn.h"
 #include "gauche/priv/macroP.h"
+#include "gauche/priv/identifierP.h"
 #include "gauche/priv/builtin-syms.h"
 
 /* avoid C++ reserved name conflict.
@@ -70,28 +71,37 @@ ScmObj Scm_MakeSyntax(ScmSymbol *name, ScmObj handler)
  * Macro object
  */
 
-static void macro_print(ScmObj obj, ScmPort *port, ScmWriteContext *mode)
-{
-    ScmSymbol *name = SCM_MACRO(obj)->name;
-    Scm_Printf(port, "#<macro %A>", (name ? SCM_OBJ(name) : SCM_FALSE));
-}
-
-SCM_DEFINE_BUILTIN_CLASS_SIMPLE(Scm_MacroClass, macro_print);
-
-ScmObj Scm_MakeMacro(ScmSymbol *name, ScmObj transformer)
+ScmObj Scm_MakeMacroFull(ScmObj name, ScmObj transformer,
+                         ScmObj src, ScmObj describer)
 {
     ScmMacro *s = SCM_NEW(ScmMacro);
     SCM_SET_CLASS(s, SCM_CLASS_MACRO);
     s->name = name;
     s->transformer = transformer;
+    s->src = src;
+    s->describer = describer;
     return SCM_OBJ(s);
 }
 
+#if GAUCHE_API_0_95
+/* Kept for the backward compatibility */
+ScmObj Scm_MakeMacro(ScmSymbol *name, ScmObj transformer)
+{
+    return Scm_MakeMacroFull(SCM_OBJ_SAFE(name), transformer,
+                             SCM_FALSE, SCM_FALSE);
+}
+#endif
+
 ScmObj Scm_MacroTransformer(ScmMacro *mac)
 {
     return mac->transformer;
 }
 
+ScmObj Scm_MacroName(ScmMacro *mac)
+{
+    return (mac->name? SCM_OBJ(mac->name) : SCM_FALSE);
+}
+
 /*===================================================================
  * SyntaxPattern object
  * Repesents a repeatable subpattern
@@ -189,11 +199,12 @@ static ScmObj macro_transform_old(ScmObj *argv, int argc, void *data)
     return Scm_VMApply(proc, SCM_CDR(form));
 }
 
+/* TRANSIENT
+   This used to be called via make-macro-transformer, but no longer.
+   We leave stub here for ABI compatibility. */
 ScmObj Scm_MakeMacroTransformerOld(ScmSymbol *name, ScmProcedure *proc)
 {
-    ScmObj transformer = Scm_MakeSubr(macro_transform_old, proc, 2, 0,
-                                      SCM_FALSE);
-    return Scm_MakeMacro(name, transformer);
+    Scm_Panic("Obsoleted Scm_MakeMacroTransformerOld called!  Something is wrong!");
 }
 
 static ScmMacro *resolve_macro_autoload(ScmAutoload *adata)
@@ -220,11 +231,18 @@ static ScmObj macro_autoload(ScmObj *argv, int argc, void *data)
     return Scm_CallMacroExpander(mac, form, env);
 }
 
+#define AUTOLOAD_MACRO_SUFFIX " (autoload)"
+
 ScmObj Scm_MakeMacroAutoload(ScmSymbol *name, ScmAutoload *adata)
 {
     ScmObj transformer = Scm_MakeSubr(macro_autoload, adata,
                                       2, 0, SCM_FALSE);
-    return Scm_MakeMacro(name, transformer);
+    ScmObj name1 = Scm_StringAppendC(SCM_SYMBOL_NAME(name),
+                                     AUTOLOAD_MACRO_SUFFIX,
+                                     sizeof(AUTOLOAD_MACRO_SUFFIX)-1,
+                                     sizeof(AUTOLOAD_MACRO_SUFFIX)-1);
+    return Scm_MakeMacroFull(Scm_MakeSymbol(SCM_STRING(name1), FALSE),
+                             transformer, SCM_FALSE, SCM_FALSE);
 }
 
 /*===================================================================
@@ -250,7 +268,6 @@ ScmObj Scm_MakeMacroAutoload(ScmSymbol *name, ScmAutoload *adata)
 
 /*-------------------------------------------------------------------
  * pattern language compiler
- *   - convert literals into identifiers
  *   - recognize repeatable subpatterns and replace it to SyntaxPattern node.
  *   - convert free symbols in the template into identifiers
  *   - convert pattern variables into LREF object.
@@ -261,12 +278,16 @@ ScmObj Scm_MakeMacroAutoload(ScmSymbol *name, ScmAutoload *adata)
 typedef struct {
     ScmObj name;                /* name of this macro (for error msg)*/
     ScmObj form;                /* form being compiled (for error msg) */
-    ScmObj literals;            /* list of literal identifiers */
+    ScmObj literals;            /* list of literals */
     ScmObj pvars;               /* list of (pvar . pvref) */
-    ScmObj ellipsis;            /* symbol/idendifier/keyword for ellipsis */
+    ScmObj renames;             /* list of (var . identifier)  Keep mapping
+                                   of input symbol/identifier to fresh 
+                                   identifier */
+    ScmObj ellipsis;            /* symbol/identifier/keyword for ellipsis
+                                   SCM_TRUE means default (...)
+                                   SCM_FALSE means disabled */
     int pvcnt;                  /* counter of pattern variables */
     int maxlev;                 /* maximum level */
-    ScmObj tvars;               /* list of identifies inserted in template */
     ScmModule *mod;             /* module where this macro is defined */
     ScmObj env;                 /* compiler env of this macro definition */
 } PatternContext;
@@ -275,14 +296,23 @@ typedef struct {
 #define PVREF_LEVEL(pvref)     (int)SCM_PVREF_LEVEL(pvref)
 #define PVREF_COUNT(pvref)     (int)SCM_PVREF_COUNT(pvref)
 
+#define PVREF_LEVEL_MAX 0xff
+#define PVREF_COUNT_MAX 0xff
+
 /* add pattern variable pvar.  called when compiling a pattern */
 static inline ScmObj add_pvar(PatternContext *ctx,
                               ScmSyntaxPattern *pat,
                               ScmObj pvar)
 {
+    if (pat->level > PVREF_LEVEL_MAX) {
+        Scm_Error("Pattern levels too deeply nested in the macro definition of %S", ctx->name);
+    }
+    if (ctx->pvcnt > PVREF_COUNT_MAX) {
+        Scm_Error("Too many pattern variables in the macro definition of %S", ctx->name);
+    }
     ScmObj pvref = SCM_MAKE_PVREF(pat->level, ctx->pvcnt);
     if (!SCM_FALSEP(Scm_Assq(pvar, ctx->pvars))) {
-        Scm_Error("pattern variable %S appears more than once in the macro definition of %S: %S",
+        Scm_Error("Pattern variable %S appears more than once in the macro definition of %S: %S",
                   pvar, ctx->name, ctx->form);
     }
     ctx->pvcnt++;
@@ -315,42 +345,14 @@ static inline ScmObj pvref_to_pvar(PatternContext *ctx, ScmObj pvref)
     return SCM_CAR(q);
 }
 
-/* search an identifier with name NAME from a list of identifiers */
-static ScmObj id_memq(ScmObj name, ScmObj list)
-{
-    ScmObj n;
-    if (SCM_IDENTIFIERP(name)) {
-        n = SCM_OBJ(SCM_IDENTIFIER(name)->name);
-    } else {
-        n = name;
-    }
-    ScmObj lp;
-    SCM_FOR_EACH(lp, list) {
-        if (SCM_OBJ(SCM_IDENTIFIER(SCM_CAR(lp))->name) == n)
-            return SCM_CAR(lp);
-    }
-    return SCM_FALSE;
-}
-
-/* Check if obj is ellipsis.  What we really need is free-identifier=?,
-   but we leave it for the new low-level macro subsystem.  This is a
-   hack to make it work in most of the time. */
 static int isEllipsis(PatternContext *ctx, ScmObj obj)
 {
-    if (SCM_FALSEP(ctx->ellipsis)) return FALSE;
-    if (SCM_IDENTIFIERP(obj)) {
-        if (SCM_IDENTIFIERP(ctx->ellipsis)) {
-            static ScmObj free_identifier_eq_proc = SCM_UNDEFINED;
-            SCM_BIND_PROC(free_identifier_eq_proc, "free-identifier=?",
-                          Scm_GaucheInternalModule());
-            if (!SCM_NULLP(SCM_IDENTIFIER(obj)->env)) return FALSE;
-            return !SCM_FALSEP(Scm_ApplyRec2(free_identifier_eq_proc,
-                                             ctx->ellipsis, obj));
-        } else {
-            return SCM_EQ(ctx->ellipsis,
-                          SCM_OBJ(Scm_UnwrapIdentifier(SCM_IDENTIFIER(obj))));
-        }
+    if (SCM_FALSEP(ctx->ellipsis)) return FALSE; /* inside (... TEMPLATE) */
+    if (SCM_TRUEP(ctx->ellipsis)) {
+        /* default ellipsis (...) */
+        return Scm__ERCompare(SCM_SYM_ELLIPSIS, obj, ctx->mod, ctx->env);
     } else {
+        /* specified ellipsis */
         return SCM_EQ(ctx->ellipsis, obj);
     }
 }
@@ -362,16 +364,13 @@ static int isEllipsis(PatternContext *ctx, ScmObj obj)
     Scm_Error("Bad ellipsis usage in macro definition of %S: %S",       \
                Ctx->name, Ctx->form)
 
-/* convert literal symbols into identifiers */
-static ScmObj preprocess_literals(ScmObj literals, ScmModule *mod, ScmObj env)
+static ScmObj check_literals(ScmObj literals)
 {
     ScmObj lp, h = SCM_NIL, t = SCM_NIL;
     SCM_FOR_EACH(lp, literals) {
         ScmObj lit = SCM_CAR(lp);
-        if (SCM_IDENTIFIERP(lit))
+        if (SCM_SYMBOLP(lit) || SCM_IDENTIFIERP(lit))
             SCM_APPEND1(h, t, lit);
-        else if (SCM_SYMBOLP(lit))
-            SCM_APPEND1(h, t, Scm_MakeIdentifier(lit, mod, env));
         else if (SCM_KEYWORDP(lit))
             /* This branch is to allow r7rs-compliant code to go through
                with legacy mode.  If keyword-symbol integration is turned on,
@@ -383,14 +382,33 @@ static ScmObj preprocess_literals(ScmObj literals, ScmModule *mod, ScmObj env)
     return h;
 }
 
-/* compile a pattern or a template.
-   In a pattern, replace literal symbols for identifiers; leave
-   non-literal symbols (i.e. pattern variables) as they are, but
-   records it's presence in the context.   Also, when encounters
-   a repeatable subpattern, replace it with SyntaxPattern node.
-   In a template, replace symbols for identifiers except pattern variables.
-*/
+/* Renaming: Map input variable (symbol or identifier) to fresh identifier.
+   The same (eq?) variable must map to the same identifier. */
+static ScmObj rename_variable(ScmObj var,
+                              ScmObj *id_alist, /* ((var . id) ...) */
+                              ScmModule *mod,
+                              ScmObj env)
+{
+    ScmObj id, p = Scm_Assq(var, *id_alist);
+    if (SCM_PAIRP(p)) return SCM_CDR(p);
+    if (SCM_SYMBOLP(var)) {
+        id = Scm_MakeIdentifier(var, mod, env);
+    } else {
+        SCM_ASSERT(SCM_IDENTIFIERP(var));
+        id = Scm_WrapIdentifier(SCM_IDENTIFIER(var));
+    }
+    *id_alist = Scm_Acons(var, id, *id_alist);
+    return id;
+}
 
+
+/* Compile a pattern or a template.
+   In the pattern, we replace variables to identifiers.
+   We also recognize pattern variables, and replace them for PVARs in
+   the pattern, and for PVREFs in the template.
+   When encounters a repeatable subpattern, replace it with 
+   SyntaxPattern node.
+*/
 static ScmObj compile_rule1(ScmObj form,
                             ScmSyntaxPattern *spat,
                             PatternContext *ctx,
@@ -417,40 +435,62 @@ static ScmObj compile_rule1(ScmObj form,
         ScmObj pp;
         SCM_FOR_EACH(pp, form) {
             if (ELLIPSIS_FOLLOWING(pp, ctx)) {
-                int num_trailing = 0;
-
                 if (patternp && ellipsis_seen) {
                     Scm_Error("in definition of macro %S: "
                               "Ellipses are not allowed to appear "
                               "within the same list/vector more than once "
                               "in a pattern: %S", ctx->name, form);
-                    ellipsis_seen = TRUE;
                 }
+                ellipsis_seen = TRUE;
 
+                ScmObj base = SCM_CAR(pp);
+                pp = SCM_CDR(pp);
+
+                int num_trailing = 0;
+                int ellipsis_nesting = 1;
                 if (patternp) {
                     /* Count trailing items to set ScmSyntaxPattern->repeat. */
-                    ScmObj trailing = SCM_CDDR(pp);
+                    ScmObj trailing = SCM_CDR(pp);
                     while (SCM_PAIRP(trailing)) {
                         num_trailing++;
                         trailing = SCM_CDR(trailing);
                     }
+                } else {
+                    /* srfi-149 allows more than one ellipsis to follow a
+                       template. */
+                    while (ELLIPSIS_FOLLOWING(pp, ctx)) {
+                        ellipsis_nesting++;
+                        if (!SCM_PAIRP(SCM_CDR(pp))) break;
+                        pp = SCM_CDR(pp);
+                    }
                 }
-                ScmSyntaxPattern *nspat = make_syntax_pattern(spat->level + 1,
-                                                              num_trailing);
-                if (ctx->maxlev <= spat->level) ctx->maxlev++;
-                nspat->pattern = compile_rule1(SCM_CAR(pp), nspat, ctx,
-                                               patternp);
-                SCM_APPEND1(h, t, SCM_OBJ(nspat));
+                /* at this point, pp points the last ellipsis */
+
+                if (ctx->maxlev < spat->level + ellipsis_nesting)
+                    ctx->maxlev = spat->level + ellipsis_nesting + 1;
+
+                ScmSyntaxPattern *outermost =
+                    make_syntax_pattern(spat->level+1, num_trailing);
+                ScmSyntaxPattern *outer = outermost;
+                for (int i=1; i<ellipsis_nesting; i++) {
+                    ScmSyntaxPattern *inner =
+                        make_syntax_pattern(spat->level+i+1, 0);
+                    outer->pattern = SCM_OBJ(inner);
+                    outer = inner;
+                }
+                outer->pattern = compile_rule1(base, outer, ctx, patternp);
+                outermost->vars = outer->vars;
+                SCM_APPEND1(h, t, SCM_OBJ(outermost));
                 if (!patternp) {
                     ScmObj vp;
-                    if (SCM_NULLP(nspat->vars)) {
+                    if (SCM_NULLP(outermost->vars)) {
                         Scm_Error("in definition of macro %S: "
                                   "a template contains repetition "
                                   "of constant form: %S",
                                   ctx->name, form);
                     }
-                    SCM_FOR_EACH(vp, nspat->vars) {
-                        if (PVREF_LEVEL(SCM_CAR(vp)) >= nspat->level) break;
+                    SCM_FOR_EACH(vp, outermost->vars) {
+                        if (PVREF_LEVEL(SCM_CAR(vp)) >= outermost->level) break;
                     }
                     if (SCM_NULLP(vp)) {
                         Scm_Error("in definition of macro %S: "
@@ -459,8 +499,7 @@ static ScmObj compile_rule1(ScmObj form,
                                   ctx->name, form);
                     }
                 }
-                spat->vars = Scm_Append2(spat->vars, nspat->vars);
-                pp = SCM_CDR(pp);
+                spat->vars = Scm_Append2(spat->vars, outermost->vars);
             } else {
                 SCM_APPEND1(h, t,
                             compile_rule1(SCM_CAR(pp), spat, ctx, patternp));
@@ -476,35 +515,28 @@ static ScmObj compile_rule1(ScmObj form,
         ScmObj l = Scm_VectorToList(SCM_VECTOR(form), 0, -1);
         return Scm_ListToVector(compile_rule1(l, spat, ctx, patternp), 0, -1);
     }
-#if 0
-    else if (patternp && SCM_IDENTIFIERP(form)) {
-        /* this happens in a macro produced by another macro */
-        form = SCM_OBJ(SCM_IDENTIFIER(form)->name);
-    }
-#endif
     if (SCM_SYMBOLP(form)||SCM_IDENTIFIERP(form)) {
         if (isEllipsis(ctx, form)) BAD_ELLIPSIS(ctx);
-        ScmObj q = id_memq(form, ctx->literals);
-        if (!SCM_FALSEP(q)) return q;
-
+        if (!SCM_FALSEP(Scm_Memq(form, ctx->literals))) {
+            if (patternp)
+                return rename_variable(form, &ctx->renames, ctx->mod, ctx->env);
+            else        
+                return form;  /* template renaming is done in expansion time */
+        }
+        if (patternp && Scm__ERCompare(form, SCM_SYM_UNDERBAR,
+                                       ctx->mod, ctx->env)) { 
+            return SCM_SYM_UNDERBAR;
+        }
         if (patternp) {
             return add_pvar(ctx, spat, form);
         } else {
-            ScmObj id, pvref = pvar_to_pvref(ctx, spat, form);
+            ScmObj pvref = pvar_to_pvref(ctx, spat, form);
             if (pvref == form) {
-                /* form is not a pattern variable.  make it an identifier. */
-                if (!SCM_FALSEP(q = id_memq(form, ctx->tvars))) return q;
-                if (SCM_IDENTIFIERP(form)) {
-                    id = form;
-                } else {
-                    id = Scm_MakeIdentifier(form, ctx->mod, ctx->env);
-                }
-                ctx->tvars = Scm_Cons(id, ctx->tvars);
-                return id;
+                return form;
             } else {
                 spat->vars = Scm_Cons(pvref, spat->vars);
+                return pvref;
             }
-            return pvref;
         }
     }
     return form;
@@ -528,14 +560,30 @@ static ScmSyntaxRules *compile_rules(ScmObj name,
 
     ctx.name = name;
     ctx.ellipsis = ellipsis;
-    ctx.literals = preprocess_literals(literals, mod, env);
+    ctx.literals = check_literals(literals);
     ctx.mod = mod;
     ctx.env = env;
+    ctx.renames = SCM_NIL;
+
+    /* Corner case when literal contains the same (free-identifer=?) symbol
+       as ellipsis.  R7RS specifies the literal has precedence.
+       https://srfi-email.schemers.org/srfi-148/msg/6115633 */
+    if (!SCM_FALSEP(ellipsis)) {
+        ScmObj cp;
+        SCM_FOR_EACH(cp, ctx.literals) {
+            if (isEllipsis(&ctx, SCM_CAR(cp))) {
+                ctx.ellipsis = SCM_FALSE;
+                break;
+            }
+        }
+    }
 
     ScmSyntaxRules *sr = make_syntax_rules(numRules);
     sr->name = name;
     sr->numRules = numRules;
     sr->maxNumPvars = 0;
+    sr->mod = mod;
+    sr->env = env;
     ScmObj rp = rules;
     for (int i=0; i < numRules; i++, rp = SCM_CDR(rp)) {
         ScmObj rule = SCM_CAR(rp);
@@ -544,7 +592,6 @@ static ScmSyntaxRules *compile_rules(ScmObj name,
         ScmSyntaxPattern *pat  = make_syntax_pattern(0, 0);
         ScmSyntaxPattern *tmpl = make_syntax_pattern(0, 0);
         ctx.pvars = SCM_NIL;
-        ctx.tvars = SCM_NIL;
         ctx.pvcnt = 0;
         ctx.maxlev = 0;
 
@@ -723,29 +770,6 @@ static inline void match_insert(ScmObj pvref, ScmObj matched, MatchVar *mvec)
     }
 }
 
-/* see if literal identifier ID in the pattern matches the given object */
-static inline int match_identifier(ScmIdentifier *id, ScmObj obj,
-                                   ScmObj mod, ScmObj env)
-{
-    if (SCM_SYMBOLP(obj)) {
-        /* This is temporary: We don't want to allocate identifier for
-           every bare symbol we want to match.  But this is the shortcut
-           to do the right thing (using free-identifier=? to comapre literals.
-           Let's think optimization later. */
-        SCM_ASSERT(SCM_MODULEP(mod));
-        obj = Scm_MakeIdentifier(obj, SCM_MODULE(mod), env);
-    }
-    if (SCM_IDENTIFIERP(obj)) {
-        /* free-identifier=? is defined in Scheme (libmod.scm)  */
-        static ScmObj free_identifier_eq_proc = SCM_UNDEFINED;
-        SCM_BIND_PROC(free_identifier_eq_proc, "free-identifier=?",
-                      Scm_GaucheInternalModule());
-        return !SCM_FALSEP(Scm_ApplyRec2(free_identifier_eq_proc,
-                                         SCM_OBJ(id), obj));
-    }
-    return FALSE;
-}
-
 static inline int match_subpattern(ScmObj form, ScmSyntaxPattern *pat,
                                    ScmObj rest, ScmObj mod, ScmObj env,
                                    MatchVar *mvec)
@@ -779,8 +803,11 @@ static int match_synrule(ScmObj form, ScmObj pattern, ScmObj mod, ScmObj env,
         match_insert(pattern, form, mvec);
         return TRUE;
     }
+    if (SCM_EQ(pattern, SCM_SYM_UNDERBAR)) {
+        return TRUE;            /* unconditional match */
+    }
     if (SCM_IDENTIFIERP(pattern)) {
-        return match_identifier(SCM_IDENTIFIER(pattern), form, mod, env);
+        return Scm__ERCompare(pattern, form, SCM_MODULE(mod), env);
     }
     if (SCM_SYNTAX_PATTERN_P(pattern)) {
         return match_subpattern(form, SCM_SYNTAX_PATTERN(pattern),
@@ -849,7 +876,8 @@ static int match_synrule(ScmObj form, ScmObj pattern, ScmObj mod, ScmObj env,
  */
 
 /* If a pattern variable is exhausted, SCM_UNDEFINED is returned. */
-static ScmObj realize_template_rec(ScmObj template,
+static ScmObj realize_template_rec(ScmSyntaxRules *sr,
+                                   ScmObj template,
                                    MatchVar *mvec,
                                    int level,
                                    int *indices,
@@ -861,18 +889,18 @@ static ScmObj realize_template_rec(ScmObj template,
         while (SCM_PAIRP(template)) {
             ScmObj e = SCM_CAR(template);
             if (SCM_SYNTAX_PATTERN_P(e)) {
-                ScmObj r = realize_template_rec(e, mvec, level, indices, idlist, exlev);
+                ScmObj r = realize_template_rec(sr, e, mvec, level, indices, idlist, exlev);
                 if (SCM_UNBOUNDP(r)) return r;
                 SCM_APPEND(h, t, r);
             } else {
-                ScmObj r = realize_template_rec(e, mvec, level, indices, idlist, exlev);
+                ScmObj r = realize_template_rec(sr, e, mvec, level, indices, idlist, exlev);
                 if (SCM_UNBOUNDP(r)) return r;
                 SCM_APPEND1(h, t, r);
             }
             template = SCM_CDR(template);
         }
         if (!SCM_NULLP(template)) {
-            ScmObj r = realize_template_rec(template, mvec, level, indices, idlist, exlev);
+            ScmObj r = realize_template_rec(sr, template, mvec, level, indices, idlist, exlev);
             if (SCM_UNBOUNDP(r)) return r;
             if (SCM_NULLP(h)) return r; /* (a ... . b) and a ... is empty */
             SCM_APPEND(h, t, r);
@@ -887,9 +915,13 @@ static ScmObj realize_template_rec(ScmObj template,
         ScmObj h = SCM_NIL, t = SCM_NIL;
         indices[level+1] = 0;
         for (;;) {
-            ScmObj r = realize_template_rec(pat->pattern, mvec, level+1, indices, idlist, exlev);
+            ScmObj r = realize_template_rec(sr, pat->pattern, mvec, level+1, indices, idlist, exlev);
             if (SCM_UNBOUNDP(r)) return (*exlev < pat->level)? r : h;
-            SCM_APPEND1(h, t, r);
+            if (SCM_SYNTAX_PATTERN_P(pat->pattern)) {
+                SCM_APPEND(h, t, r);
+            } else {
+                SCM_APPEND1(h, t, r);
+            }
             indices[level+1]++;
         }
     }
@@ -900,37 +932,27 @@ static ScmObj realize_template_rec(ScmObj template,
 
         for (int i=0; i<len; i++, pe++) {
             if (SCM_SYNTAX_PATTERN_P(*pe)) {
-                ScmObj r = realize_template_rec(*pe, mvec, level, indices, idlist, exlev);
+                ScmObj r = realize_template_rec(sr, *pe, mvec, level, indices, idlist, exlev);
                 if (SCM_UNBOUNDP(r)) return r;
                 SCM_APPEND(h, t, r);
             } else {
-                ScmObj r = realize_template_rec(*pe, mvec, level, indices, idlist, exlev);
+                ScmObj r = realize_template_rec(sr, *pe, mvec, level, indices, idlist, exlev);
                 if (SCM_UNBOUNDP(r)) return r;
                 SCM_APPEND1(h, t, r);
             }
         }
         return Scm_ListToVector(h, 0, -1);
     }
-    if (SCM_IDENTIFIERP(template)) {
-        /* we wrap the identifier, so that the symbol bindings introduced
-           by recursive macro call won't interfere each other.
-           (e.g. the macro definitions of "letrec" and "do" shown in R5RS
-           use the fact that the symbol "newtemp" introduced in each
-           iteration of macro expansion are distinct. */
-        ScmObj p = Scm_Assq(template, *idlist);
-        if (SCM_PAIRP(p)) return SCM_CDR(p);
-        else {
-            ScmObj id = Scm_WrapIdentifier(SCM_IDENTIFIER(template));
-            *idlist = Scm_Acons(template, id, *idlist);
-            return id;
-        }
+    if (SCM_SYMBOLP(template) || SCM_IDENTIFIERP(template)) {
+        return rename_variable(template, idlist, sr->mod, sr->env);
     }
     return template;
 }
 
 #define DEFAULT_MAX_LEVEL  10
 
-static ScmObj realize_template(ScmSyntaxRuleBranch *branch,
+static ScmObj realize_template(ScmSyntaxRules *sr,
+                               ScmSyntaxRuleBranch *branch,
                                MatchVar *mvec)
 {
     int index[DEFAULT_MAX_LEVEL], *indices = index;
@@ -940,7 +962,8 @@ static ScmObj realize_template(ScmSyntaxRuleBranch *branch,
     if (branch->maxLevel > DEFAULT_MAX_LEVEL)
         indices = SCM_NEW_ATOMIC2(int*, (branch->maxLevel+1) * sizeof(int));
     for (int i=0; i<=branch->maxLevel; i++) indices[i] = 0;
-    return realize_template_rec(branch->template, mvec, 0, indices, &idlist, &exlev);
+    return realize_template_rec(sr, branch->template, mvec, 0,
+                                indices, &idlist, &exlev);
 }
 
 static ScmObj synrule_expand(ScmObj form, ScmObj mod, ScmObj env, ScmSyntaxRules *sr)
@@ -960,7 +983,7 @@ static ScmObj synrule_expand(ScmObj form, ScmObj mod, ScmObj env, ScmSyntaxRules
             Scm_Printf(SCM_CUROUT, "success #%d:\n", i);
             print_matchvec(mvec, sr->rules[i].numPvars, SCM_CUROUT);
 #endif
-            ScmObj expanded = realize_template(&sr->rules[i], mvec);
+            ScmObj expanded = realize_template(sr, &sr->rules[i], mvec);
 #ifdef DEBUG_SYNRULE
             Scm_Printf(SCM_CUROUT, "result: %S\n", expanded);
 #endif
@@ -984,7 +1007,7 @@ static ScmObj synrule_transform(ScmObj *argv, int argc, void *data)
 }
 
 /* NB: a stub for the new compiler (TEMPORARY) */
-ScmObj Scm_CompileSyntaxRules(ScmObj name, ScmObj ellipsis,
+ScmObj Scm_CompileSyntaxRules(ScmObj name, ScmObj src, ScmObj ellipsis,
                               ScmObj literals, ScmObj rules,
                               ScmObj mod, ScmObj env)
 {
@@ -994,7 +1017,7 @@ ScmObj Scm_CompileSyntaxRules(ScmObj name, ScmObj ellipsis,
                                        SCM_MODULE(mod), env);
     ScmObj sr_xform = Scm_MakeSubr(synrule_transform, sr,
                                    2, 0, SCM_FALSE);
-    return Scm_MakeMacro(SCM_SYMBOL(name), sr_xform);
+    return Scm_MakeMacroFull(name, sr_xform, src, SCM_FALSE);
 }
 
 /*===================================================================
