@@ -11,12 +11,15 @@
 
 (use gauche.collection)
 (use gauche.sequence)
+(use gauche.dictionary)
 (use gauche.hook)
 (use gauche.parameter)
 (use gauche.uvector)
 (use gauche.regexp)
 (use gauche.record)
 (use gauche.generator)
+(use gauche.interpolate)
+(use gauche.threads)
 (use srfi-1)
 (use srfi-13)
 (use file.util)
