@@ -1,7 +1,7 @@
 /*
  * read.c - reader
  *
- *   Copyright (c) 2000-2016  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2000-2018  Shiro Kawai  <shiro@acm.org>
  *
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -1468,7 +1468,8 @@ static ScmObj read_regexp(ScmPort *port)
 /* gauche extension :  #[charset] */
 static ScmObj read_charset(ScmPort *port)
 {
-    return Scm_CharSetRead(port, NULL, TRUE, FALSE);
+    ScmCharSet *cs = SCM_CHAR_SET(Scm_CharSetRead(port, NULL, TRUE, FALSE));
+    return Scm_CharSetFreezeX(cs); /* literal is immutable */
 }
 
 /*----------------------------------------------------------------
