@@ -1,7 +1,7 @@
 /*
  * string.c - string implementation
  *
- *   Copyright (c) 2000-2016  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2000-2018  Shiro Kawai  <shiro@acm.org>
  *
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -321,68 +321,31 @@ ScmObj Scm_CopyStringWithFlags(ScmString *x, int flags, int mask)
     return SCM_OBJ(make_str(len, size, start, newflags));
 }
 
+/* OBSOLETED */
 ScmObj Scm_StringCompleteToIncomplete(ScmString *x)
 {
-    return Scm_CopyStringWithFlags(x, SCM_STRING_INCOMPLETE,
-                                   SCM_STRING_INCOMPLETE);
+    Scm_Warn("Obsoleted C API Scm_StringCompleteToIncomplete called");
+    ScmObj proc = SCM_UNDEFINED;
+    SCM_BIND_PROC(proc, "string-complete->incomplete", Scm_GaucheModule());
+    return Scm_ApplyRec1(proc, SCM_OBJ(x));
 }
 
+/* OBSOLETED */
 ScmObj Scm_StringIncompleteToComplete(ScmString *x,
                                       int handling,
                                       ScmChar substitute)
 {
-    ScmObj r = SCM_FALSE;
-
-    switch (handling) {
-    case SCM_ILLEGAL_CHAR_REJECT:
-    case SCM_ILLEGAL_CHAR_OMIT:
-    case SCM_ILLEGAL_CHAR_REPLACE:
-        break;
-    default:
-        Scm_Error("invalid 'handling' argument: %d", handling);
-        return SCM_UNDEFINED; /* dummy */
-    }
-
-    const ScmStringBody *b = SCM_STRING_BODY(x);
-    if (!SCM_STRING_BODY_INCOMPLETE_P(b)) {
-        /* we do simple copy */
-        r = Scm_CopyString(x);
+    Scm_Warn("Obsoleted C API Scm_StringIncompleteToComplete called");
+    ScmObj proc = SCM_UNDEFINED;
+    SCM_BIND_PROC(proc, "string-incomplete->complete", Scm_GaucheModule());
+    ScmObj r;
+    if (handling == SCM_ILLEGAL_CHAR_REJECT) {
+        r = Scm_ApplyRec1(proc, SCM_OBJ(x));
+    } else if (handling == SCM_ILLEGAL_CHAR_OMIT) {
+        r = Scm_ApplyRec2(proc, SCM_OBJ(x), SCM_MAKE_KEYWORD("omit"));
     } else {
-        const char *s = SCM_STRING_BODY_START(b);
-        ScmSmallInt siz = SCM_STRING_BODY_SIZE(b);
-        ScmSmallInt len = count_length(s, siz);
-        if (len >= 0) {
-            r = Scm_MakeString(s, siz, len, 0);
-        } else if (handling == SCM_ILLEGAL_CHAR_REJECT) {
-            r = SCM_FALSE;
-        } else {
-            const char *p = s;
-
-            ScmDString ds;
-            Scm_DStringInit(&ds);
-
-            while (p < s+siz) {
-                ScmChar ch;
-                if (p + SCM_CHAR_NFOLLOWS(*p) >= s + siz) {
-                    ch = SCM_CHAR_INVALID;
-                } else {
-                    SCM_CHAR_GET(p, ch);
-                }
-
-                if (ch != SCM_CHAR_INVALID) {
-                    Scm_DStringPutc(&ds, ch);
-                    p += SCM_CHAR_NBYTES(ch);
-                } else if (handling == SCM_ILLEGAL_CHAR_OMIT) {
-                    p++;
-                } else {        /* SCM_ILLEGAL_CHAR_REPLACE */
-                    Scm_DStringPutc(&ds, substitute);
-                    p++;
-                }
-            }
-            r = Scm_DStringGet(&ds, 0);
-        }
+        r = Scm_ApplyRec2(proc, SCM_OBJ(x), SCM_MAKE_CHAR(substitute));
     }
-
     return r;
 }
 
@@ -410,16 +373,20 @@ int Scm_StringCmp(ScmString *x, ScmString *y)
 {
     const ScmStringBody *xb = SCM_STRING_BODY(x);
     const ScmStringBody *yb = SCM_STRING_BODY(y);
-    if ((SCM_STRING_BODY_FLAGS(xb)^SCM_STRING_BODY_FLAGS(yb))&SCM_STRING_INCOMPLETE) {
-        Scm_Error("cannot compare incomplete vs complete string: %S, %S",
-                  SCM_OBJ(x), SCM_OBJ(y));
-    }
     ScmSmallInt sizx = SCM_STRING_BODY_SIZE(xb);
     ScmSmallInt sizy = SCM_STRING_BODY_SIZE(yb);
     ScmSmallInt siz = (sizx < sizy)? sizx : sizy;
     int r = memcmp(SCM_STRING_BODY_START(xb), SCM_STRING_BODY_START(yb), siz);
     if (r == 0) {
-        if (sizx == sizy) return 0;
+        if (sizx == sizy) {
+            if (SCM_STRING_BODY_INCOMPLETE_P(xb)) {
+                if (SCM_STRING_BODY_INCOMPLETE_P(yb)) return 0;
+                else                                  return 1;
+            } else {
+                if (SCM_STRING_BODY_INCOMPLETE_P(yb)) return -1;
+                else                                  return 0;
+            }
+        }
         if (sizx < sizy)  return -1;
         else              return 1;
     } else if (r < 0) {
@@ -1603,7 +1570,7 @@ int Scm_DStringSize(ScmDString *dstr)
             size += chain->chunk->bytes;
         }
     } else {
-        size = dstr->current - dstr->init.data;
+        size = dstr->init.bytes = dstr->current - dstr->init.data;
     }
     if (size > SCM_STRING_MAX_SIZE) {
         Scm_Error("Scm_DStringSize: size exceeded the range: %ld", size);
@@ -1611,6 +1578,13 @@ int Scm_DStringSize(ScmDString *dstr)
     return (int)size;
 }
 
+static ScmDStringChunk *newChunk(ScmSmallInt size)
+{
+    return SCM_NEW_ATOMIC2(ScmDStringChunk*,
+                           (sizeof(ScmDStringChunk)
+                            +size-SCM_DSTRING_INIT_CHUNK_SIZE));
+}
+
 void Scm__DStringRealloc(ScmDString *dstr, int minincr)
 {
     /* sets the byte count of the last chunk */
@@ -1630,11 +1604,8 @@ void Scm__DStringRealloc(ScmDString *dstr, int minincr)
         newsize = minincr;
     }
 
-    ScmDStringChunk *newchunk = SCM_NEW_ATOMIC2(
-        ScmDStringChunk*,
-        sizeof(ScmDStringChunk)+newsize-SCM_DSTRING_INIT_CHUNK_SIZE);
+    ScmDStringChunk *newchunk = newChunk(newsize);
     newchunk->bytes = 0;
-
     ScmDStringChain *newchain = SCM_NEW(ScmDStringChain);
 
     newchain->next = NULL;
@@ -1703,6 +1674,32 @@ const char *Scm_DStringGetz(ScmDString *dstr)
     return dstring_getz(dstr, &size, &len, FALSE);
 }
 
+/* Concatenate all chains in DString into one chunk.  Externally nothing
+   really changes, but this can be used to optimize allocation. */
+void Scm_DStringWeld(ScmDString *dstr)
+{
+    if (dstr->anchor == NULL) return; /* nothing to do */
+    ScmDStringChain *chain = dstr->anchor;
+    ScmSmallInt size = Scm_DStringSize(dstr);
+    ScmSmallInt bufsiz = size + (dstr->end - dstr->current);
+    ScmDStringChunk *newchunk = newChunk(bufsiz);
+    newchunk->bytes = size;
+    char *bptr = newchunk->data;
+    memcpy(bptr, dstr->init.data, dstr->init.bytes);
+    bptr += dstr->init.bytes;
+    for (; chain; chain = chain->next) {
+        memcpy(bptr, chain->chunk->data, chain->chunk->bytes);
+        bptr += chain->chunk->bytes;
+    }
+    dstr->init.bytes = 0;
+    dstr->anchor->chunk = newchunk;
+    dstr->anchor->next = NULL;
+    dstr->tail = dstr->anchor;
+    dstr->current = newchunk->data + size;
+    dstr->end = newchunk->data + bufsiz;
+    dstr->lastChunkSize = bufsiz;
+}
+
 /* Returns the current content of DString, along with byte size and character
    length. The returned pointer may not be NUL-terminated.
 
@@ -1712,7 +1709,16 @@ const char *Scm_DStringGetz(ScmDString *dstr)
    the returned content may be altered by further DString operation. */
 const char *Scm_DStringPeek(ScmDString *dstr, int *size, int *len)
 {
-    return dstring_getz(dstr, size, len, TRUE);
+    Scm_DStringWeld(dstr);
+    if (dstr->anchor == NULL) {
+        if (size) *size = dstr->current - dstr->init.data;
+        if (len)  *len = dstr->length;
+        return dstr->init.data;
+    } else {
+        if (size) *size = dstr->anchor->chunk->bytes;
+        if (len)  *len = dstr->length;
+        return dstr->anchor->chunk->data;
+    }
 }
 
 void Scm_DStringPutz(ScmDString *dstr, const char *str, int size)
@@ -1757,6 +1763,50 @@ void Scm_DStringPutc(ScmDString *ds, ScmChar ch)
     SCM_DSTRING_PUTC(ds, ch);
 }
 
+/* Truncate DString at the specified size.
+   Returns after-truncation size (it may be smaller than newsize if
+   the original DString isn't as large as newsize. */
+int Scm_DStringTruncate(ScmDString *dstr, int newsize)
+{
+    int origsize = Scm_DStringSize(dstr);
+
+    if (newsize < dstr->init.bytes) {
+        dstr->init.bytes = newsize;
+        dstr->anchor = NULL;
+        dstr->tail = NULL;
+        dstr->current = dstr->init.data + newsize;
+        dstr->end = dstr->init.data + SCM_DSTRING_INIT_CHUNK_SIZE;
+    } else {
+        if (newsize >= origsize) return origsize;
+        ScmDStringChain *chain = dstr->anchor;
+        int ss = dstr->init.bytes;
+        for (; chain; chain = chain->next) {
+            if (newsize < ss + chain->chunk->bytes) {
+                /* truncate this chunk */
+                if (chain == dstr->tail) {
+                    chain->chunk->bytes = newsize - ss;
+                    dstr->current = chain->chunk->data + newsize - ss;
+                } else {
+                    dstr->lastChunkSize = chain->chunk->bytes;
+                    dstr->end = chain->chunk->data + chain->chunk->bytes;
+                    chain->chunk->bytes = newsize - ss;
+                    chain->next = NULL;
+                    dstr->tail = chain;
+                    dstr->current = chain->chunk->data + newsize - ss;
+                }
+                break;
+            }
+            ss += chain->chunk->bytes;
+        }
+        SCM_ASSERT(chain != NULL);
+    }
+
+    /* If we accumulated only ASCII, we can adjust length as well. */
+    if (dstr->length == origsize || newsize == 0) dstr->length = newsize;
+    else                                          dstr->length = -1;
+    return newsize;
+}
+
 
 /* for debug */
 void Scm_DStringDump(FILE *out, ScmDString *dstr)
