@@ -146,6 +146,12 @@
     argc = (int)(SP - ARGP);
     vm->numVals = 1; /* default */
 
+    if (vm->callTrace) {
+        vm->callTrace->entries[vm->callTrace->top].base = BASE;
+        vm->callTrace->entries[vm->callTrace->top].pc = PC;
+        vm->callTrace->top = (vm->callTrace->top+1) & (vm->callTrace->size-1);
+    }
+
     /* object-apply hook.  shift args, and insert val0 into
        the fist arg slot, then call GenericObjectApply. */
     if (MOSTLY_FALSE(!SCM_PROCEDUREP(VAL0))) {
@@ -274,9 +280,15 @@
                 for (int i=0;i<argc; i++, ap++) SCM_FLONUM_ENSURE_MEM(*ap);
             }
 #endif /*GAUCHE_FFX*/
-            mm = Scm_SortMethods(mm, ARGP, argc);
-            nm = Scm_MakeNextMethod(SCM_GENERIC(VAL0), SCM_CDR(mm),
-                                    ARGP, argc, TRUE, APP);
+            if (SCM_PAIRP(SCM_CDR(mm))) {
+                mm = Scm_SortMethods(mm, ARGP, argc);
+            }
+            if (SCM_METHOD_LEAF_P(SCM_CAR(mm))) {
+                nm = SCM_TRUE;  /* Dummy */
+            } else {
+                nm = Scm_MakeNextMethod(SCM_GENERIC(VAL0), SCM_CDR(mm),
+                                        ARGP, argc, TRUE, APP);
+            }
             VAL0 = SCM_CAR(mm);
             proctype = SCM_PROC_METHOD;
         }
@@ -302,8 +314,12 @@
             VAL0 = SCM_OBJ(n->generic);
             proctype = SCM_PROC_GENERIC;
         } else {
-            nm = Scm_MakeNextMethod(n->generic, SCM_CDR(n->methods),
-                                    ARGP, argc, TRUE, apply_call_p);
+            if (SCM_METHOD_LEAF_P(SCM_CAR(n->methods))) {
+                nm = SCM_TRUE;  /* Dummy */
+            } else {
+                nm = Scm_MakeNextMethod(n->generic, SCM_CDR(n->methods),
+                                        ARGP, argc, TRUE, apply_call_p);
+            }
             VAL0 = SCM_CAR(n->methods);
             proctype = SCM_PROC_METHOD;
         }
