@@ -387,6 +387,25 @@
 (use data.random)
 (test-module 'data.random)
 
+(define (test-regular-strings regexp)
+  (let1 g (regular-strings$ regexp)
+    (test* #"regular-string$ ~regexp"
+           regexp
+           (generator->list g 10)
+           (^[rx ss] (every (^s (equal? (rxmatch->string rx s) s)) ss)))))
+
+(test-regular-strings #/a/)
+(test-regular-strings #/[a-z]/)
+(test-regular-strings #/[^a-z]/)
+(test-regular-strings #/abc|def/)
+(test-regular-strings #/ab*/)
+(test-regular-strings #/(ab|cd)+/)
+(test-regular-strings #/(abc){3}/)
+(test-regular-strings #/(abc){3,5}/)
+(test-regular-strings #/(abc){,3}/)
+(test-regular-strings #/(?i:abc)/)
+(test-regular-strings #/abc/i)
+
 ;;;========================================================================
 ;; depends on data.random
 (test-section "data.heap")
@@ -609,12 +628,20 @@
              (dotimes [i (+ (size-of initial-storage) 1)]
                (ring-buffer-add-back! rb i))))
     (let1 rb (make-ring-buffer initial-storage :overflow-handler 'overwrite)
-      (test* #"overflow - overwrite (~c)"
+      (test* #"overflow - overwrite forward (~c)"
              (iota (size-of initial-storage)
                    (size-of initial-storage))
              (begin
                (dotimes [i (* (size-of initial-storage) 2)]
                  (ring-buffer-add-back! rb i))
+               (remove-all-from-front rb))))
+    (let1 rb (make-ring-buffer initial-storage :overflow-handler 'overwrite)
+      (test* #"overflow - overwrite backward (~c)"
+             (reverse (iota (size-of initial-storage)
+                            (size-of initial-storage)))
+             (begin
+               (dotimes [i (* (size-of initial-storage) 2)]
+                 (ring-buffer-add-front! rb i))
                (remove-all-from-front rb)))))
 
   ;; body of test-ring-buffer
