@@ -221,32 +221,69 @@
 
 (use srfi-14)
 
-(test* "char-set=" #t
-       (char-set= (char-set #\ #\ #\ #\ #\)
-                  (string->char-set "")))
-(test* "char-set=" #t
-       (char-set= (list->char-set '(#\ #\ #\ #\))
-                  (string->char-set "󤤤")))
-(test* "char-set=" #t
-       (char-set= (->char-set "")
+(let ((data '(#[-¾--]
+              #[a])))
+  (define (t x)
+    (test* "mutable/immutable roundtrip" #t
+           (char-set= (char-set-copy x)
+                      (char-set-copy (char-set-freeze! (char-set-copy x))))))
+  (for-each t data))
+
+(test-char-set-1 "char-set-contains?" #t (cut char-set-contains? <> #\)
+                 (char-set #\ #\ #\ #\ #\))
+(test-char-set-1 "char-set-contains?" #f (cut char-set-contains? <> #\)
+                 (char-set #\ #\ #\ #\ #\))
+(test-char-set-1 "char-set-contains?" #t (cut char-set-contains? <> #\)
+                 (char-set-union
+                  (integer-range->char-set (char->integer #\)
+                                           (char->integer #\))
                   (integer-range->char-set (char->integer #\)
                                            (char->integer #\))))
-(test* "char-set<=" #t
-       (char-set<= (list->char-set '(#\ #\))
-                   char-set:full))
-(test* "char-set<=" #t (char-set<= #[] #[]))
-(test* "char-set<=" #t (char-set<= #[] #[-]))
-(test* "char-set<=" #f (char-set<= #[-] #[]))
-(test* "char-set<=" #t (char-set<= #[--] #[-]))
-(test* "char-set<=" #f (char-set<= #[-] #[--]))
-(test* "char-set<=" #f (char-set<= #[--] #[-]))
-(test* "char-set<=" #f (char-set<= #[--] #[-]))
-(test* "char-set<=" #t (char-set<= #[--] #[--]))
-(test* "char-set<=" #t (char-set<= #[--] #[--]))
-(test* "char-set<=" #t (char-set<= #[--] #[--]))
-(test* "char-set<=" #t (char-set<= #[-] #[--]))
-(test* "char-set<=" #t (char-set<= #[--] #[--]))
-(test* "char-set<=" #f (char-set<= #[-] #[--]))
+(test-char-set-1 "char-set-contains?" #t (cut char-set-contains? <> #\)
+                 (char-set-union
+                  (integer-range->char-set (char->integer #\)
+                                           (char->integer #\))
+                  (integer-range->char-set (char->integer #\)
+                                           (char->integer #\))))
+(test-char-set-1 "char-set-contains?" #f (cut char-set-contains? <> #\)
+                 (char-set-union
+                  (integer-range->char-set (char->integer #\)
+                                           (char->integer #\))
+                  (integer-range->char-set (char->integer #\)
+                                           (char->integer #\))))
+(test-char-set-1 "char-set-contains?" #f (cut char-set-contains? <> #\)
+                 (char-set-union
+                  (integer-range->char-set (char->integer #\)
+                                           (char->integer #\))
+                  (integer-range->char-set (char->integer #\)
+                                           (char->integer #\))))
+
+(test-char-set-2 "char-set=" #t char-set=
+                 (char-set #\ #\ #\ #\ #\)
+                 (string->char-set ""))
+(test-char-set-2 "char-set=" #t char-set=
+                 (list->char-set '(#\ #\ #\ #\))
+                 (string->char-set "󤤤"))
+(test-char-set-2 "char-set=" #t char-set=
+                 (->char-set "")
+                 (integer-range->char-set (char->integer #\)
+                                          (char->integer #\)))
+(test-char-set-2 "char-set<=" #t char-set<=
+                 (list->char-set '(#\ #\))
+                 char-set:full)
+(test-char-set-2 "char-set<=" #t char-set<= #[] #[])
+(test-char-set-2 "char-set<=" #t char-set<= #[] #[-])
+(test-char-set-2 "char-set<=" #f char-set<= #[-] #[])
+(test-char-set-2 "char-set<=" #t char-set<= #[--] #[-])
+(test-char-set-2 "char-set<=" #f char-set<= #[-] #[--])
+(test-char-set-2 "char-set<=" #f char-set<= #[--] #[-])
+(test-char-set-2 "char-set<=" #f char-set<= #[--] #[-])
+(test-char-set-2 "char-set<=" #t char-set<= #[--] #[--])
+(test-char-set-2 "char-set<=" #t char-set<= #[--] #[--])
+(test-char-set-2 "char-set<=" #t char-set<= #[--] #[--])
+(test-char-set-2 "char-set<=" #t char-set<= #[-] #[--])
+(test-char-set-2 "char-set<=" #t char-set<= #[--] #[--])
+(test-char-set-2 "char-set<=" #f char-set<= #[-] #[--])
 
 ;;-------------------------------------------------------------------
 (test-section "ports")
