@@ -73,12 +73,27 @@
          (hash-table-put! h-eq 'a 8)
          (hash-table-get  h-eq 'a)))
 
+(test* "a adjoin" 8
+       (begin
+         (hash-table-adjoin! h-eq 'a 9)
+         (hash-table-get h-eq 'a)))
+
+(test* "a replace" 7
+       (begin
+         (hash-table-replace! h-eq 'a 7)
+         (hash-table-get h-eq 'a)))
+
 (test* "b => non" #t
        (hash-table-get  h-eq 'b #t))
 
 (test* "b => error" (test-error)
        (hash-table-get h-eq 'b))
 
+(test* "b replace" #f
+       (begin
+         (hash-table-replace! h-eq 'b 100)
+         (hash-table-get h-eq 'b #f)))
+
 (test* "b => \"b\"" "b"
        (begin
          (hash-table-put! h-eq 'b "b")
@@ -94,6 +109,12 @@
          (hash-table-put! h-eq 'c #\c)
          (hash-table-get  h-eq 'c)))
 
+(test* "d adjoin" "D"
+       (begin
+         (hash-table-adjoin! h-eq 'd "D")
+         (hash-table-adjoin! h-eq 'd "d")
+         (hash-table-get h-eq 'd)))
+
 (test* "e => 10" 10
        (begin
          (hash-table-put! h-eq 'e 8)
@@ -106,14 +127,14 @@
          (hash-table-update! h-eq 'f (^x (+ x 1)) 2)
          (hash-table-get h-eq 'f)))
 
-(test* "eq? test" 7
+(test* "eq? test" 8
        (begin
          (hash-table-put! h-eq (string #\d) 4)
          (hash-table-put! h-eq (string #\d) 5)
          (length (hash-table-keys h-eq))))
 
 (test* "hash-table-values(1)" #t
-       (lset= equal? (hash-table-values h-eq) '(8 "b" #\c 3 4 5 10)))
+       (lset= equal? (hash-table-values h-eq) '(7 "b" #\c "D" 3 4 5 10)))
 
 (test* "delete!" '(#t #f #f)
        (let* ((a (hash-table-delete! h-eq 'c))
@@ -373,11 +394,11 @@
 ;;------------------------------------------------------------------
 (test-section "iterators")
 
-(define h-it (hash-table 'eq?
-                         '(a . 3)
-                         '(c . 8)
-                         '(b . 4)
-                         '(d . 10)))
+(define h-it (hash-table-from-pairs 'eq?
+                                    '(a . 3)
+                                    '(c . 8)
+                                    '(b . 4)
+                                    '(d . 10)))
 
 (test* "hash-table"
        '(a b c d)
@@ -411,6 +432,61 @@
          (list (assoc "a" a)
                (assoc "b" a))))
 
+(test* "hash-table-find" 16
+       (hash-table-find h-it (^[k v] (and (eq? k 'c) (* v 2)))))
+
+(test* "hash-table-find" 'oops
+       (hash-table-find h-it (^[k v] (and (eq? k 'e) (* v 2)))
+                        (^[] 'oops)))
+
+;;------------------------------------------------------------------
+(test-section "compare as sets")
+
+(let ([a (hash-table-r7 eq-comparator 'a 1)]
+      [b (hash-table-r7 eq-comparator 'a 1 'b 2)]
+      [c (hash-table-r7 eq-comparator 'a 1 'b 2 'c 3)]
+      [d (hash-table-r7 eq-comparator 'a 1 'b 2 'c -3)]
+      [e (hash-table-r7 eq-comparator 'a 1 'e 2)]
+      [f (hash-table-r7 eq-comparator 'a 1 'b 2 'c 3 'e 2)]
+      [g (hash-table-r7 eq-comparator 'a 1 'b 2 'c 3)])
+  (define (inv a) (and a (- a)))
+  (define-syntax D
+    (syntax-rules ()
+      ([_ x y expected] `(,x ,y x y ,expected))))
+  (define data
+    (list (D a b -1)
+          (D a c -1)
+          (D b c -1)
+          (D b d -1)
+          (D b e #f)
+          (D c d #f)
+          (D c e #f)
+          (D d e #f)
+          (D e f -1)
+          (D c g 0)))
+
+  (dolist [d data]
+    (receive (x y xname yname expected) (apply values d)
+      (test* "hash-table-compare-as-sets"
+             `((,xname ,yname ,expected)
+               (,yname ,xname ,(and expected (- expected))))
+             (list
+              `(,xname ,yname ,(hash-table-compare-as-sets x y eqv? #f))
+              `(,yname ,xname ,(hash-table-compare-as-sets y x eqv? #f))))))
+
+  (test* "hash-table=? c d" #f
+         (hash-table=? eqv-comparator c d))
+  (test* "hash-table=? c g" #t
+         (hash-table=? eqv-comparator c g))
+  (test* "hash-table=? c c" #t
+         (hash-table=? eqv-comparator c c))
+  (test* "hash-table=? f a" #f
+         (hash-table=? eqv-comparator f a))
+  (test* "hash-table=? c d (custom compar)" #t
+         (hash-table=? (make-comparator number? (^[a b] (= (abs a) (abs b)))
+                                        #f #f)
+                       c d)))
+              
 (test-module 'gauche.hashutil) ; autoloaded module
 
 (test-end)
