@@ -637,114 +637,142 @@
 ;;-------------------------------------------------------------------
 (test-section "format")
 
-(test* "format ~s" "\"abc\""         (format #f "~s" "abc"))
-(test* "format ~s" "\"abc\"     "    (format #f "~10s" "abc"))
-(test* "format ~s" "     \"abc\""    (format #f "~10@s" "abc"))
-(test* "format ~s" "\"abc\"      "   (format #f "~10,3s" "abc"))
-(test* "format ~s" "      \"abc\""   (format #f "~10,3@s" "abc"))
-(test* "format ~s" "\"abc\" "        (format #f "~,,1s" "abc"))
-(test* "format ~s" " \"abc\""        (format #f "~,,1@s" "abc"))
-(test* "format ~s" "\"abc\"*****"    (format #f "~10,,,'*s" "abc"))
-(test* "format ~s" "*****\"abc\""    (format #f "~10,,,'*@s" "abc"))
+(test* "format ~s" "\"abc\""         (format "~s" "abc"))
+(test* "format ~s" "\"abc\"     "    (format "~10s" "abc"))
+(test* "format ~s" "     \"abc\""    (format "~10@s" "abc"))
+(test* "format ~s" "\"abc\"      "   (format "~10,3s" "abc"))
+(test* "format ~s" "      \"abc\""   (format "~10,3@s" "abc"))
+(test* "format ~s" "\"abc\" "        (format "~,,1s" "abc"))
+(test* "format ~s" " \"abc\""        (format "~,,1@s" "abc"))
+(test* "format ~s" "\"abc\"*****"    (format "~10,,,'*s" "abc"))
+(test* "format ~s" "*****\"abc\""    (format "~10,,,'*@s" "abc"))
 
-(test* "format ~s" "\"abc\"*****"    (format #f "~10,,,'*,15s" "abc"))
-(test* "format ~s" "*****\"abc\""    (format #f "~10,,,'*,15@s" "abc"))
+(test* "format ~s" "\"abc\"*****"    (format "~10,,,'*,15s" "abc"))
+(test* "format ~s" "*****\"abc\""    (format "~10,,,'*,15@s" "abc"))
 (test* "format ~s" "(\"abc\" \"def\" \"g"
-       (format #f "~10,,,'*,15s" '("abc" "def" "ghi" "jkl")))
+       (format "~10,,,'*,15s" '("abc" "def" "ghi" "jkl")))
 (test* "format ~s" "(\"abc\" \"def\" \"g"
-       (format #f "~10,,,'*,15@s" '("abc" "def" "ghi" "jkl")))
+       (format "~10,,,'*,15@s" '("abc" "def" "ghi" "jkl")))
 (test* "format ~s" "(\"abc\" \"def ..."
-       (format #f "~10,,,'*,15:s" '("abc" "def" "ghi" "jkl")))
+       (format "~10,,,'*,15:s" '("abc" "def" "ghi" "jkl")))
 (test* "format ~s" "(\"abc\" \"def ..."
-       (format #f "~10,,,'*,15@:s" '("abc" "def" "ghi" "jkl")))
+       (format "~10,,,'*,15@:s" '("abc" "def" "ghi" "jkl")))
 
-(test* "format ~a" "abc"           (format #f "~a" "abc"))
-(test* "format ~a" "abc       "    (format #f "~10a" "abc"))
-(test* "format ~a" "       abc"    (format #f "~10@a" "abc"))
-(test* "format ~a" "abc         "  (format #f "~10,3a" "abc"))
-(test* "format ~a" "         abc"  (format #f "~10,3@a" "abc"))
-(test* "format ~a" "abc "          (format #f "~,,1a" "abc"))
-(test* "format ~a" " abc"          (format #f "~,,1@a" "abc"))
-(test* "format ~a" "abc*******"    (format #f "~10,,,'*a" "abc"))
-(test* "format ~a" "*******abc"    (format #f "~10,,,'*@a" "abc"))
+(test* "format ~a" "abc"           (format "~a" "abc"))
+(test* "format ~a" "abc       "    (format "~10a" "abc"))
+(test* "format ~a" "       abc"    (format "~10@a" "abc"))
+(test* "format ~a" "abc         "  (format "~10,3a" "abc"))
+(test* "format ~a" "         abc"  (format "~10,3@a" "abc"))
+(test* "format ~a" "abc "          (format "~,,1a" "abc"))
+(test* "format ~a" " abc"          (format "~,,1@a" "abc"))
+(test* "format ~a" "abc*******"    (format "~10,,,'*a" "abc"))
+(test* "format ~a" "*******abc"    (format "~10,,,'*@a" "abc"))
 
 (test* "format ~a" "(abc def ghi jk"
-       (format #f "~10,,,'*,15a" '("abc" "def" "ghi" "jkl")))
+       (format "~10,,,'*,15a" '("abc" "def" "ghi" "jkl")))
 (test* "format ~a" "(abc def ghi jk"
-       (format #f "~10,,,'*,15@a" '("abc" "def" "ghi" "jkl")))
+       (format "~10,,,'*,15@a" '("abc" "def" "ghi" "jkl")))
 (test* "format ~a" "(abc def gh ..."
-       (format #f "~10,,,'*,15:a" '("abc" "def" "ghi" "jkl")))
+       (format "~10,,,'*,15:a" '("abc" "def" "ghi" "jkl")))
 (test* "format ~a" "(abc def gh ..."
-       (format #f "~10,,,'*,15@:a" '("abc" "def" "ghi" "jkl")))
+       (format "~10,,,'*,15@:a" '("abc" "def" "ghi" "jkl")))
 
-(test* "format ~d" "12345"       (format #f "~d" 12345))
-(test* "format ~d" "-12345"      (format #f "~d" -12345))
-(test* "format ~d" "+12345"      (format #f "~@d" 12345))
-(test* "format ~d" "-12345"      (format #f "~@d" -12345))
-(test* "format ~d" "     12345"  (format #f "~10d" 12345))
-(test* "format ~d" "    -12345"  (format #f "~10d" -12345))
-(test* "format ~d" "    +12345"  (format #f "~10@d" 12345))
-(test* "format ~d" "    -12345"  (format #f "~10@d" -12345))
-(test* "format ~d" "0000012345"  (format #f "~10,'0d" 12345))
-(test* "format ~d" "0000-12345"  (format #f "~10,'0d" -12345))
+(test* "format ~w" "(#0=(1) #0#)"
+       (format "~w" (let1 x '(1) (list x x))))
 
-(test* "format ~:d" "1"  (format #f "~:d" 1))
-(test* "format ~:d" "-1"  (format #f "~:d" -1))
-(test* "format ~:d" "12"  (format #f "~:d" 12))
-(test* "format ~:d" "-12"  (format #f "~:d" -12))
-(test* "format ~:d" "123"  (format #f "~:d" 123))
-(test* "format ~:d" "-123"  (format #f "~:d" -123))
-(test* "format ~:d" "+123"  (format #f "~:@d" 123))
-(test* "format ~:d" "1,234"  (format #f "~:d" 1234))
-(test* "format ~:d" "-1,234"  (format #f "~:d" -1234))
-(test* "format ~:d" "+1,234"  (format #f "~:@d" 1234))
-(test* "format ~:d" "12,345"  (format #f "~:d" 12345))
-(test* "format ~:d" "-12,345"  (format #f "~:d" -12345))
-(test* "format ~:d" "123,456,789"  (format #f "~:d" 123456789))
-(test* "format ~:d" "-123,456,789"  (format #f "~:d" -123456789))
-(test* "format ~:d" "123.456.789"  (format #f "~,,'.:d" 123456789))
-(test* "format ~:d" "-123.456.789" (format #f "~,,'.:d" -123456789))
-(test* "format ~:d" "1.2345.6789"  (format #f "~,,'.,4:d" 123456789))
-(test* "format ~:d" "-1.2345.6789" (format #f "~,,'.,4:d" -123456789))
-(test* "format ~:d" "    12,345"  (format #f "~10:d" 12345))
-(test* "format ~:d" "   -12,345"  (format #f "~10:d" -12345))
-(test* "format ~:d" "   +12,345"  (format #f "~10:@d" 12345))
+(test* "format ~c" "|a|" (format "|~c|" #\a))
+(test* "format ~c" "|#\\a|" (format "|~@c|" #\a))
+(test* "format ~c" (test-error) (format "|~c|" 1))
 
-(test* "format ~b" "10101"       (format #f "~b" 21))
-(test* "format ~b" "-10101"      (format #f "~b" -21))
-(test* "format ~b" "+10101"      (format #f "~@b" 21))
-(test* "format ~b" "-10101"      (format #f "~@b" -21))
-(test* "format ~b" "     10101"  (format #f "~10b" 21))
-(test* "format ~b" "    -10101"  (format #f "~10b" -21))
-(test* "format ~b" "    +10101"  (format #f "~10@b" 21))
-(test* "format ~b" "    -10101"  (format #f "~10@b" -21))
-(test* "format ~b" "0000010101"  (format #f "~10,'0b" 21))
-(test* "format ~b" "0000-10101"  (format #f "~10,'0b" -21))
+(test* "format ~d" "12345"       (format "~d" 12345))
+(test* "format ~d" "-12345"      (format "~d" -12345))
+(test* "format ~d" "+12345"      (format "~@d" 12345))
+(test* "format ~d" "-12345"      (format "~@d" -12345))
+(test* "format ~d" "     12345"  (format "~10d" 12345))
+(test* "format ~d" "    -12345"  (format "~10d" -12345))
+(test* "format ~d" "    +12345"  (format "~10@d" 12345))
+(test* "format ~d" "    -12345"  (format "~10@d" -12345))
+(test* "format ~d" "0000012345"  (format "~10,'0d" 12345))
+(test* "format ~d" "0000-12345"  (format "~10,'0d" -12345))
 
-(test* "format ~b" "101"         (format #f "~,,' ,4:b" 5))
-(test* "format ~b" "101 0101"    (format #f "~,,' ,4:b" 85))
+(test* "format ~:d" "1"  (format "~:d" 1))
+(test* "format ~:d" "-1"  (format "~:d" -1))
+(test* "format ~:d" "12"  (format "~:d" 12))
+(test* "format ~:d" "-12"  (format "~:d" -12))
+(test* "format ~:d" "123"  (format "~:d" 123))
+(test* "format ~:d" "-123"  (format "~:d" -123))
+(test* "format ~:d" "+123"  (format "~:@d" 123))
+(test* "format ~:d" "1,234"  (format "~:d" 1234))
+(test* "format ~:d" "-1,234"  (format "~:d" -1234))
+(test* "format ~:d" "+1,234"  (format "~:@d" 1234))
+(test* "format ~:d" "12,345"  (format "~:d" 12345))
+(test* "format ~:d" "-12,345"  (format "~:d" -12345))
+(test* "format ~:d" "123,456,789"  (format "~:d" 123456789))
+(test* "format ~:d" "-123,456,789"  (format "~:d" -123456789))
+(test* "format ~:d" "123.456.789"  (format "~,,'.:d" 123456789))
+(test* "format ~:d" "-123.456.789" (format "~,,'.:d" -123456789))
+(test* "format ~:d" "1.2345.6789"  (format "~,,'.,4:d" 123456789))
+(test* "format ~:d" "-1.2345.6789" (format "~,,'.,4:d" -123456789))
+(test* "format ~:d" "    12,345"  (format "~10:d" 12345))
+(test* "format ~:d" "   -12,345"  (format "~10:d" -12345))
+(test* "format ~:d" "   +12,345"  (format "~10:@d" 12345))
 
-(test* "format ~o" "12345"       (format #f "~o" 5349))
-(test* "format ~o" "-12345"      (format #f "~o" -5349))
-(test* "format ~o" "+12345"      (format #f "~@o" 5349))
-(test* "format ~o" "-12345"      (format #f "~@o" -5349))
-(test* "format ~o" "     12345"  (format #f "~10o" 5349))
-(test* "format ~o" "    -12345"  (format #f "~10o" -5349))
-(test* "format ~o" "    +12345"  (format #f "~10@o" 5349))
-(test* "format ~o" "    -12345"  (format #f "~10@o" -5349))
-(test* "format ~o" "0000012345"  (format #f "~10,'0o" 5349))
-(test* "format ~o" "0000-12345"  (format #f "~10,'0o" -5349))
+(test* "format ~b" "10101"       (format "~b" 21))
+(test* "format ~b" "-10101"      (format "~b" -21))
+(test* "format ~b" "+10101"      (format "~@b" 21))
+(test* "format ~b" "-10101"      (format "~@b" -21))
+(test* "format ~b" "     10101"  (format "~10b" 21))
+(test* "format ~b" "    -10101"  (format "~10b" -21))
+(test* "format ~b" "    +10101"  (format "~10@b" 21))
+(test* "format ~b" "    -10101"  (format "~10@b" -21))
+(test* "format ~b" "0000010101"  (format "~10,'0b" 21))
+(test* "format ~b" "0000-10101"  (format "~10,'0b" -21))
 
-(test* "format ~x" "12345"       (format #f "~x" 74565))
-(test* "format ~x" "-12345"      (format #f "~x" -74565))
-(test* "format ~x" "+12345"      (format #f "~@x" 74565))
-(test* "format ~x" "-12345"      (format #f "~@x" -74565))
-(test* "format ~x" "     12345"  (format #f "~10x" 74565))
-(test* "format ~x" "    -12345"  (format #f "~10x" -74565))
-(test* "format ~x" "    +12345"  (format #f "~10@x" 74565))
-(test* "format ~x" "    -12345"  (format #f "~10@x" -74565))
-(test* "format ~x" "0000012345"  (format #f "~10,'0x" 74565))
-(test* "format ~x" "0000-12345"  (format #f "~10,'0x" -74565))
+(test* "format ~b" "101"         (format "~,,' ,4:b" 5))
+(test* "format ~b" "101 0101"    (format "~,,' ,4:b" 85))
+
+(test* "format ~o" "12345"       (format "~o" 5349))
+(test* "format ~o" "-12345"      (format "~o" -5349))
+(test* "format ~o" "+12345"      (format "~@o" 5349))
+(test* "format ~o" "-12345"      (format "~@o" -5349))
+(test* "format ~o" "     12345"  (format "~10o" 5349))
+(test* "format ~o" "    -12345"  (format "~10o" -5349))
+(test* "format ~o" "    +12345"  (format "~10@o" 5349))
+(test* "format ~o" "    -12345"  (format "~10@o" -5349))
+(test* "format ~o" "0000012345"  (format "~10,'0o" 5349))
+(test* "format ~o" "0000-12345"  (format "~10,'0o" -5349))
+
+(test* "format ~x" "12345"       (format "~x" 74565))
+(test* "format ~x" "-12345"      (format "~x" -74565))
+(test* "format ~x" "+12345"      (format "~@x" 74565))
+(test* "format ~x" "-12345"      (format "~@x" -74565))
+(test* "format ~x" "     12345"  (format "~10x" 74565))
+(test* "format ~x" "    -12345"  (format "~10x" -74565))
+(test* "format ~x" "    +12345"  (format "~10@x" 74565))
+(test* "format ~x" "    -12345"  (format "~10@x" -74565))
+(test* "format ~x" "0000012345"  (format "~10,'0x" 74565))
+(test* "format ~x" "0000-12345"  (format "~10,'0x" -74565))
+
+(test* "format ~f" "1.2345"      (format "~f" 1.2345))
+(test* "format ~5,3f" "1.234"    (format "~5,3f" 1.2345))
+;(test* "format ~5,3:f" "1.234"   (format "~5,3:f" 1.2345)) ;banker's rounding
+(test* "format ~5,3:f" "1.235"   (format "~5,3:f" 1.2345)) ;commercial rounding
+(test* "format ~6,2f" "  1.23"   (format "~6,2f" 1.2345))
+(test* "format ~6,2f" "  5.01"   (format "~6,2f" 5.015))
+(test* "format ~6,2:f" "  5.02"  (format "~6,2:f" 5.015))
+(test* "format ~6,2:f" "-10.00"   (format "~6,2:f" -9.995))
+(test* "format ~10f" "     abcde"  (format "~10,2f" 'abcde))
+(test* "format ~1,2f" "0.00"     (format "~1,2f" 0.003))
+(test* "format ~1,0f" "-1."      (format "~1,0f" -0.555))
+
+(test* "format carry over ~,4f" "0.0100" (format "~,4f" 0.00999))
+(test* "format carry over across decimal point ~,1f" "124.0"
+       (format "~,1f" 123.95))
+(test* "format carry over across decimal point ~,2f" "-1000.00"
+       (format "~,2f" -999.995))
+
+(test* "format ~,1000f" (string-append "0.01" (make-string 998 #\0))
+       (format "~,1000f" 0.01))
 
 (test* "format ~nr" "wud0up"  (format "~36r" 1985913745))
 (test* "format ~nr" "    wud0up"  (format "~36,10r" 1985913745))
@@ -753,30 +781,30 @@
 (test* "format ~nR" "WUD0UP"  (format "~36R" 1985913745))
 
 (test* "format v param" "     12345"
-       (format #f "~vd" 10 12345))
+       (format "~vd" 10 12345))
 (test* "format v param" "0000012345"
-       (format #f "~v,vd" 10 #\0 12345))
+       (format "~v,vd" 10 #\0 12345))
 
 (test* "format ~*" "1 2 4 5"
-       (format #f "~a ~a ~*~a ~a" 1 2 3 4 5))
+       (format "~a ~a ~*~a ~a" 1 2 3 4 5))
 (test* "format ~*" "1 5"
-       (format #f "~a ~3*~a" 1 2 3 4 5))
+       (format "~a ~3*~a" 1 2 3 4 5))
 (test* "format ~:*" "1 2 2 3 4 5"
-       (format #f "~a ~a ~:*~a ~a ~a ~a" 1 2 3 4 5))
+       (format "~a ~a ~:*~a ~a ~a ~a" 1 2 3 4 5))
 (test* "format ~:*" "1 2 3 1 2 3 4 5"
-       (format #f "~a ~a ~a ~3:*~a ~a ~a ~a ~a" 1 2 3 4 5))
+       (format "~a ~a ~a ~3:*~a ~a ~a ~a ~a" 1 2 3 4 5))
 (test* "format ~:*" "1 2 3 1"
-       (format #f "~a ~a ~a ~3:*~a" 1 2 3 4 5))
+       (format "~a ~a ~a ~3:*~a" 1 2 3 4 5))
 (test* "format ~@*" "1 2 5"
-       (format #f "~a ~a ~4@*~a" 1 2 3 4 5))
+       (format "~a ~a ~4@*~a" 1 2 3 4 5))
 
 (test* "format incomplete tilde sequence" (test-error)
-       (format #f "~"))
+       (format "~"))
 (test* "format incomplete tilde sequence" (test-error)
-       (format #f "~123"))
+       (format "~123"))
 
 (test* "format skip to the end" "||"
-       (format #f "|~*|" 1))
+       (format "|~*|" 1))
 
 ;; regression check for format/ss
 (test* "format/ss" "z  " (format/ss "~v,a" 3 'z))
