@@ -74,10 +74,24 @@
 ;;--------------------------------------------------------------------------
 (test-section "append and reverse")
 
-(test* "append" '(1 2 3 4 5) (append '(1 2 3) '() '(4 5)))
-(test* "append" '(1 2 3 . 5) (append '(1 2 3) 5))
-(test* "append!" '(1 2 3 4 5) (append (list 1 2 3) '() '(4 5)))
-(test* "append!" '(1 2 3 . 5) (append (list 1 2 3) 5))
+(test* "append (inlined)" '()           (append))
+(test* "append (inlined)" 1             (append 1))
+(test* "append (inlined)" '(1 2 3 4 5)  (append '(1 2 3) '() '(4 5)))
+(test* "append (inlined)" '(1 2 3 . 5)  (append '(1 2 3) 5))
+(test* "append (inlined)" (test-error)  (append 1 2))
+(test* "append (inlined)" (test-error)  (append '(1) '(2 . 3) 4))
+(test* "append" '()           (apply append '()))
+(test* "append" 1             (apply append '(1)))
+(test* "append" '(1 2 3 4 5)  (apply append '((1 2 3) () (4 5))))
+(test* "append" '(1 2 3 . 5)  (apply append '((1 2 3) 5)))
+(test* "append" (test-error)  (apply append '(1 2)))
+(test* "append" (test-error)  (apply append '((1) (2 . 3) 4)))
+(test* "append!" '()          (append!))
+(test* "append!" 1            (append! 1))
+(test* "append!" '(1 2 3 4 5) (append! (list 1 2 3) '() '(4 5)))
+(test* "append!" '(1 2 3 . 5) (append! (list 1 2 3) 5))
+(test* "append!" (test-error) (append! (list* 1 2 3) 5))
+(test* "append!" (test-error) (append! (list 1 2 3) (list* 1 2 3) 5))
 (test* "reverse" '(5 4 3 2 1) (reverse '(1 2 3 4 5)))
 (test* "reverse" '() (reverse '()))
 (test* "reverse!" '(5 4 3 2 1) (reverse! (list 1 2 3 4 5)))
