@@ -21,25 +21,6 @@
 ;;----------------------------------------------------------------------
 ;; 
 
-(test-section "primitive macro transformer")
-
-(define-syntax defmac  ; (defmac name args . body)
-  (primitive-macro-transformer
-   (lambda (form def-env use-env)
-     (let ([name (cadr form)]
-           [args (caddr form)]
-           [body (cdddr form)])
-       `(define-syntax ,name
-          (let ((,name (lambda ,args ,@body)))
-            (primitive-macro-transformer
-             (lambda (f d u) (apply ,name (cdr f))))))))))
-
-(defmac myif (test then else) `(if ,test ,then ,else))
-
-(test-macro "defmac if" (if a b c) (myif a b c))
-
-;; explicit renaming macros
-
 (test-section "ER macro basics")
 
 (define-syntax er-when
@@ -204,6 +185,28 @@
              '((1 2) . #(1 2))
              (foo)))))
 
+;; er-macro and with-module
+;; cf. https://github.com/shirok/Gauche/issues/250
+(define er-macro-scope-test-a 'a)
+
+(define-module er-macro-test-1
+  (define er-macro-scope-test-a 'b))
+
+(with-module er-macro-test-1
+  (define-syntax er-macro-test-x
+    (er-macro-transformer
+     (^[f r c] (r 'er-macro-scope-test-a)))))
+
+(test* "er-macro and with-module" 'b
+       ((with-module er-macro-test-1 er-macro-test-x)))
+
+;; er-macro and eval
+(test* "er-macro and eval" 'b
+       (eval '(let-syntax ((m (er-macro-transformer
+                               (^[f r c] (r 'er-macro-scope-test-a)))))
+                (m))
+             (find-module 'er-macro-test-1)))
+
 ;; quasirename
 ;; Note: currently quasirename depends on util.match, too.
 (let ((unquote list)
@@ -246,6 +249,18 @@
 (test-macro "simple" (f z (#\b #\a)) (simple (#\b #\a) z))
 (test-macro "simple" (f z #(2 1)) (simple #(2 1) z))
 
+(define-syntax underbar (syntax-rules ()
+                          [(_) 0]
+                          [(_ _) 1]
+                          [(_ _ _) 2]
+                          [(_ _ _ _) 3]
+                          [(_ _ _ _ . _) many]))
+(test-macro "underbar" 0 (underbar))
+(test-macro "underbar" 1 (underbar a))
+(test-macro "underbar" 2 (underbar a b))
+(test-macro "underbar" 3 (underbar a b c))
+(test-macro "underbar" many (underbar a b c d))
+
 (define-syntax repeat (syntax-rules ()
                         ((_ 0 (?a ?b) ...)     ((?a ...) (?b ...)))
                         ((_ 1 (?a ?b) ...)     (?a ... ?b ...))
@@ -333,12 +348,38 @@
                      j)
             (nest5 (z (a b c) (d e f g) (h i) . j)))
 
+(define-syntax nest6 (syntax-rules ()
+                       ((_ (?a ...) ...)
+                        (?a ... ...)))) ;srfi-149
+(test-macro "nest6" (a b c d e f g h i j)
+            (nest6 (a b c d) (e f g) (h i) (j)))
+(test-macro "nest6" (a b c d e f g)
+            (nest6 (a b c d) () (e) () (f g)))
 
+(define-syntax nest7 (syntax-rules ()
+                       ((_ (?a ...) ...)
+                        (?a ... ... z ?a ... ...)))) ;srfi-149
+(test-macro "nest7" (a b c d e f g h i j z a b c d e f g h i j)
+            (nest7 (a b c d) (e f g) (h i) (j)))
+(test-macro "nest7" (a b c d e f g z a b c d e f g)
+            (nest7 (a b c d) () (e) () (f g)))
+
+(define-syntax nest8 (syntax-rules ()
+                       ((_ ((?a ...) ...) ...)
+                        (?a ... ... ... z)))) ;srfi-149
+(test-macro "nest8" (a b c d e f g h i j z)
+            (nest8 ((a b c d) (e f g)) ((h i) (j))))
+(test-macro "nest8" (a b c d e f g h i j z)
+            (nest8 ((a b c d) () (e f g)) () ((h i) () (j) ())))
+
+;; mixlevel is allowed by srfi-149
 (define-syntax mixlevel1 (syntax-rules ()
                            ((_ (?a ?b ...)) ((?a ?b) ...))))
 
 (test-macro "mixlevel1" ((1 2) (1 3) (1 4) (1 5) (1 6))
             (mixlevel1 (1 2 3 4 5 6)))
+(test-macro "mixlevel1" ()
+            (mixlevel1 (1)))
 
 (define-syntax mixlevel2 (syntax-rules ()
                            ((_ (?a ?b ...) ...)
@@ -359,33 +400,33 @@
 (test "bad ellipsis 1" (test-error)
       (lambda () 
         (eval '(define-syntax badellipsis
-                 (syntax-rules () (t) (3 ...)))
+                 (syntax-rules () [(t) (3 ...)]))
               (interaction-environment))))
 (test "bad ellipsis 2" (test-error)
       (lambda ()
         (eval '(define-syntax badellipsis
-                 (syntax-rules () (t a) (a ...)))
+                 (syntax-rules () [(t a) (a ...)]))
               (interaction-environment))))
 (test "bad ellipsis 3" (test-error)
       (lambda ()
         (eval '(define-syntax badellipsis
-                 (syntax-rules () (t a b ...) (a ...)))
+                 (syntax-rules () [(t a b ...) (a ...)]))
               (interaction-environment))))
 (test "bad ellipsis 4" (test-error)
       (lambda ()
         (eval '(define-syntax badellipsis
-                 (syntax-rules () (t a ...) ((a ...) ...)))
+                 (syntax-rules () [(t a ...) ((a ...) ...)]))
               (interaction-environment))))
 
 (test "bad ellipsis 5" (test-error)
       (lambda ()
         (eval '(define-syntax badellipsis
-                 (syntax-rules () ((a ... b ...)) ((a ...) (b ...))))
+                 (syntax-rules () [(t (a ... b ...)) ((a ...) (b ...))]))
               (interaction-environment))))
 (test "bad ellipsis 6" (test-error)
       (lambda ()
         (eval '(define-syntax badellipsis
-                 (syntax-rules () ((... a b)) (... a b )))
+                 (syntax-rules () [(t (... a b)) (... a b )]))
               (interaction-environment))))
 
 (define-syntax hygiene (syntax-rules ()
@@ -484,6 +525,14 @@
 
 (test "alt-elli2" '((a a) (b b) (c c)) (lambda () (alt-elli2 a b c)))
 
+;; https://srfi-email.schemers.org/srfi-148/msg/6115633
+(define-syntax alt-elli3
+  (syntax-rules ... (...)
+    [(m x y ...) 'ellipsis]
+    [(m x ...)   'literal]))
+
+(test "alt-elli3" 'literal (lambda () (alt-elli3 x ...)))
+
 ;;----------------------------------------------------------------------
 ;; cond, taken from R5RS section 7.3
 
@@ -926,6 +975,85 @@
         (gen-idef-5 ooo? eee?)
         (list (ooo? 5) (eee? 7))))
 
+;; crazy case when define is redefined
+(define-module mac-idef
+  (export (rename my-define define))
+  (define (my-define . args) args))
+
+(define-module mac-idef.user
+  (import mac-idef))
+
+(test "define (redefined)" '(5 2)
+      (lambda ()
+        (with-module mac-idef.user
+          (let ((a 5)) (define a 2)))))
+
+(define-module mac-idef2
+  (export (rename my-define define))
+  (define-syntax my-define
+    (syntax-rules ()
+      [(_ var expr) (define (var) expr)])))
+
+(define-module mac-idef2.user
+  (import mac-idef2))
+
+(test "define (redefined2)" 5
+      (lambda ()
+        (with-module mac-idef2.user
+          (let ((a 5)) (define x a) (x)))))
+
+(test "internal define-syntax and scope 1" 'inner
+      (let ((x 'outer))
+        (lambda ()
+          (define x 'inner)
+          (define-syntax foo
+            (syntax-rules ()
+              [(_) x]))
+          (foo))))
+
+(test "internal define-syntax and scope 2" 'inner
+      (let ((x 'outer))
+        (lambda ()
+          (define-syntax foo
+            (syntax-rules ()
+              [(_) x]))
+          (define x 'inner)
+          (foo))))
+
+(test "internal define-syntax and scope 3" '(inner inner)
+      (let ((x 'outer))
+        (lambda ()
+          (define-syntax def
+            (syntax-rules ()
+              [(_ v) (define v x)]))
+          (define x 'inner)
+          (def y)
+          (list x y))))
+
+(test "internal define-syntax and scope 4" '(inner inner)
+      (let ((x 'outer))
+        (lambda ()
+          (define-syntax def
+            (syntax-rules ()
+              [(_ v) (define v (lambda () x))]))
+          (def y)
+          (define x 'inner)
+          (list x (y)))))
+
+(test "internal define-syntax and scope 5" '(inner (inner . innermost))
+      (let ((x 'outer))
+        (lambda ()
+          (define-syntax def1
+            (syntax-rules ()
+              [(_ v) (def2 v x)]))
+          (define-syntax def2
+            (syntax-rules ()
+              [(_ v y) (define v (let ((x 'innermost))
+                                   (lambda () (cons y x))))]))
+          (def1 z)
+          (define x 'inner)
+          (list x (z)))))
+
 ;;----------------------------------------------------------------------
 ;; macro defining macros
 
@@ -963,18 +1091,17 @@
                                     body)))))
           (mdm-foo3 list 3 (list 4)))))
 
-;; this doesn't work for now, due to the bug of macro expander
-'(test "let-syntax - let-syntax" 3
+(test "letrec-syntax - let-syntax" 3
       (lambda ()
-        (let-syntax ((mdm-foo4
-                      (syntax-rules ()
-                        ((mdm-foo4 () n) n)
-                        ((mdm-foo4 (x . xs) n)
-                         (let-syntax ((mdm-foo5
-                                       (syntax-rules ()
-                                         ((mdm-foo5)
-                                          (mdm-foo4 xs (+ n 1))))))
-                           (mdm-foo5))))))
+        (letrec-syntax ((mdm-foo4
+                         (syntax-rules ()
+                           ((mdm-foo4 () n) n)
+                           ((mdm-foo4 (x . xs) n)
+                            (let-syntax ((mdm-foo5
+                                          (syntax-rules ()
+                                            ((mdm-foo5)
+                                             (mdm-foo4 xs (+ n 1))))))
+                              (mdm-foo5))))))
           (mdm-foo4 (#f #f #f) 0))))
 
 (define-syntax mdm-foo3
@@ -1112,12 +1239,11 @@
 (test-section "identifier comparison")
 
 ;; This is EXPERIMENTAL: may be changed in later release.
-
-(define-syntax hoge (syntax-rules () ((hoge foo ...) (cdr b))))
+(define-syntax expand-id-compare (syntax-rules () ((hoge foo ...) (cdr b))))
 (test "comparison of identifiers" '(cdr b)
-      (lambda () (macroexpand '(hoge bar))))
-(test "comparison of identifiers" (macroexpand '(hoge bar))
-      (lambda () (macroexpand '(hoge bar))))
+      (lambda () (macroexpand '(expand-id-compare bar))))
+(test "comparison of identifiers" (macroexpand '(expand-id-compare bar))
+      (lambda () (macroexpand '(expand-id-compare bar))))
 
 ;;----------------------------------------------------------------------
 ;; keyword and extended lambda list
@@ -1338,6 +1464,20 @@
 (test "if-let1" 'bar
       (lambda () (if-let1 it (memq 'a '(b c d)) 'boo 'bar)))
 
+(test "let-values" '(2 1 1 (2) (2 1))
+      (lambda () (let ([a 1] [b 2])
+                   (let-values ([(a b) (values b a)]
+                                [(c . d) (values a b)]
+                                [e (values b a)])
+                     (list a b c d e)))))
+
+(test "let*-values" '(2 1 2 (1) (1 2))
+      (lambda () (let ([a 1] [b 2])
+                   (let*-values ([(a b) (values b a)]
+                                 [(c . d) (values a b)]
+                                 [e (values b a)])
+                     (list a b c d e)))))
+
 (test "ecase" 'b
       (lambda () (ecase 3 ((1) 'a) ((2 3) 'b) ((4) 'c))))
 (test "ecase" (test-error)
@@ -1405,4 +1545,233 @@
 (test "reject first-class macro usage" (test-error)
       (lambda () (bad-fi #t 'a 'b)))
 
+;;----------------------------------------------------------------------
+;; compiler macros
+
+(test-section "define-inline/syntax")
+
+(define-inline/syntax cpm
+  (lambda (a b) (+ a b))
+  (er-macro-transformer
+   (lambda (f r c) `(,(r '*) ,(cadr f) ,(caddr f)))))
+(test "compiler macro" '(6 5 6)
+      (lambda ()
+        (list (cpm 2 3)
+              (apply cpm '(2 3))
+              (let ((* -)) (cpm 2 3)))))
+
+;;----------------------------------------------------------------------
+;; syntax error
+
+(test-section "syntax-error")
+
+(define-syntax test-syntax-error
+  (syntax-rules ()
+    [(_ a) 'ok]
+    [(_ a b) (syntax-errorf "bad {~a ~a}" a b)]
+    [(_ x ...) (syntax-error "bad number of arguments" x ...)]))
+
+;; NB: These tests depends on the fact that the compile "wraps"
+;; the error by <compile-error-mixin> in order.  If the compiler changes
+;; the error handling, adjust the tests accordingly.
+;; Our purpose here is to make sure syntax-error preserves the offending macro
+;; call (test-syntax-error ...).
+(test "syntax-error"
+      '("bad number of arguments x y z"
+        (test-syntax-error x y z)
+        (list (test-syntax-error x y z)))
+      (lambda ()
+        (guard [e (else (let1 xs (filter <compile-error-mixin>
+                                         (slot-ref e '%conditions))
+                          (cons (slot-ref e 'message)
+                                (map (lambda (x) (slot-ref x 'expr)) xs))))]
+          (eval '(list (test-syntax-error x y z))
+                (interaction-environment)))))
+(test "syntax-errorf"
+      '("bad {x y}"
+        (test-syntax-error x y)
+        (list (test-syntax-error x y)))
+      (lambda ()
+        (guard [e (else (let1 xs (filter <compile-error-mixin>
+                                         (slot-ref e '%conditions))
+                          (cons (slot-ref e 'message)
+                                (map (lambda (x) (slot-ref x 'expr)) xs))))]
+          (eval '(list (test-syntax-error x y))
+                (interaction-environment)))))
+
+;;----------------------------------------------------------------------
+;; 'compare-ellipsis-1' test should output the following error.
+;;
+;; *** ERROR: in definition of macro mac-sub1:
+;; template's ellipsis nesting is deeper than pattern's:
+;; (#<identifier user#list.2d80660> #<identifier user#x.2d80690>
+;;  #<identifier user#ooo.2d806f0>)
+;;
+;; 'compare-ellipsis-2' test should output the following error.
+;;
+;; *** ERROR: in definition of macro mac-sub1:
+;; template's ellipsis nesting is deeper than pattern's:
+;; (#<identifier user#list.2969870> #<identifier user#x.29698a0>
+;;  #<identifier user#ooo.2969900>)
+
+(test-section "compare ellipsis")
+
+(define-syntax ell-test
+  (syntax-rules (ooo)
+    ((_ zzz)
+     (let-syntax
+         ((mac-sub1
+           (syntax-rules ooo ()
+             ((_ x zzz)
+              (list x ooo)))))
+       (mac-sub1 1 2 3)))))
+
+(test* "compare-ellipsis-1"
+       (test-error <error> #/^in definition of macro/)
+       (eval
+        '(ell-test ooo)
+        (interaction-environment)))
+
+(test* "compare-ellipsis-2"
+       (test-error <error> #/^in definition of macro/)
+       (eval
+        '(let ((ooo 'yyy)) (ell-test ooo))
+        (interaction-environment)))
+
+;;----------------------------------------------------------------------
+;; 'compare-literals-2' test should output the following error.
+;;
+;; *** ERROR: malformed #<identifier user#lit-test-2.29d4060>:
+;; (#<identifier user#lit-test-2.29d4060> #<identifier user#temp.29d40c0>)
+;; While compiling: (lit-test-2 temp 1)
+
+(test-section "compare literals")
+
+(define-syntax lit-test-1
+  (syntax-rules (temp)
+    ((_ temp x)
+     (lit-test-1 temp))
+    ((_ temp)
+     'passed)))
+
+(test* "compare-literals-1" 'passed (lit-test-1 temp 1))
+
+(define-syntax lit-test-2
+  (syntax-rules (temp)
+    ((_ temp x)
+     (let ((temp 100))
+       (lit-test-2 temp)))
+    ((_ temp)
+     'failed)))
+
+(test* "compare-literals-2"
+       (test-error <error> #/^malformed/)
+       (eval '(lit-test-2 temp 1) (interaction-environment)))
+
+;;----------------------------------------------------------------------
+;; 'generate-underbar-1' test should output the following error.
+;;
+;; *** ERROR: unbound variable: #<identifier user#_.2d4ac00>
+
+(test-section "generate underbar")
+
+(define-syntax gen-underbar
+  (syntax-rules (_)
+    ((gen-underbar)
+     (let-syntax
+         ((mac-sub1
+           (syntax-rules ()
+             ((mac-sub1 _)
+              _))))
+       (mac-sub1 'failed)))))
+
+(test* "generate-underbar-1"
+       (test-error <error> #/^unbound variable/)
+       (gen-underbar))
+
+;;----------------------------------------------------------------------
+;; 'pattern-variables-1' test should output the following error.
+;;
+;; *** ERROR: too many pattern variables in the macro definition of pat-vars
+;; While compiling: (syntax-rules () ((_ (z1 (x1 x2 x3 x4 x5 x6 x7 x8 x9 x10
+;; x11 x12 x13 x14 x15 x16 x17 x ...
+;; While compiling: (define-syntax pat-vars (syntax-rules () ((_ (z1 (x1 x2 x3
+;; x4 x5 x6 x7 x8 x9 x10 x11 x ...
+
+(test-section "pattern variables check")
+
+(test* "pattern-variables-1"
+       (test-error <error> #/^Too many pattern variables/)
+       (eval
+        '(define-syntax pat-vars
+           (syntax-rules ()
+             ((_ (z1 (x1 x2 x3 x4 x5 x6 x7 x8 x9 x10
+                      x11 x12 x13 x14 x15 x16 x17 x18 x19 x20
+                      x21 x22 x23 x24 x25 x26 x27 x28 x29 x30
+                      x31 x32 x33 x34 x35 x36 x37 x38 x39 x40
+                      x41 x42 x43 x44 x45 x46 x47 x48 x49 x50
+                      x51 x52 x53 x54 x55 x56 x57 x58 x59 x60
+                      x61 x62 x63 x64 x65 x66 x67 x68 x69 x70
+                      x71 x72 x73 x74 x75 x76 x77 x78 x79 x80
+                      x81 x82 x83 x84 x85 x86 x87 x88 x89 x90
+                      x91 x92 x93 x94 x95 x96 x97 x98 x99 x100
+                      x101 x102 x103 x104 x105 x106 x107 x108 x109 x110
+                      x111 x112 x113 x114 x115 x116 x117 x118 x119 x120
+                      x121 x122 x123 x124 x125 x126 x127 x128 x129 x130
+                      x131 x132 x133 x134 x135 x136 x137 x138 x139 x140
+                      x141 x142 x143 x144 x145 x146 x147 x148 x149 x150
+                      x151 x152 x153 x154 x155 x156 x157 x158 x159 x160
+                      x161 x162 x163 x164 x165 x166 x167 x168 x169 x170
+                      x171 x172 x173 x174 x175 x176 x177 x178 x179 x180
+                      x181 x182 x183 x184 x185 x186 x187 x188 x189 x190
+                      x191 x192 x193 x194 x195 x196 x197 x198 x199 x200
+                      x201 x202 x203 x204 x205 x206 x207 x208 x209 x210
+                      x211 x212 x213 x214 x215 x216 x217 x218 x219 x220
+                      x221 x222 x223 x224 x225 x226 x227 x228 x229 x230
+                      x231 x232 x233 x234 x235 x236 x237 x238 x239 x240
+                      x241 x242 x243 x244 x245 x246 x247 x248 x249 x250
+                      x251 x252 x253 x254 x255 x256)))
+              (print z1 " " x255 " " x256))))
+        (interaction-environment)))
+
+(test* "pattern-variables-2"
+       (test-error <error> #/^Pattern levels too deeply nested/)
+       (let ()
+         (define (build-deep-nested-pattern n f)
+           (if (= n 0)
+             `(define-syntax pat-vars
+                (syntax-rules ()
+                  ((_ ,f)
+                   (quote ,f))))
+             (build-deep-nested-pattern (- n 1) `(,f ...))))
+         (eval
+          (build-deep-nested-pattern 256 'x)
+          (interaction-environment))))
+
+;;----------------------------------------------------------------------
+;; srfi-147 begin
+;; (not yest supported)
+
+'(test-section "srfi-147 begin")
+
+'(test "srfi-147 begin (internal) 1"
+      '(yes no)
+      (lambda ()
+        (define-syntax foo
+          (begin (define-syntax bar if)
+                 (syntax-rules ()
+                   [(_ x y z) (bar z x y)])))
+        (list (foo 'yes 'no (zero? 0))
+              (foo 'yes 'no (zero? 1)))))
+
+'(test "srfi-147 begin (internal) 2"
+      11
+      (lambda ()
+        (let-syntax ([foo (syntax-rules ()
+                            [(_ a) (begin (define x (* a 2))
+                                          (syntax-rules ()
+                                            [(_ b) (+ b x)]))])])
+          (define-syntax bar (foo 3))
+          (bar 5))))
+      
 (test-end)
