@@ -3,6 +3,7 @@
 (use gauche.test)
 (use gauche.vm.insn)
 (use srfi-1)
+(use gauche.uvector)
 (use util.match)
 
 (define (proc->insn/split proc)
@@ -41,6 +42,25 @@
 (test* "inlining add4 + constant folding" '(((CONSTI 9)) ((RET)))
        (proc->insn/split (^[] (+ (add4 2) 3))))
 
+;; pass1 setter inlining
+(test* "inlining setter" '(((LREF1-PUSH)) ((LREF0)) ((VEC-SETI 0)) ((RET)))
+       (proc->insn/split (^[a b] (set! (vector-ref a 0) b))))
+(test* "inlining setter" #t
+       (let1 xs (proc->insn/split (^[a b] (set! (car a) b)))
+         (or (any (^x (equal? (car x) '(GREF-TAIL-CALL 2))) xs)
+             xs))) ;return xs in case of failure for easier diag
+
+;; pass3/late-inline
+(define-inline (late-inline-test-1 ref) (cut ref <> 0))
+
+(test* "pass3/late-inline"
+       '(((LREF0)) ((VEC-REFI 0)) ((RET)))
+       (proc->insn/split (late-inline-test-1 vector-ref)))
+
+(test* "pass3/late-inline"
+       '(((LREF0-PUSH)) ((CONSTI 0)) ((UVEC-REF 1)) ((RET)))
+       (proc->insn/split (late-inline-test-1 u8vector-ref)))
+
 (test-section "lambda lifting")
 
 ;; bug reported by teppey
@@ -138,5 +158,6 @@
            (test (bar)))
          (foo)))
 
+
 (test-end)
 
