@@ -60,6 +60,10 @@
   (cond-expand [gauche.os.windows `(".\\gosh" "-ftest" ".\\testc.o" ,@args)]
                [else args]))
 
+(define (cmd-in-subdir . args)
+  (cond-expand [gauche.os.windows `("..\\gosh" "-ftest" "..\\testc.o" ,@args)]
+               [else args]))
+
 (define (cmds . args)
   (let1 cmdlist (apply cmd args)
     (string-concatenate (apply append (map (^x `(,x " ")) cmdlist)))))
@@ -265,26 +269,36 @@
     (^[] (for-each print '("banana" "habana" "tabata" "cabara"))))
 
   (test* "pipelining 1" "banana\nhabana\n"
-         (let1 ps (run-process-pipeline `(,(cmd cat "test.o")
-                                          ,(cmd grep "bana"))
-                                        :output :pipe)
-           (begin0 (port->string (process-output (last ps)))
-             (for-each process-wait ps))))
+         (let1 p (run-pipeline `(,(cmd cat "test.o")
+                                 ,(cmd grep "bana"))
+                               :output :pipe)
+           (process-wait p)
+           (port->string (process-output p))))
 
   (test* "pipelining 2" "tabata\ncabara\n"
-         (let1 ps (run-process-pipeline `(,(cmd cat)
-                                          ,(cmd grep "-v" "bana"))
-                                        :input "test.o" :output :pipe)
-           (begin0 (port->string (process-output (last ps)))
-             (for-each process-wait ps))))
+         (let1 p (run-pipeline `(,(cmd cat)
+                                 ,(cmd grep "-v" "bana"))
+                               :input "test.o" :output :pipe)
+           (process-wait p)
+           (port->string (process-output p))))
 
   (test* "pipelining 3" "banana\ncabara\n"
-         (let1 ps (run-process-pipeline `(,(cmd cat)
-                                          ,(cmd grep "-v" "ta")
-                                          ,(cmd grep "-v" "ha"))
-                                        :input "test.o" :output :pipe)
-           (begin0 (port->string (process-output (last ps)))
-             (for-each process-wait ps))))  
+         (let1 p (run-pipeline `(,(cmd cat)
+                                 ,(cmd grep "-v" "ta")
+                                 ,(cmd grep "-v" "ha"))
+                               :input "test.o" :output :pipe)
+           (process-wait p)
+           (port->string (process-output p))))
+
+  (test* "pipelining 4" "habana\ntabata\ncabara\n"
+         (let1 p (run-pipeline `(,(cmd cat)
+                                 ,(cmd grep "aba"))
+                               :input :pipe :output :pipe)
+           (display "banana\nhabana\ntabata\ncabara\n"
+                    (process-input p))
+           (close-port (process-input p))
+           (process-wait p)
+           (port->string (process-output p))))
   )
 
 ;;-------------------------------
@@ -522,6 +536,16 @@
              (s (call-with-input-file "test.o" port->string-list)))
          (equal? r s)))
 
+(rmrf "test2.o")
+(sys-mkdir "test2.o" #o755)
+(with-output-to-file "test2.o/probe"
+  (^[] (display "Aloha!")))
+(test* "process-output->string (different directory)" "Aloha!"
+       (process-output->string (cmd-in-subdir cat "probe")
+                               :directory "test2.o"))
+
+(rmrf "test2.o")
+
 (rmrf "testc.o")
 
 (let ()
@@ -563,6 +587,10 @@
   (t "'ab c d' ef \"g h\"" '("ab c d" "ef" "g h")      'posix)
   (t "'ab\\\\cd' \"ab\\\\cd\"" '("ab\\\\cd" "ab\\cd")  'posix)
   (t "'ab\\xcd' \"ab\\xcd\"" '("ab\\xcd" "ab\\xcd")    'posix)
+  (t "a\"\"b"              '("ab")                     'posix)
+  (t "a''b"                '("ab")                     'posix)
+  (t "a'\"'b"              '("a\"b")                   'posix)
+  (t "a\"'\"b"             '("a'b")                    'posix)
 
   (t "$abc"                (test-error)                'posix)
   (t "\"$abc\""            (test-error)                'posix)
