@@ -9,10 +9,34 @@
 
 (test-start "utility scripts")
 
-(define *nulldev*
+(define *top-srcdir*
+  (sys-normalize-pathname (or (sys-getenv "top_srcdir") "..")
+                          :absolute #t :canonicalize #t))
+(define *top-builddir*
+  (sys-normalize-pathname (or (sys-getenv "top_builddir") "..")
+                          :absolute #t :canonicalize #t))
+
+(define *executable-suffix* (gauche-config "--executable-suffix"))
+
+(define (fix-path path)
   (cond-expand
-   [gauche.os.windows "NUL"]
-   [else "/dev/null"]))
+   [gauche.os.windows (regexp-replace-all #/\\/ path "/")]
+   [else              path]))
+
+(define (wrap-with-test-directory thunk :optional (mkdir? #t))
+  (remove-files "test.o" "test1.o")
+  (when mkdir?
+    (make-directory* "test.o")
+    (make-directory* "test1.o"))
+  (unwind-protect (thunk)
+    ;; cleanup may fail due to the timing (exp. on mingw), so we retry
+    ;; with delay.
+    (let loop ([retry 0])
+      (unless (>= retry 3)
+        (or (guard (e [<system-error> #f])
+              (remove-files "test.o" "test1.o")
+              #t)
+            (begin (sys-sleep 1) (loop (+ retry 1))))))))
 
 ;;=======================================================================
 (test-section "gosh")
@@ -58,6 +82,45 @@
              (process-output->string '("./gosh" "-ftest" "test.o")))
          (delete-files "test.o")))
 
+(test* "r7rs + srfi-22 by interpreter name" "baz bar"
+       (unwind-protect
+           (begin
+             (delete-files "test.o" "scheme-r7rs")
+             (copy-file #"./gosh~*executable-suffix*" "./scheme-r7rs")
+             (sys-chmod "./scheme-r7rs" #o755)
+             (with-output-to-file "test.o"
+               (^[]
+                 (write
+                  '(import (scheme base) (scheme write)))
+                 (write
+                  '(define (main args) (display "bar\n") 0))
+                 (write
+                  '(display "baz\n"))))
+             (process-output->string '("./scheme-r7rs" "-ftest" "test.o")
+                                     :on-abnormal-exit :ignore))
+         (delete-files "test.o" "scheme-r7rs")))
+
+;; This caused assertion failure in 0.9.5, because 'main' was called
+;; via Scm_ApplyRec without base VM running.
+;; See https://github.com/shirok/Gauche/issues/244
+(test* "proper error handling of 'main'" "ok"
+       (unwind-protect
+           (let1 gauche.h (if-let1 top (sys-getenv "top_srcdir")
+                            (build-path top "src/gauche.h")
+                            "gauche.h")
+             (delete-files "test.o")
+             (with-output-to-file "test.o"
+               (^[]
+                 (write
+                  '(use gauche.partcont))
+                 (write
+                  `(define (main args)
+                     (reset (shift k (call-with-input-file ,gauche.h k)))
+                     (print 'ok)
+                     0))))
+             (process-output->string '("./gosh" "-ftest" "test.o")))
+         (delete-files "test.o")))
+
 ;;=======================================================================
 (test-section "gauche-config")
 
@@ -142,7 +205,7 @@
        (process-exit-status
         (run-with-parent-directory-in-paths
          `("../gosh" "-ftest" "./configure")
-         :output *nulldev* :wait #t :directory "test.o")))
+         :output :null :wait #t :directory "test.o")))
 (test* "Makefile substitution" '()
        (and (file-exists? "test.o/Makefile")
             (filter #/@\w+@/ (file->string-list "test.o/Makefile"))))
@@ -164,7 +227,7 @@
        (process-exit-status
         (run-with-parent-directory-in-paths
          `("../gosh" "-ftest" "../test.o/configure")
-         :output *nulldev* :wait #t :directory "test2.o")))
+         :output :null :wait #t :directory "test2.o")))
 
 (test* "Makefiles in proper builddir" '(#t #t)
        (list (file-exists? "test2.o/Makefile")
@@ -195,7 +258,7 @@
               (run-with-parent-directory-in-paths
                `("../gosh" "-ftest" "../test.o/configure"
                  "--with-local=/a/b:/c/d")
-               :output *nulldev* :wait #t :directory "test2.o")))
+               :output :null :wait #t :directory "test2.o")))
             (cadr (memv :configure
                         (car (file->sexp-list "test2.o/package.gpd"))))))
 
@@ -213,85 +276,82 @@
 
 (define (run-install . args)
   (run-process `("./gosh" "-ftest" ,*gauche-install-script* ,@args)
-               :output *nulldev* :wait #t))
+               :output :null :wait #t))
 
-(remove-files "test.o" "test1.o")
+(define (test-install)
+  (test* "-d" #t
+         (begin (run-install "-d" "test1.o/dest")
+                (file-is-directory? "test1.o/dest")))
 
-(test* "-d" #t
-       (begin (run-install "-d" "test1.o/dest")
-              (file-is-directory? "test1.o/dest")))
+  (create-directory-tree "."
+                         `(test.o ((bin ((command1 ,(make-string 20000))
+                                         (command2 ,(make-string 20000))))
+                                   (lib ((lib1 ,(make-string 1000000))
+                                         (lib2 ,(make-string 500000))))
+                                   (etc ((conf1 ,(make-string 1000)))))))
+  (test* "file -> file" #t
+         (begin (run-install "-m" "444" "test.o/lib/lib1" "test1.o/dest/lib1")
+                (and (file-is-regular? "test1.o/dest/lib1")
+                     (= (file-perm "test1.o/dest/lib1") #o444)
+                     (file-equal? "test.o/lib/lib1" "test1.o/dest/lib1"))))
 
-(create-directory-tree "."
-                       `(test.o ((bin ((command1 ,(make-string 20000))
-                                       (command2 ,(make-string 20000))))
-                                 (lib ((lib1 ,(make-string 1000000))
-                                       (lib2 ,(make-string 500000))))
-                                 (etc ((conf1 ,(make-string 1000)))))))
+  (remove-files "test1.o/dest/lib1")
+  (test* "files -> dir" #t
+         (begin (run-install "-m" "444" "test.o/lib/lib1" "test.o/lib/lib2"
+                             "test1.o/dest")
+                (and (file-is-regular? "test1.o/dest/lib1")
+                     (file-is-regular? "test1.o/dest/lib2")
+                     (= (file-perm "test1.o/dest/lib1")
+                        (file-perm "test1.o/dest/lib1")
+                        #o444)
+                     (file-equal? "test.o/lib/lib1" "test1.o/dest/lib1")
+                     (file-equal? "test.o/lib/lib2" "test1.o/dest/lib2"))))
 
-(test* "file -> file" #t
-       (begin (run-install "-m" "444" "test.o/lib/lib1" "test1.o/dest/lib1")
-              (and (file-is-regular? "test1.o/dest/lib1")
-                   (= (file-perm "test1.o/dest/lib1") #o444)
-                   (file-equal? "test.o/lib/lib1" "test1.o/dest/lib1"))))
+  (remove-files "test1.o/dest/lib1" "test1.o/dest/lib2")
+  (test* "-T" #t
+         (begin (run-install "-T" "test1.o/dest" "-m" "555"
+                             "test.o/bin/command1"
+                             "test.o/bin/command2")
+                (and (= (file-perm "test1.o/dest/test.o/bin/command1")
+                        (file-perm "test1.o/dest/test.o/bin/command2")
+                        (cond-expand
+                         [gauche.os.windows #o444]
+                         [else #o555]))
+                     (file-equal? "test.o/bin/command1"
+                                  "test1.o/dest/test.o/bin/command1")
+                     (file-equal? "test.o/bin/command2"
+                                  "test1.o/dest/test.o/bin/command2"))))
 
-(remove-files "test1.o/dest/lib1")
+  (test* "-U" #t
+         (begin (run-install "-U" "test1.o/dest" "-m" "555"
+                             "test.o/bin/command1"
+                             "test.o/bin/command2")
+                (and (not (file-exists? "test1.o/dest/test.o/bin/command1"))
+                     (not (file-exists? "test1.o/dest/test.o/bin/command2")))))
 
-(test* "files -> dir" #t
-       (begin (run-install "-m" "444" "test.o/lib/lib1" "test.o/lib/lib2"
-                           "test1.o/dest")
-              (and (file-is-regular? "test1.o/dest/lib1")
-                   (file-is-regular? "test1.o/dest/lib2")
-                   (= (file-perm "test1.o/dest/lib1")
-                      (file-perm "test1.o/dest/lib1")
-                      #o444)
-                   (file-equal? "test.o/lib/lib1" "test1.o/dest/lib1")
-                   (file-equal? "test.o/lib/lib2" "test1.o/dest/lib2"))))
+  (test* "-T -p" #t
+         (begin (run-install "-T" "test1.o/dest" "-m" "555" "-p" "test.o"
+                             "test.o/bin/command1"
+                             "test.o/bin/command2")
+                (and (= (file-perm "test1.o/dest/bin/command1")
+                        (file-perm "test1.o/dest/bin/command2")
+                        (cond-expand
+                         [gauche.os.windows #o444]
+                         [else #o555]))
+                     (file-equal? "test.o/bin/command1"
+                                  "test1.o/dest/bin/command1")
+                     (file-equal? "test.o/bin/command2"
+                                  "test1.o/dest/bin/command2"))))
 
-(remove-files "test1.o/dest/lib1" "test1.o/dest/lib2")
+  (test* "-U -p" #t
+         (begin (run-install "-U" "test1.o/dest" "-m" "555" "-p" "test.o"
+                             "test.o/bin/command1"
+                             "test.o/bin/command2")
+                (and (not (file-exists? "test1.o/dest/bin/command1"))
+                     (not (file-exists? "test1.o/dest/bin/command2")))))
+  )
 
-(test* "-T" #t
-       (begin (run-install "-T" "test1.o/dest" "-m" "555"
-                           "test.o/bin/command1"
-                           "test.o/bin/command2")
-              (and (= (file-perm "test1.o/dest/test.o/bin/command1")
-                      (file-perm "test1.o/dest/test.o/bin/command2")
-                      (cond-expand
-                       [gauche.os.windows #o444]
-                       [else #o555]))
-                   (file-equal? "test.o/bin/command1"
-                                "test1.o/dest/test.o/bin/command1")
-                   (file-equal? "test.o/bin/command2"
-                                "test1.o/dest/test.o/bin/command2"))))
-
-(test* "-U" #t
-       (begin (run-install "-U" "test1.o/dest" "-m" "555"
-                           "test.o/bin/command1"
-                           "test.o/bin/command2")
-              (and (not (file-exists? "test1.o/dest/test.o/bin/command1"))
-                   (not (file-exists? "test1.o/dest/test.o/bin/command2")))))
-
-(test* "-T -p" #t
-       (begin (run-install "-T" "test1.o/dest" "-m" "555" "-p" "test.o"
-                           "test.o/bin/command1"
-                           "test.o/bin/command2")
-              (and (= (file-perm "test1.o/dest/bin/command1")
-                      (file-perm "test1.o/dest/bin/command2")
-                      (cond-expand
-                       [gauche.os.windows #o444]
-                       [else #o555]))
-                   (file-equal? "test.o/bin/command1"
-                                "test1.o/dest/bin/command1")
-                   (file-equal? "test.o/bin/command2"
-                                "test1.o/dest/bin/command2"))))
-
-(test* "-U -p" #t
-       (begin (run-install "-U" "test1.o/dest" "-m" "555" "-p" "test.o"
-                           "test.o/bin/command1"
-                           "test.o/bin/command2")
-              (and (not (file-exists? "test1.o/dest/bin/command1"))
-                   (not (file-exists? "test1.o/dest/bin/command2")))))
-
-(remove-files "test.o" "test1.o")
+(wrap-with-test-directory test-install #f)
 
 ;;=======================================================================
 (test-section "gauche-package")
@@ -299,17 +359,11 @@
 (test-script
  (build-path (or (sys-getenv "top_srcdir") "..") "src" "gauche-package.in"))
 
-(remove-files "test.o")
-(make-directory* "test.o")
-
 (define (package-generate-tests)
   (define (file-check name)
     (test* #"checking existence of ~name" #t
            (file-exists? #"test.o/Test/~name")))
   (define pwd (sys-getcwd))
-  (define top-srcdir (sys-normalize-pathname
-                      (or (sys-getenv "top_srcdir") "..")
-                      :absolute #t :canonicalize #t))
 
   ;; When we run the test before installation, we need to tweak
   ;; some directories.  We can distinguish it, for install-check
@@ -336,7 +390,7 @@
              (define prefix (if (equal? (sys-basename (sys-getcwd)) "Test")
                               "../" ""))
              (define sep (cond-expand [gauche.os.windows ";"][else ":"]))
-             (define top-srcdir ,top-srcdir)
+             (define top-srcdir ,*top-srcdir*)
              ;; This is used to copy templates from.
              (define (gauche-library-directory) #"~|top-srcdir|/ext/x")
              ;; Intercept gauche-config to override compiler flags
@@ -371,7 +425,103 @@
            (or (zero? (process-exit-status p)) o)))
   )
 
-(unwind-protect (package-generate-tests)
-  (remove-directory* "test.o"))
+(wrap-with-test-directory package-generate-tests)
+
+;;=======================================================================
+(test-section "precomp")
+
+(define (precomp-test-1)
+  (do-process `("../../src/gosh" "-ftest"
+                ,#"-I~|*top-srcdir*|/test/test-precomp"
+                ,(build-path *top-srcdir* "src/precomp") "--strip-prefix"
+                ,(fix-path (build-path *top-srcdir* "test/test-precomp"))
+                ,@(map (^[file] (fix-path (build-path *top-srcdir* "test/test-precomp" file)))
+                       '("foo.scm" "foo/bar1.scm" "foo/bar2.scm" "foo/bar3.scm")))
+              :directory "test.o")
+  (test* "precomp generated files"
+         '("test.o/foo--bar1.c"
+           "test.o/foo--bar2.c"
+           "test.o/foo--bar3.c"
+           "test.o/foo.c"
+           "test.o/foo.sci"
+           "test.o/foo/bar1.sci"
+           "test.o/foo/bar2.sci"
+           "test.o/foo/bar3.sci")
+         (sort (map fix-path (directory-fold "test.o" cons '()))))
+  )
+
+(define (precomp-test-2)
+  (do-process `("../../src/gosh" "-ftest"
+                ,#"-I~|*top-srcdir*|/test/test-precomp"
+                ,(build-path *top-srcdir* "src/precomp") "--strip-prefix"
+                ,(fix-path (build-path *top-srcdir* "test/test-precomp"))
+                "--single-interface"
+                ,@(map (^[file] (fix-path (build-path *top-srcdir* "test/test-precomp" file)))
+                       '("foo.scm" "foo/bar1.scm" "foo/bar2.scm" "foo/bar3.scm")))
+              :directory "test.o")
+  (test* "precomp generated files with --single-interface"
+         '("test.o/foo--bar1.c"
+           "test.o/foo--bar2.c"
+           "test.o/foo--bar3.c"
+           "test.o/foo.c"
+           "test.o/foo.sci")
+         (sort (map fix-path (directory-fold "test.o" cons '()))))
+
+  (test* "consolidated sci file"
+         '(";; generated automatically.  DO NOT EDIT"
+           "#!no-fold-case"
+           "(define-module foo.bar2 (use util.match) (export bar2))"
+           "(select-module foo.bar2)"
+           "(dynamic-load \"foo\" :init-function \"Scm_Init_foo__bar2\")"
+           "(provide \"foo/bar2\")"
+           "(define-module foo.bar3 (use foo.bar2) (export bar3))"
+           "(select-module foo.bar3)"
+           "(dynamic-load \"foo\" :init-function \"Scm_Init_foo__bar3\")"
+           "(provide \"foo/bar3\")"
+           "(define-module foo.bar1 (use foo.bar2) (export bar1))"
+           "(select-module foo.bar1)"
+           "(dynamic-load \"foo\" :init-function \"Scm_Init_foo__bar1\")"
+           "(provide \"foo/bar1\")"
+           "(define-module foo (use foo.bar1) (use foo.bar3) (export foo-master))"
+           "(select-module foo)"
+           "(dynamic-load \"foo\" :init-function \"Scm_Init_foo\")")
+         (file->string-list "test.o/foo.sci"))
+  )
+
+(wrap-with-test-directory precomp-test-1)
+(wrap-with-test-directory precomp-test-2)
+
+;;=======================================================================
+(test-section "build-standalone")
+
+(test-script
+ (build-path (or (sys-getenv "top_srcdir") "..") "src" "build-standalone"))
+
+(define (run-build-static opts args)
+  (do-process `("../../src/gosh" "-ftest"
+                ,#"-I=~|*top-srcdir*|/test/test-static"
+                ,(build-path *top-srcdir* "src/build-standalone")
+                ,#"--header-dir=~|*top-srcdir*|/src"
+                ,#"--header-dir=~|*top-builddir*|/src"
+                ,#"--header-dir=~|*top-srcdir*|/gc/include"
+                ,#"--library-dir=~|*top-builddir*|/src"
+                ,@opts
+                ,@args)
+              :directory "test.o"))
+
+(define (static-test-1)
+  (test* "static link test" #t
+         (run-build-static `("-o" "staticmain" 
+                             "-I" ,(build-path *top-srcdir* "test/test-static"))
+                           `(,(fix-path (build-path *top-srcdir* 
+                                                    "test/test-static/main.scm"))
+                             "foo/bar.scm" 
+                             "foo/bar-impl.scm"
+                             "foo/baz.scm")))
+  (test* "static link executable" "ARGS: (A b CdE)"
+         (process-output->string '(test.o/staticmain A b CdE)))
+  )
+
+(wrap-with-test-directory static-test-1)
 
 (test-end)
