@@ -23,6 +23,31 @@
 (test* "immutable" #f (string-immutable? (string #\a #\b)))
 (test* "immutable" #f (string-immutable? (string)))
 
+(let ([data '(("abc" "abc" =)
+              (""    "abc" <)
+              ("abd" "abc" >)
+              (#*"abc" #*"abc" =)
+              (#*"a"   #*"abc" <)
+              (#*"abd" #*"abc" >)
+              ("abc" #*"abc" <)
+              ("abc" #*"abb" >)
+              (#*"abc" "abc" >)
+              (#*"ab"  "abc" <))])
+  (define (t a b result)
+    (test* (string-append "compare " a " and " b)
+           (list (boolean (memq result '(=)))
+                 (boolean (memq result '(<)))
+                 (boolean (memq result '(< =)))
+                 (boolean (memq result '(>)))
+                 (boolean (memq result '(> =))))
+           (list (string=? a b)
+                 (string<? a b)
+                 (string<=? a b)
+                 (string>? a b)
+                 (string>=? a b))))
+  (dolist [x data]
+    (apply t x)))
+
 (test* "string->list" '(#\a #\b #\c #\d #\e #\f #\g)
        (string->list "abcdefg"))
 (test* "string->list" '(#\c #\d #\e #\f #\g)
@@ -182,7 +207,7 @@
        (string-split "aa*bbb*c**" #\*))
 (test* "string-split (char)" '("aa")
        (string-split "aa" #\*))
-(test* "string-split (char)" '("")
+(test* "string-split (char)" '()
        (string-split "" #\*))
 (test* "string-split (char)" '("" "")
        (string-split "*" #\*))
@@ -196,7 +221,7 @@
        (string-split "aa**bbb**c*c" "--"))
 (test* "string-split (string)" '("aa" "bbb" "c*c" "")
        (string-split "aa**bbb**c*c**" "**"))
-(test* "string-split (string)" '("")
+(test* "string-split (string)" '()
        (string-split "" "**"))
 (test* "string-split (string)" '("" "")
        (string-split "**" "**"))
@@ -211,14 +236,14 @@
        (string-split "--aa--bbb---c-c-" #/\w+/))
 (test* "string-split (regexp)" '("--aa--bbb---c-c-")
        (string-split "--aa--bbb---c-c-" #/z+/))
-(test* "string-split (regexp)" (test-error) ;; test detection of infinite loop
+(test* "string-split (regexp)" '("" "a" "a" "" "b" "b" "b" "" "c" "" "c" "" "")
        (string-split "--aa--bbb---c-c-" #/-*/))
 
 (test* "string-split (charset)" '("aa" "bbb" "c" "d")
        (string-split "aa---bbb***c&d" #[\W]))
 (test* "string-split (charset)" '("" "---" "***" "&" "")
        (string-split "aa---bbb***c&d" #[\w]))
-(test* "string-split (charset)" '("")
+(test* "string-split (charset)" '()
        (string-split "" #[\w]))
 (test* "string-split (charset)" '("" "")
        (string-split "a" #[\w]))
@@ -237,7 +262,7 @@
 
 (test* "string-split (char)" '("aa")
        (string-split "aa" #\* 1))
-(test* "string-split (char)" '("")
+(test* "string-split (char)" '()
        (string-split "" #\* 2))
 (test* "string-split (char)" '("" "")
        (string-split "*" #\* 3))
@@ -267,9 +292,9 @@
        (string-split "aa**bbb**c*c**" "**" 2))
 (test* "string-split (string)" '("aa" "bbb" "c*c" "")
        (string-split "aa**bbb**c*c**" "**" 3))
-(test* "string-split (string)" '("")
+(test* "string-split (string)" '()
        (string-split "" "**" 0))
-(test* "string-split (string)" '("")
+(test* "string-split (string)" '()
        (string-split "" "**" 1))
 (test* "string-split (string)" '("**")
        (string-split "**" "**" 0))
@@ -314,8 +339,8 @@
        (string-split "--aa--bbb---c-c-" #/\w+/ 3))
 (test* "string-split (regexp)" '("--aa--bbb---c-c-")
        (string-split "--aa--bbb---c-c-" #/z+/ 2))
-(test* "string-split (regexp)" (test-error) ;; test detection of infinite loop
-       (string-split "--aa--bbb---c-c-" #/-*/ 2))
+(test* "string-split (regexp)" '("" "a" "a" "--bbb---c-c-")
+       (string-split "--aa--bbb---c-c-" #/-*/ 3))
 (test* "string-split (bad limit)" (test-error)
        (string-split "--aa--bbb---c-c-" #/-+/ 'a))
 
