@@ -152,18 +152,22 @@
       (lambda () (string-incomplete->complete #*"あいう\xe3\xe3" :omit)))
 (test "string-incomplete->complete (omit)" "あいう"
       (lambda () (string-incomplete->complete #*"あいう\xe3\x80" :omit)))
-(test "string-incomplete->complete (replace)" "あいふう"
-      (lambda () (string-incomplete->complete #*"あい\x80う" #\ふ)))
+(test "string-incomplete->complete (replace)" "あい?う"
+      (lambda () (string-incomplete->complete #*"あい\x80う" :replace)))
 (test "string-incomplete->complete (replace)" "ふあいう"
-      (lambda () (string-incomplete->complete #*"\x80あいう" #\ふ)))
+      (lambda () (string-incomplete->complete #*"\x80あいう" :replace #\ふ)))
 (test "string-incomplete->complete (replace)" "ふふあいう"
-      (lambda () (string-incomplete->complete #*"\x80\xe3あいう" #\ふ)))
+      (lambda () (string-incomplete->complete #*"\x80\xe3あいう" :replace #\ふ)))
 (test "string-incomplete->complete (replace)" "あいうふ"
-      (lambda () (string-incomplete->complete #*"あいう\xe3" #\ふ)))
+      (lambda () (string-incomplete->complete #*"あいう\xe3" :replace #\ふ)))
 (test "string-incomplete->complete (replace)" "あいうふふ"
-      (lambda () (string-incomplete->complete #*"あいう\xe3\xe3" #\ふ)))
+      (lambda () (string-incomplete->complete #*"あいう\xe3\xe3" :replace #\ふ)))
 (test "string-incomplete->complete (replace)" "あいうふふ"
-      (lambda () (string-incomplete->complete #*"あいう\xe3\x80" #\ふ)))
+      (lambda () (string-incomplete->complete #*"あいう\xe3\x80" :replace #\ふ)))
+(test "string-incomplete->complete (escape)" "あ??い?91う?80え"
+      (lambda () (string-incomplete->complete #*"あ?い\x91う\x80え" :escape)))
+(test "string-incomplete->complete (escape)" "あ__い_91う_80え"
+      (lambda () (string-incomplete->complete #*"あ_い\x91う\x80え" :escape #\_)))
 
 (test "string=?" #t (lambda () (string=? #*"あいう" #*"あいう")))
 
@@ -244,20 +248,69 @@
 
 (use srfi-14)
 
-(test "char-set" #t
-      (lambda () (char-set= (char-set #\あ #\い #\う #\え #\お)
-                            (string->char-set "おうえいあ"))))
-(test "char-set" #t
-      (lambda () (char-set= (list->char-set '(#\あ #\い #\う #\ん))
-                            (string->char-set "んんいいいああう"))))
-(test "char-set" #t
-      (lambda () (char-set<= (list->char-set '(#\ほ #\げ))
-                             char-set:full)))
-(test "char-set" #t
-      (lambda ()
-        (char-set= (->char-set "ぁぃぅぇぉあいうえ")
-                   (integer-range->char-set (char->integer #\ぁ)
-                                            (char->integer #\お)))))
+(let ((data '(#[ぁ-ん他-唖ァ-ン]
+              #[aあ唖ア])))
+  (define (t x)
+    (test* "char-set mutable/immutable roundtrip" #t
+           (char-set= (char-set-copy x)
+                      (char-set-copy (char-set-freeze! (char-set-copy x))))))
+  (for-each t data))
+
+(test-char-set-1 "char-set-contains?" #t (cut char-set-contains? <> #\あ)
+                 (char-set #\あ #\い #\う #\え #\お))
+(test-char-set-1 "char-set-contains?" #f (cut char-set-contains? <> #\ぁ)
+                 (char-set #\あ #\い #\う #\え #\お))
+(test-char-set-1 "char-set-contains?" #t (cut char-set-contains? <> #\ほ)
+                 (char-set-union
+                  (integer-range->char-set (char->integer #\ぁ)
+                                           (char->integer #\ん))
+                  (integer-range->char-set (char->integer #\ァ)
+                                           (char->integer #\ン))))
+(test-char-set-1 "char-set-contains?" #t (cut char-set-contains? <> #\ケ)
+                 (char-set-union
+                  (integer-range->char-set (char->integer #\ぁ)
+                                           (char->integer #\ん))
+                  (integer-range->char-set (char->integer #\ァ)
+                                           (char->integer #\ン))))
+(test-char-set-1 "char-set-contains?" #f (cut char-set-contains? <> #\ん)
+                 (char-set-union
+                  (integer-range->char-set (char->integer #\ぁ)
+                                           (char->integer #\ん))
+                  (integer-range->char-set (char->integer #\ァ)
+                                           (char->integer #\ン))))
+(test-char-set-1 "char-set-contains?" #f (cut char-set-contains? <> #\ヴ)
+                 (char-set-union
+                  (integer-range->char-set (char->integer #\ぁ)
+                                           (char->integer #\ん))
+                  (integer-range->char-set (char->integer #\ァ)
+                                           (char->integer #\ン))))
+
+(test-char-set-2 "char-set=" #t char-set=
+                 (char-set #\あ #\い #\う #\え #\お)
+                 (string->char-set "おうえいあ"))
+(test-char-set-2 "char-set=" #t char-set=
+                 (list->char-set '(#\あ #\い #\う #\ん))
+                 (string->char-set "んんいいいああう"))
+(test-char-set-2 "char-set<=" #t char-set<=
+                 (list->char-set '(#\ほ #\げ))
+                 char-set:full)
+(test-char-set-2 "char-set<=" #t char-set<= #[あい] #[あい])
+(test-char-set-2 "char-set<=" #t char-set<= #[あい] #[あ-い])
+(test-char-set-2 "char-set<=" #f char-set<= #[あ-い] #[あい])
+(test-char-set-2 "char-set<=" #t char-set<= #[あ-いう-え] #[あ-え])
+(test-char-set-2 "char-set<=" #f char-set<= #[あ-え] #[あ-いう-え])
+(test-char-set-2 "char-set<=" #f char-set<= #[あ-いう-ぉ] #[あ-え])
+(test-char-set-2 "char-set<=" #f char-set<= #[ぁ-いう-え] #[あ-え])
+(test-char-set-2 "char-set<=" #t char-set<= #[あ-いか-き] #[あ-うお-け])
+(test-char-set-2 "char-set<=" #t char-set<= #[あ-いか-き] #[あ-うお-き])
+(test-char-set-2 "char-set<=" #t char-set<= #[あ-いか-き] #[あ-うか-こ])
+(test-char-set-2 "char-set<=" #t char-set<= #[か-き] #[あ-うか-こ])
+(test-char-set-2 "char-set<=" #t char-set<= #[か-きく-け] #[あ-うか-こ])
+(test-char-set-2 "char-set<=" #f char-set<= #[う-く] #[あ-えか-こ])
+(test-char-set-2 "->char-set" #t char-set=
+                 (->char-set "ぁぃぅぇぉあいうえ")
+                 (integer-range->char-set (char->integer #\ぁ)
+                                          (char->integer #\お)))
 
 ;;-------------------------------------------------------------------
 (test-section "ports")
